/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "delckidrevesa.h"

#include <qgl.h>
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

#define LEAF_SPIRAL		0
#define LEAF_STACKED	1
#define LEAF_STAGGERED	2
#define LEAF_BUNCHED	3
#define LEAF_CONIFEROUS	4

#define LEAF_WHOLE 12
#define BRANCH_WHOLE 6

using namespace drevo;

DelckiDrevesa::DelckiDrevesa( Pogled& pogled, SerializatorDelckovDrevesa &serializatorDelckovDrevesa )
		: pogled( pogled ), serializatorDelckovDrevesa( serializatorDelckovDrevesa ) {} //DelckiDrevesa::DelckiDrevesa

void DelckiDrevesa::narisiSteblo( double visina, double sirinaSpodaj, double sirinaZgoraj, Presek *presekSpodaj, int idVeje ) {
	/// serializiramo?
	if ( serializatorDelckovDrevesa.exportingToPoints )
		serializatorDelckovDrevesa.serializirajVejo( visina, sirinaSpodaj, sirinaZgoraj, presekSpodaj, idVeje );

	if ( !pogled.wireframe ) {
		/// steblo renderiramo kar s cilindrom, vendar ima ta spodnji in zgornji presek različno velik
		if ( pogled.lowQuality < BRANCH_WHOLE ) {
			glRotatef( -90, 1.0, 0.0, 0.0 );
			glBindTexture( GL_TEXTURE_2D, pogled.textureNames[ 0 ] );
			if ( pogled.highQualityRendering && pogled.lowQuality == 0 )
				gluCylinder( pogled.kvadraticnePloskve, sirinaSpodaj, sirinaZgoraj, visina, 10, 5 );
			else
				gluCylinder( pogled.kvadraticnePloskve, sirinaSpodaj, sirinaZgoraj, visina, 5, 1 );
		} else {
			glBindTexture( GL_TEXTURE_2D, pogled.textureNames[ 0 ] );
			glBegin( GL_LINES );
			{
				glTexCoord1d( 0 );
				glVertex3f( 0, 0, 0 );
				glTexCoord1d( 1 );
				glVertex3f( 0, visina, 0 );
			}
			glEnd();
		}
	} else {
		glColor3f(0,0,0);
		glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
		glBegin( GL_QUAD_STRIP );
		{
			/// levo spredaj
			glVertex3f( -sirinaSpodaj, 0, -sirinaSpodaj );		/// spodaj
			glVertex3f( -sirinaZgoraj, visina, -sirinaZgoraj );	/// zgoraj

			/// desno spredaj
			glVertex3f( sirinaSpodaj, 0, -sirinaSpodaj );		/// spodaj
			glVertex3f( sirinaZgoraj, visina, -sirinaZgoraj );	/// zgoraj

			/// desno zadaj
			glVertex3f( sirinaSpodaj, 0, sirinaSpodaj );		/// spodaj
			glVertex3f( sirinaZgoraj, visina, sirinaZgoraj );	/// zgoraj

			/// levo zadaj
			glVertex3f( -sirinaSpodaj, 0, sirinaSpodaj );		/// spodaj
			glVertex3f( -sirinaZgoraj, visina, sirinaZgoraj );	/// zgoraj

			/// sklene levo spredaj
			glVertex3f( -sirinaSpodaj, 0, -sirinaSpodaj );		/// spodaj
			glVertex3f( -sirinaZgoraj, visina, -sirinaZgoraj );	/// zgoraj
		}
		glEnd();
		glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
	}
} //DelckiDrevesa::narisiSteblo

void DelckiDrevesa::narisiListe( int tipListov, double velikost, double globinaVejice, int stListov, Tocka &vektorZasukaVeter, int idVeje ) {
	/// zasučemo za 20 stopinj!
	double kotZasukaVeter = maxKotZasukaVetra;

	//sin(fi) = norma(vektorZasuka)

	///x komponento zanemarimo, ker je 0
	double norma = sqrt( pow( vektorZasukaVeter.y, 2 ) + pow( vektorZasukaVeter.z, 2 ) );
	/// testiramo, ali je zasuk prevelik in bi morda že obrnili čez vektor dol in bi lahko prili v luping
	double fi = asin( norma ) * 180 / M_PI;
	if ( fabs( kotZasukaVeter ) > fabs( fi ) )
		kotZasukaVeter = kotZasukaVeter > 0 ? fabs( fi ) : -fabs( fi );
	kotZasukaVeter *= sin( pogled.cas * 1.97 );

	if ( !pogled.wireframe ) {
		glBindTexture( GL_TEXTURE_2D, pogled.textureNames[ 1 ] );

		switch ( tipListov ) {
		case LEAF_SPIRAL:  /// spiralno okoli za kot 197
			{
				double delcek = globinaVejice / stListov;
				/// pomik ven
				glTranslatef( 0, 0, velikost );
				for ( int i = 0; i < stListov; i++ ) {
					glPushMatrix();
					{
						/// pomik nazaj v srediče
						glTranslatef( 0, 0, -velikost );
						{
							if ( i != 0 ) {
								/// zasuk okoli srediča, da bodo listi razporejeni okoli
								glRotatef( 197 * i, 0.0, 1.0, 0.0 );
							}
							/// veter
							glRotatef( kotZasukaVeter, vektorZasukaVeter.y, vektorZasukaVeter.x, vektorZasukaVeter.z );
						}
						glTranslatef( 0, 0, velikost );

						/// rotacija navzdol glede na vejo
						glRotatef( 30 * ( ( double ) i / stListov * ( double ) ( stListov - i ) / stListov ), 1, 0, 0 );

						if ( serializatorDelckovDrevesa.exportingToPoints )
							serializatorDelckovDrevesa.serializirajList( velikost, idVeje );
						risiList( velikost );
					}
					glPopMatrix();

					/// pomik dol po steblu
					glTranslatef( 0, -delcek, 0 );
				}
			} break;
		case LEAF_STACKED:  /// na vsako stran, izmenjujoče, za kot 180
			{
				double delcek = globinaVejice / stListov;
				/// pomik ven
				glTranslatef( 0, 0, velikost );
				for ( int i = 0; i < stListov; i++ ) {
					glPushMatrix();
					{
						/// pomik nazaj v srediče
						glTranslatef( 0, 0, -velikost );
						{
							if ( i != 0 ) {
								/// zasuk okoli srediča, da bodo listi razporejeni okoli
								glRotatef( 180 * i, 0.0, 1.0, 0.0 );
							}
							/// veter
							glRotatef( kotZasukaVeter, vektorZasukaVeter.y, vektorZasukaVeter.x, vektorZasukaVeter.z );
						}
						glTranslatef( 0, 0, velikost );

						/// rotacija navzdol glede na vejo
						glRotatef( 30 * ( ( double ) i / stListov * ( double ) ( stListov - i ) / stListov ), 1, 0, 0 );

						if ( serializatorDelckovDrevesa.exportingToPoints )
							serializatorDelckovDrevesa.serializirajList( velikost, idVeje );
						risiList( velikost );
					}
					glPopMatrix();

					/// pomik dol po steblu
					glTranslatef( 0, -delcek, 0 );
				}
			} break;
		case LEAF_STAGGERED:  /// na vsako stran, izmenjujoče; če je liho t., pride en na vrh
			{
				/// je listov liho t.?
				if ( stListov % 2 == 1 ) {
					stListov--;
					/// če je liho t. listov, dodamo enega na koncu!
					glTranslatef( 0, velikost, 0 );
					glRotatef( 180, 0, 0, 1 );
					if ( serializatorDelckovDrevesa.exportingToPoints )
						serializatorDelckovDrevesa.serializirajList( velikost, idVeje );
					risiList( velikost );
					glRotatef( -180, 0, 0, 1 );
					glTranslatef( 0, -velikost, 0 );
				}
				double delcek = globinaVejice / stListov;
				/// pomik ven
				glTranslatef( 0, 0, velikost );
				for ( int i = 0; i < stListov; i++ ) {
					glPushMatrix();
					{
						/// pomik nazaj v srediče
						glTranslatef( 0, 0, -velikost );
						{
							if ( i != 0 ) {
								/// zasuk okoli srediča, da bodo listi razporejeni okoli
								glRotatef( 197 * i, 0.0, 1.0, 0.0 );
							}
							/// veter
							glRotatef( kotZasukaVeter, vektorZasukaVeter.y, vektorZasukaVeter.x, vektorZasukaVeter.z );
						}
						glTranslatef( 0, 0, velikost );

						/// rotacija navzdol glede na vejo
						glRotatef( 30 * ( ( double ) i / stListov * ( double ) ( stListov - i ) / stListov ), 1, 0, 0 );

						if ( serializatorDelckovDrevesa.exportingToPoints )
							serializatorDelckovDrevesa.serializirajList( velikost, idVeje );
						risiList( velikost );
					}
					glPopMatrix();

					/// pomik dol po steblu
					glTranslatef( 0, -delcek, 0 );
				}
			} break;
		case LEAF_BUNCHED: {
				double delcek = 90 / stListov;
				for ( int i = 0; i < stListov; i++ ) {
					glPushMatrix();
					{
						/// zasuk okoli srediča, da bodo listi razporejeni okoli
						glRotatef( -90 - 45 + delcek * i, 1.0, 0.0, 0.0 );

						/// pomik ven
						glTranslatef( 0, 0, velikost );

						/// veter
						glRotatef( kotZasukaVeter, vektorZasukaVeter.y, vektorZasukaVeter.x, vektorZasukaVeter.z );

						/// rotacija navzdol glede na vejo
						//glRotatef(30*((double)i/stListov*(double)(stListov-i)/stListov),1,0,0);

						if ( serializatorDelckovDrevesa.exportingToPoints )
							serializatorDelckovDrevesa.serializirajList( velikost, idVeje );
						risiList( velikost );
					}
					glPopMatrix();
				}
			}
			break;
		case LEAF_CONIFEROUS: {
				double delcek = globinaVejice / stListov;
				for ( int i = 0; i < stListov; i++ ) {
					glPushMatrix();
					{
						if ( i != 0 ) {
							/// zasuk okoli srediča, da bodo listi razporejeni okoli
							glRotatef( 197 * i, 0.0, 1.0, 0.0 );
						}
						/// veter
						glRotatef( kotZasukaVeter, vektorZasukaVeter.y, vektorZasukaVeter.x, vektorZasukaVeter.z );

						/// rotacija navzdol glede na vejo
						glRotatef( -70, 1, 0, 0 );
						/// pomik ven
						glTranslatef( 0, 0, velikost );

						if ( serializatorDelckovDrevesa.exportingToPoints )
							serializatorDelckovDrevesa.serializirajList( velikost, idVeje );
						risiList( velikost );
					}
					glPopMatrix();

					/// pomik dol po steblu
					glTranslatef( 0, -delcek, 0 );
				}
			}
			break;
		};
	} else {
		glBegin( pogled.wireframe ? GL_LINE_LOOP : GL_QUADS );

		double delcek = globinaVejice / stListov;
		double globina, stranVeje;

		for ( int i = 0; i < stListov; i++ ) {
			globina = -i * delcek;
			stranVeje = i % 2 == 0 ? velikost : -velikost;
			/// zacetna tocka
			glVertex3f( 0, globina, 0 );
			/// vmesna tocka - desno
			glVertex3f( velikost, globina, stranVeje );
			/// koncna tocka - konica naprej
			glVertex3f( 0, globina, 2 * stranVeje );
			/// vmesna tocka - levo
			glVertex3f( -velikost, globina, stranVeje );
		}
		glEnd();
	}
	//glBlendFunc(GL_SRC_COLOR, GL_DST_COLOR);
} //DelckiDrevesa::narisiListe

void DelckiDrevesa::risiList( float velikost ) {
	glColor4f(0.5f, 0.5f, 0.5f, 0.8f);
	if ( pogled.lowQuality < LEAF_WHOLE ) {
		glRotatef( -90, 1.0, 0.0, 0.0 ); /// z postane y->disk na y=0
		gluDisk( pogled.kvadraticnePloskve, 0, velikost, 4, 1 );
		glRotatef( 90, 1.0, 0.0, 0.0 );
		glPointSize( 1.4f );
		glBegin( GL_POINTS );
		{
			glVertex3f( 0, 0, 0 );
		}
		glEnd();
	} else {
		glPointSize( 1.4f );
		glBegin( GL_POINTS );
		{
			glVertex3f( 0, 0, 0 );
		}
		glEnd();
	}
	glColor4f(0.5f, 0.5f, 0.5f, 1.0f);
} //DelckiDrevesa::risiList
