/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef DREVO_DELCKIDREVESA_H
#define DREVO_DELCKIDREVESA_H

#include "pogled.h"
#include "serializatordelckovdrevesa.h"

namespace drevo {
	/*!
	 * \brief izris elementov drevesa (tessels)
	 *@author Ales Zamuda <ales.zamuda@uni-mb.si>
	 */
	class DelckiDrevesa {
	public:
		/**
		 * Ustvari nov objekt in shrani povezavi za pogled in serializator
		 * @param pogled pogled na drevo, določa način izrisa, LOD in teksture
		 * @param serializatorDelckovDrevesa objekt za serializacijo točk v datoteko
		 * @return
		 */
		DelckiDrevesa(Pogled& pogled, SerializatorDelckovDrevesa &serializatorDelckovDrevesa);
		/**
		 * Nariše steblo na trenutni lokaciji, v obliki stožca
		 * @param visina višina stožca
		 * @param sirinaSpodaj širina stožca spodaj
		 * @param sirinaZgoraj širina stožca zgoraj
		 * @param presekSpodaj presek stožca spodaj
		 * @param idVeje zaporedna številka prave veje po G, ne le odseka (za združevanje pri branju CRS)
		 */
		void narisiSteblo(double visina, double sirinaSpodaj, double sirinaZgoraj, Presek *presekSpodaj, int idVeje);
		/**
		 * Nariše liste na veji, tako da tvori lokacije listov na veji in kljiče privatno metodo risiList.
		 * @param tipListov tip porazdelitve listov na veji
		 * @param velikost velikost lista
		 * @param globinaVejice dolžina veje
		 * @param stListov število listov na veji
		 * @param vektorZasukaVeter zasuk posameznega lista zaradi vetra
		 * @param idVeje zaporedna številka prave veje po G, ne le odseka (za združevanje pri branju CRS)
		 * \sa risiList
		 */
		void narisiListe(int tipListov, double velikost, double globinaVejice, int stListov, Tocka &vektorZasukaVeter, int idVeje);
		/**
		 * Maksimalen zasuk lista v vetru.
		 */
		double maxKotZasukaVetra;
	private:
		/**
		 * Nariše en list na trenutni lokaciji.
		 * @param velikost velikost stranice kvadrata, ki predstavlja list
		 */
		void risiList(float velikost);

		/**
		 * definicije pogleda na drevo: LOD, teksture, wireframe,...
		 */
		Pogled& pogled;
		/**
		 * pomožni objekti za serializacijo drevesa v datoteko
		 */
		SerializatorDelckovDrevesa &serializatorDelckovDrevesa;
	};
}
#endif
