/**************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "drevo.h"
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

using namespace drevo;

Drevo::Drevo() : ZACETEK_LISTOV( 1 ), delckiDrevesa( pogled, serializatorDelckovDrevesa ) {} //Drevo::Drevo

Drevo::~Drevo() {
	///TODO: če ni current, lahko vrže error
	gluDeleteQuadric( pogled.kvadraticnePloskve );
}

void Drevo::createKvadraticnePloskve() {
	pogled.createKvadraticnePloskve();
}

void Drevo::reloadIdTextures( bool bark, bool leaf ) {
	/// tekstura se tu več ne generira:
	//glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
	// 	glGenTextures(2, pogled.textureNames);

	if ( bark ) {
		/// bark
		glBindTexture( GL_TEXTURE_2D, pogled.textureNames[ 0 ] );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
		///teksture ne generiramo
		//glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, TEXTURES_WX, TEXTURES_WY, 0, GL_RGBA, GL_UNSIGNED_BYTE, drevo.pogled.barkTexture);
	}
	if ( leaf ) {
		/// leaf
		glBindTexture( GL_TEXTURE_2D, pogled.textureNames[ 1 ] );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
		///teksture ne generiramo
		//glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, TEXTURES_WX, TEXTURES_WY, 0, GL_RGBA, GL_UNSIGNED_BYTE, drevo.pogled.leafTexture);
	}
}

void Drevo::drevca( int stZil, int G, int W, double karakteristicnaVisina, double aktivnaVisina, Matrika modelMatrixInverse, Matrika modelMatrixInversePostGrowthEffects, Presek *presekSpodaj, int idPraveVeje ) {
	/// preverimo, ali lahko drevo na tem mestu obrežemo, torej končamo z razvojem
	if ( constraint( G, W, stZil ) )
		return ; //nič novih vej1

	/// gravimorfizem
	double gravimorfizemAngle = 0;
	Tocka gravimorfizemVektor;
	aplicirajGravmorfizem( G, W, modelMatrixInverse, gravimorfizemAngle, gravimorfizemVektor );

	/// phyllotaxis
	glRotatef( pd.phyllotaxisAngle, 0.0, 1.0, 0.0 );

	/// DIMENZIONIRANJE
	/// doda skalar na celotno viino drevesa
	double dejanskaVisinaTrenutneVeje = aktivnaVisina /**pd.dolzinaSteblaNaZilo*/;
	/// deli trenutno steblo in določi kot vejitve obeh izhajajočih vej
	int stZilGlavno, stZilStransko;
	double kotVejitveGlavno, kotVejitveStransko;
	deliVejo( G, W, stZil, stZilGlavno, stZilStransko, kotVejitveGlavno, kotVejitveStransko );
	/// dimenzionira trenutno vejo in dolžini izhajajočih vej
	double sirinaSpodaj, sirinaZgoraj, L1, l1, L2, l2;
	izracunajDimenzijeVeje( G, W, stZil, stZilGlavno, stZilStransko, karakteristicnaVisina, sirinaSpodaj, sirinaZgoraj, L1, l1, L2, l2 );

	int dopustniW = pogled.lowQuality == 0 ? MAX_WEI : 10 - pogled.lowQuality;

	double windAngle = 0;
	Tocka windVektor;
	aplicirajWindSway( stZil, aktivnaVisina, modelMatrixInversePostGrowthEffects, windAngle, windVektor );

	///TODO: poisci pravo barvo
	dolociBarvo( G, W, pogled.colorMonopodial );
	/// nariemo vejo
	if ( W < dopustniW )
		dodajVejo( dejanskaVisinaTrenutneVeje, sirinaSpodaj, sirinaZgoraj, presekSpodaj, idPraveVeje );
	/// prevežemo presek zgoraj od prej na zdajnji presek spodaj
	Presek presekSpodajNovi;
	if ( serializatorDelckovDrevesa.exportingToPoints ) {
		presekSpodajNovi = serializatorDelckovDrevesa.getPresekZgoraj();
		popraviPhyllotaxis( presekSpodajNovi );
	}

	/// če niso vse žile le v stransko, narii liste, ki so na vejah z eno žilo
	bool terminateWithLeafLOD = W > dopustniW * dopustniW;
	///TODO: terminateLOD sirina?
	if ( terminateWithLeafLOD || stZil == 1 )
		narisiListe( dejanskaVisinaTrenutneVeje,       /*terminateWithLeafLOD ? sirinaSpodaj * 10 : sirinaSpodaj,*/ modelMatrixInversePostGrowthEffects, idPraveVeje );
	else { /// delitev
		/// pomik gor po steblu, zamik na stran novega stebla (raste direktno iz konca skorje, ne iz srede debla)
		glTranslatef( 0, dejanskaVisinaTrenutneVeje, 0 );

		/// gravicentralizem
		double gravicentralizem = kotVejitveGlavno * pd.graviCentralizem;
		if ( G == 0 )
			glRotatef( gravicentralizem, 0.0, 0.0, 1.0 );
		else
			gravicentralizem = 0;

		/// stransko steblo?
		if ( stZilStransko >= 1 ) {
			Matrika nova = modelMatrixInverse;
			priraviInverznoMatriko( nova, gravimorfizemAngle, gravimorfizemVektor, gravicentralizem, kotVejitveStransko, 0, windVektor );

			Matrika novaPostEffects = modelMatrixInversePostGrowthEffects;
			priraviInverznoMatriko( novaPostEffects, gravimorfizemAngle, gravimorfizemVektor, gravicentralizem, kotVejitveStransko, windAngle, windVektor );

			double sirinaVeje = sqrt( ( double ) stZilStransko ) * pd.sirinaSteblaNaZilo;
			double pomikVejeVRastOdZnotraj = -( sirinaZgoraj - sirinaVeje ) / 2.0;

			/// trenutno matriko postavimo na sklad
			glPushMatrix();
			{
				glTranslatef( pomikVejeVRastOdZnotraj, 0, 0 );
				/// zasuk v smer stranske veje
				glRotatef( kotVejitveStransko, 0.0, 0.0, 1.0 );
				/// kliče rast stranske veje, presek za to je novi: NULL
				drevca( stZilStransko, G + 1, W + 1, L2, l2, nova, novaPostEffects, NULL, idVejaNext++ );
			}
			glPopMatrix();

			/// zasuk v smer za glavno vejo
			glRotatef( -kotVejitveGlavno, 0.0, 0.0, 1.0 );
		} else
			kotVejitveGlavno = 0;

		/// glavno deblo
		if ( stZilGlavno >= 1 ) {
			/// tu lahko povozimo trenutno inverzno matriko
			priraviInverznoMatriko( modelMatrixInverse, gravimorfizemAngle, gravimorfizemVektor, gravicentralizem, -kotVejitveGlavno, 0, windVektor );
			priraviInverznoMatriko( modelMatrixInversePostGrowthEffects, gravimorfizemAngle, gravimorfizemVektor, gravicentralizem, kotVejitveStransko, windAngle, windVektor );
			/// glavna veja, gravelius se ohranja
			drevca( stZilGlavno, G, W + 1, L1, l1, modelMatrixInverse, modelMatrixInversePostGrowthEffects, &presekSpodajNovi, idPraveVeje );
		}
	}
} //Drevo::drevca

void Drevo::aplicirajGravmorfizem( int G, int W, Matrika &modelMatrixInverse, double &gravimorfizemAngle, Tocka &gravimorfizemVektor ) {
	/// če je vpliv 0, se ne matramo računa
	if ( pd.graviMorfizem[ G ][ W ] == 0 ) {
		gravimorfizemAngle = 0;
		return ;
	}

	gravimorfizemAngle = pd.graviMorfizem[ G ][ W ];

	/// imamo koordinatni sistem K1, ki je izhodični koordinatni sistem drevesa,
	/// ter koordinatni sistem K2, ki je trenutni koordinatni sistem
	/// za apliciranje gravimorfizma, torej sukanje v tendenci vektorja dol iz K1,
	/// rotiramo za kot moči gravimorfizma okoli vektorja, ki ga dobimo: (x pomeni vektorski produkt)
	/// 	v = (0,1,0) x (modelMatrixInverse * (0, -1, 0))
	/// ->kar pomeni okoli vektorja, ki je pravokoten na vektor vzdolž veje ter vektor iz koordinatnega sistema K1,
	///   ki kaže dol (slednjega dobimo s transformacijo v koordinatni sistem K1 iz koor. sist. K2, kar pa je inverz matrike K2)
	/// -> da dobimo iz K1 ven K2, pomnožimo z neko matriko M, da pa pridemo iz K2 nazaj v K1, pomnoimo K2 z inverzom od M (M*K2)

	Tocka vzdolzK2( 0, 1, 0 );
	Tocka dolK2( 0, -1, 0 );
	Tocka dolK1;
	/// homogeno koordinato točke damo na 0 -> in dobimo krajevni vektor v to točko
	/// -> za koordinato x ne upotevamo "+ modelMatrixInverse.el[12]", za z pa "+ modelMatrixInverse.el[14]"
	dolK1.x = modelMatrixInverse.el[ 0 ] * dolK2.x + modelMatrixInverse.el[ 4 ] * dolK2.y + modelMatrixInverse.el[ 8 ] * dolK2.z;
	dolK1.z = modelMatrixInverse.el[ 2 ] * dolK2.x + modelMatrixInverse.el[ 6 ] * dolK2.y + modelMatrixInverse.el[ 10 ] * dolK2.z;

	/// gravimorfizem pomakne vejo dol, torej rotira vejo glede na z os absolutnega koordinatnega sistema za nek kot
	gravimorfizemVektor = Tocka( dolK1.z, 0, -dolK1.x ); /// enako kot gravimorfizemVektor=Tocka::vektorskiProd(vzdolzK2, dolK1)

	//sin(fi) = norma(gravimorfizemVektor)
	double norma = sqrt( pow( dolK1.z, 2 ) + pow( dolK1.x, 2 ) ); /// y komponento zanemarimo, ker je 0

	/// testiramo, ali je zasuk prevelik in bi morda že obrnili čez vektor dol in bi lahko prili v luping
	double fi = asin( norma ) * 180 / M_PI;
	if ( fabs( gravimorfizemAngle ) > fabs( fi ) )
		gravimorfizemAngle = gravimorfizemAngle > 0 ? fabs( fi ) : -fabs( fi );

	glRotatef( gravimorfizemAngle, gravimorfizemVektor.x, gravimorfizemVektor.y, gravimorfizemVektor.z );
} //Drevo::aplicirajGravmorfizem

void Drevo::priraviInverznoMatriko( Matrika &modelMatrixInverse, double gravimorfizemAngle,
                                    Tocka &gravimorfizemVektor, double gravicentralizem, double kotVejitve,
                                    double windAngle, Tocka &windVektor ) {
	/// pripravimo novo inverzno matriko
	/// inverzna transformacija: trenutna se bo izvrila za prejnjimi
	glPushMatrix();
	{
		glLoadIdentity();
		{
			/// veter na vejo
			if ( windAngle != 0 )
				glRotatef( -windAngle, windVektor.x, windVektor.y, windVektor.z );
			/// kot vejitve izhajajoče veje
			if ( kotVejitve != 0 )
				glRotatef( -kotVejitve, 0.0, 0.0, 1.0 );
			/// gravicentralizem
			if ( gravicentralizem != 0 )
				glRotatef( -gravicentralizem, 0.0, 0.0, 1.0 );
			/// phyllotaxis
			glRotatef( -pd.phyllotaxisAngle, 0.0, 1.0, 0.0 );
			/// gravimorfizem
			if ( gravimorfizemAngle != 0 )
				glRotatef( -gravimorfizemAngle, gravimorfizemVektor.x, gravimorfizemVektor.y, gravimorfizemVektor.z );
		}
		glMultMatrixf( modelMatrixInverse.el ); /// zmnožimo s prejnjimi transformacijami

		/// shranimo novo inverzno matriko
		glGetFloatv( GL_MODELVIEW_MATRIX, modelMatrixInverse.el );
	}
	glPopMatrix();
} //Drevo::priraviInverznoMatriko

void Drevo::dodajVejo( double dejanskaVisinaTrenutneVeje, double sirinaSpodaj, double sirinaZgoraj, Presek *presekSpodaj, int idVeje ) {
	/// pomik dol za spojek, spojek = približno sirinaSpodaj
	glPushMatrix();
	{
		//glTranslatef(0,-sirinaSpodaj, 0);//izkljuceno, tudi spodaj pri klicu
		/// narie trenutno steblo
		delckiDrevesa.narisiSteblo( dejanskaVisinaTrenutneVeje /*+sirinaSpodaj*/, sirinaSpodaj, sirinaZgoraj, presekSpodaj, idVeje );
	}
	glPopMatrix(); ///nazaj na pravo višino
} //Drevo::dodajVejo

void Drevo::deliVejo( int G, int W, int stZil, int &stZilGlavno, int &stZilStransko, double &kotVejitveGlavno, double &kotVejitveStransko ) {
	/// deli trenutno steblo
	int ostaleZileVGlavnem = ( int ) floor( pd.verjetnostVejitve[ G ][ W ] * ( stZil - 2 ) + 0.5 );
	stZilGlavno = 1 + ostaleZileVGlavnem;
	stZilStransko = stZil - stZilGlavno;

	/// določi kot vejitve obeh vej
	kotVejitveGlavno = ( double ) stZilStransko / stZil * pd.kotVejitve[ G ][ W ];
	kotVejitveStransko = pd.kotVejitve[ G ][ W ] - kotVejitveGlavno;
} //Drevo::deliVejo

void Drevo::izracunajDimenzijeVeje( int G, int W, double stZil, double stZilGlavno, double stZilStransko, double L0,
                                    double &sirinaSpodaj, double &sirinaZgoraj,
                                    double &L1, double &l1, double &L2, double &l2 ) {
	/// izračunamo irino spodaj
	double koren0 = sqrt( ( double ) stZil );
	sirinaSpodaj = koren0 * pd.sirinaSteblaNaZilo;
	/// izračunamo irino zgoraj
	double koren1 = sqrt( ( double ) stZilGlavno );
	sirinaZgoraj = koren1 * pd.sirinaSteblaNaZilo;

	/// izračunamo kvociente za viino nadaljnih vej
	/// velike črke=karakteristična; male črke=aktivna dolina
	/// L0 = karakteristična dolžina trenutne veje
	/// L1 = karakteristična dolžina za glavno podvejo (MAJOR)
	/// L2 = karakteristična dolžina za stransko podvejo (MINOR)
	/// R1 = kvocient karakteristične dolžina za glavno podvejo (MAJOR) glede na trenutno vejo
	/// R2 = kvocient karakteristične dolžina za stransko podvejo (MINOR) glede na trenutno vejo

	/// l1 = aktivna dolžina za glavno podvejo (MAJOR)
	/// l2 = aktivna dolžina za stransko podvejo (MINOR)
	/// r1 = kvocient aktivne dolžine za glavno podvejo (MAJOR) glede na trenutno vejo
	/// r2 = kvocient aktivne dolžine za stransko podvejo (MINOR) glede na trenutno vejo

	/// za glavno vejo..R1,r1->L1,l1
	double R1 = koren1 / koren0;
	if ( pd.charMin[ G ][ W ] > 0 && pd.charMin[ G ][ W ] > R1 )
		R1 = pd.charMin[ G ][ W ];
	if ( pd.charMax[ G ][ W ] > 0 && pd.charMax[ G ][ W ] < R1 )
		R1 = pd.charMax[ G ][ W ];
	/// izračunamo nadaljno karakteristično viino
	L1 = R1 * L0;
	l1 = pd.lengthScaling[ G ][ W ] * L0;

	/// ..e za stransko vejo..R2,r2->L1,l2
	double koren2 = sqrt( ( double ) stZilStransko );
	double R2 = koren2 / koren0;
	if ( pd.charMin[ G ][ W ] > 0 && pd.charMin[ G ][ W ] > R2 )
		R2 = pd.charMin[ G ][ W ];
	if ( pd.charMax[ G ][ W ] > 0 && pd.charMax[ G ][ W ] < R2 )
		R2 = pd.charMax[ G ][ W ];
	/// izračunamo nadaljno aktivno viino
	L2 = R2 * L0;
	/// hack: obe dolžini isti zato, da je modeliranje lažje
	l2 = pd.lengthScaling[ G ][ W ] * L0;
} //Drevo::izracunajDimenzijeVeje

void Drevo::aplicirajWindSway( int stZil, double visinaVeje, Matrika &modelMatrixInversePostGrowthEffects, double &windAngle, Tocka &windVektor ) {
	/// 2 tipa animacije vetra: sunki vetra iz neke strani ter zibanje v vetru
	if ( pogled.animateWindSway ) {
		/// /////// SUNKI VETRA //////////////Wind guss
		Tocka vektorZasukaVeter;
		izracunajWindVektorZasuka( modelMatrixInversePostGrowthEffects, vektorZasukaVeter );

		/// zasučemo za 20 stopinj!
		/// izklopil sem čas pogleda
		double kotZasukaVeter = pd.windSpeed * ( 1.1 - stZil / pd.stVsehZil ) ;//* fabs( cos( pogled.cas ) );
		windAngle = kotZasukaVeter;

		//sin(fi) = norma(vektorZasuka)
		/// x komponento zanemarimo, ker je 0
		double norma = sqrt( pow( vektorZasukaVeter.y, 2 ) + pow( vektorZasukaVeter.z, 2 ) );
		/// testiramo, ali je zasuk prevelik in bi morda že obrnili čez vektor dol in bi lahko prili v luping
		double fi = asin( norma ) * 180 / M_PI;
		if ( fabs( kotZasukaVeter ) > fabs( fi ) )
			kotZasukaVeter = kotZasukaVeter > 0 ? fabs( fi ) : -fabs( fi );

		windVektor = vektorZasukaVeter;

		/// ////// ZABANJE V VETRU ///////////////////Wind sway
		///kilometri na uro=50
		double windSpeed = pd.windSpeedNodirect;

		/// vpliv vetra: samo nagib ne zadostuje
		/// da pridemo do nihanja, modeliramo dinamičen sistem, ki oscilira

		/// vsaka veja ima svoj offset - metoda Myosotis za reitev klasičnega problema mehanike
		double sway_offset_x = ( double ) rand() / RAND_MAX * M_PI * 2;
		double sway_offset_z = ( double ) rand() / RAND_MAX * M_PI * 2;
		///žile z več vejami so bolj odporne na veter in sicer glede na koren tevila iz debeline
		double trdnostVeje = 1.02 - pow( ( double ) stZil / pd.stVsehZil, 1 / 4.0 );

		/// trdnost veje zmanja vpliv
		/// viina veje poveča vpliv
		/// hitrost vetra poveča vpliv
		double sway_x = sin( pogled.cas + sway_offset_x ) * M_PI * trdnostVeje * visinaVeje * windSpeed;
		double sway_z = sin( pogled.cas + sway_offset_z ) * M_PI * trdnostVeje * visinaVeje * windSpeed;

		glRotatef( sway_x, 1, 0, 0 );
		glRotatef( sway_z, 0, 0, 1 );
		/*vektorZasukaVeter.x += sway_x*5;
		  vektorZasukaVeter.z += sway_z*5;*/

		/// sunek vetra + zibanje v vetru
		glRotatef( kotZasukaVeter, vektorZasukaVeter.x, vektorZasukaVeter.y, vektorZasukaVeter.z );
	}
} //Drevo::aplicirajWindSway

void Drevo::izracunajWindVektorZasuka( Matrika &modelMatrixInversePostGrowthEffects, Tocka& vektorZasuka ) {
	/// imamo koordinatni sistem K1, ki je izhodični koordinatni sistem drevesa,
	/// ter koordinatni sistem K2, ki je trenutni koordinatni sistem
	/// za apliciranje vetra, torej sukanje v tendenci vektorja desno iz K1,
	/// rotiramo za kot moči vetra okoli vektorja, ki ga dobimo: (x pomeni vektorski produkt)
	/// 	v = (0,1,0) x (modelMatrixInversePostGrowthEffects * (0, -1, 0))
	/// ->kar pomeni okoli vektorja, ki je pravokoten na vektor vzdolž veje ter vektor iz koordinatnega sistema K1,
	///  ki kaže desno (slednjega dobimo s transformacijo v koordinatni sistem K1 iz koor. sist. K2, kar pa je inverz matrike K2)
	///    -> da dobimo iz K1 ven K2, pomnožimo z neko matriko M, da pa pridemo iz K2 nazaj v K1, pomnožimo K2 z inverzom od M (M*K2)

	//Tocka vzdolzK2(0,1,0);//za ilustracijo
	//Tocka desnoK2(-1,0,0);
	Tocka desnoK1;
	/// homogeno koordinato točke damo na 0 -> in dobimo krajevni vektor v to točko
	/// -> za koordinato y ne upotevamo "+ modelMatrixInversePostGrowthEffects.el[13]", za z pa "+ modelMatrixInversePostGrowthEffects.el[14]"
	desnoK1.y = -modelMatrixInversePostGrowthEffects.el[ 1 ];
	desnoK1.z = -modelMatrixInversePostGrowthEffects.el[ 2 ];

	/// veter pomakne vejo na stran, torej rotira vejo glede na y os absolutnega koordinatnega sistema za nek kot
	/// menjal:y,-z
	vektorZasuka = Tocka( 0, desnoK1.y, desnoK1.z ); /// enako kot vektorZasuka=Tocka::vektorskiProd(vzdolzK2, desnoK1)
} //Drevo::izracunajWindVektorZasuka

bool Drevo::constraint( int G, int W, int stZil ) {
	if ( stZil <= 0 )
		return true;
	if ( G != 0 && ( G >= MAX_GRA || W >= MAX_WEI ) ) {
		return true; ///obrežemo preveč razvejajoče veje
	}

	return false;
} //Drevo::constraint

void Drevo::dolociBarvo( int G, int W, bool colorMonopodial ) {
	/// RENDER
	if ( colorMonopodial ) {
		barvaStebla[ 0 ] = G * 0.1;
		barvaStebla[ 1 ] = W * 0.1;
		if ( G == 1 )
			barvaStebla[ 2 ] = 1;
		else
			barvaStebla[ 2 ] = 0.2f;
		glColor3fv( barvaStebla );
	} else
		/// barvanje posebnega nivoja
		if ( pogled.barvajGMin <= G && pogled.barvajGMax >= G ||
		        pogled.barvajWMin <= W && pogled.barvajWMax >= W ) {
			barvaStebla[ 0 ] = 0; /// rdeča
			barvaStebla[ 1 ] = 1;
			barvaStebla[ 2 ] = 1;
			glColor3fv( barvaStebla );
		} else {
			/// karakteristična barva se dobi iz piksla na sredini
			///TODO: kaj narediti s to barvo?
			//glColor4ubv( pogled.barkTexture[ TEXTURES_WY / 2 ][ TEXTURES_WX / 2 ] );

			glColor3f( 1, 1, 1 );

		}
} //Drevo::dolociBarvo

void Drevo::popraviPhyllotaxis( Presek &presekSpodajNovi ) {
	Presek presek = presekSpodajNovi;
	int pomik = presekSpodajNovi.stTock / 4; /// za pi/2
	for ( int i = 0; i < presekSpodajNovi.stTock; i++ ) {
		int naslTocka = ( i + pomik ) % presekSpodajNovi.stTock;
		presek.tocke[ i ] = presekSpodajNovi.tocke[ naslTocka ];
	}
	presekSpodajNovi = presek;
	/*
	Tocka sredisce(0,0,0);
	for (int i = 0; i < presekSpodajNovi.stTock; i++) {
		sredisce = sredisce + presekSpodajNovi.tocke[i];
	}
	sredisce = sredisce * (1.0/presekSpodajNovi.stTock);
	for (int i = 0; i < presekSpodajNovi.stTock; i++) {
		rotirajTocko(sredisce, presekSpodajNovi.tocke[i], pd.phyllotaxisAngle, windAngle, windVektor);
	}
	*/
} //Drevo::popraviPhyllotaxis

void Drevo::rotirajTocko( Tocka &sredisce, Tocka &tocka, double kot, double &windAngle, Tocka &windVektor ) {
	glPushMatrix();
	{
		/// veter na vejo
		if ( windAngle != 0 )
			glRotatef( -windAngle, windVektor.x, windVektor.y, windVektor.z );
		//glTranslatef(sredisce.x, sredisce.y, sredisce.z);
		glRotatef( kot, 0, 1, 0 ); /// tudi to točko obrnemo
		Tocka t = tocka - sredisce;
		glTranslatef( t.x, t.y, t.z );

		Matrika matrixWorld;
		glGetFloatv( GL_MODELVIEW_MATRIX, matrixWorld.el );
		tocka.x = matrixWorld.a14;
		tocka.y = matrixWorld.a24;
		tocka.z = matrixWorld.a34;
	}
	glPopMatrix();
} //Drevo::rotirajNazaj

void Drevo::narisiListe( double dejanskaVisinaTrenutneVeje, Matrika &modelMatrixInversePostGrowthEffects, int idVeje ) {
	/// izračun vetra
	Tocka vektorZasuka;
	izracunajWindVektorZasuka( modelMatrixInversePostGrowthEffects, vektorZasuka );

	glTranslatef( 0, dejanskaVisinaTrenutneVeje, 0 );
	if ( pd.listi ) {
		/// karakteristična barva se dobi iz piksla na sredini
		//TODO:brez barve!
		/*		glColor4ub( pogled.leafTexture[ TEXTURES_WY / 2 ][ TEXTURES_WX / 2 ][ 0 ],
				            pogled.leafTexture[ TEXTURES_WY / 2 ][ TEXTURES_WX / 2 ][ 1 ],
				            pogled.leafTexture[ TEXTURES_WY / 2 ][ TEXTURES_WX / 2 ][ 2 ],
				            ( GLubyte ) 200 );*/
		glColor3f( 1, 1, 1 );
		delckiDrevesa.narisiListe( pd.tipListov, pd.listSize,
		                           dejanskaVisinaTrenutneVeje,
		                           pd.gostotaListov, vektorZasuka, idVeje );
	}
} //Drevo::narisiListe

void Drevo::postaviPrivzeteLastnostiDrevesa() {
	pd = pdPrepared; ///že poprej nekaj nastavljeno preko Control
} //Drevo::postaviPrivzeteLastnostiDrevesa

void Drevo::markSerializingToFile( const std::string &fileNameExportCRS ) {
	this->fileNameExportCRS = fileNameExportCRS;
	this->serializatorDelckovDrevesa.exportingToPoints = true;
} //Drevo::markSerializingToFile

void Drevo::upodobiPrikaz( ) {
	postaviPrivzeteLastnostiDrevesa();
	int stUpodobljenihZil=pd.stVsehZil;
	double starost = pd.dolzinaDebla;
	if ( pogled.animateGrow ) {
		double stZil = sqrt( ( double ) pd.stVsehZil );
		
		if ( pogled.starostDrevesa > 1 )
			pogled.starostDrevesa = 0;
		stZil *= pogled.starostDrevesa;

		pogled.starostDrevesa += pogled.korak; /// 100 let
		starost = pd.dolzinaDebla * pogled.starostDrevesa;
		stUpodobljenihZil=( int ) pow( stZil, 2 );
	}
	
	if (pogled.animateOrbitAround)
		pogled.orbit+=5.37;

	Matrika modelMatrixInverse;
	modelMatrixInverse.el[ 0 ] = 1;
	modelMatrixInverse.el[ 1 ] = 0;
	modelMatrixInverse.el[ 2 ] = 0;
	modelMatrixInverse.el[ 3 ] = 0;
	modelMatrixInverse.el[ 4 ] = 0;
	modelMatrixInverse.el[ 5 ] = 1;
	modelMatrixInverse.el[ 6 ] = 0;
	modelMatrixInverse.el[ 7 ] = 0;
	modelMatrixInverse.el[ 8 ] = 0;
	modelMatrixInverse.el[ 9 ] = 0;
	modelMatrixInverse.el[ 10 ] = 1;
	modelMatrixInverse.el[ 11 ] = 0;
	modelMatrixInverse.el[ 12 ] = 0;
	modelMatrixInverse.el[ 13 ] = 0;
	modelMatrixInverse.el[ 14 ] = 0;
	modelMatrixInverse.el[ 15 ] = 1;
	Matrika modelMatrixInversePostGrowthEffects = modelMatrixInverse;

	barvaStebla[ 0 ] = 1.0;
	barvaStebla[ 1 ] = 0.4f;
	barvaStebla[ 2 ] = 0.2f;

	srand( 1 );
	pogled.windGauss += 0.5;
	if ( pogled.windGauss > 10 )
		pogled.windGauss = 0;
	int framerate = 100;
	pogled.cas += M_PI * 2 / framerate;
	idVejaNext = 1;

	delckiDrevesa.maxKotZasukaVetra = pogled.animateWindSway ? 20 + sin( pogled.cas ) * sin( pogled.cas ) * 50 : 0;
	/// kvaliteto izrisa točk določa HighQualityRendering: kvadrati ali petnajstkotniki
	if ( serializatorDelckovDrevesa.exportingToPoints )
		serializatorDelckovDrevesa.zacniSerializacijo( fileNameExportCRS, pogled.highQualityRendering ? 16 : 4 );

	if (pogled.wireframe) {
		glDisable( GL_BLEND );
		glDisable(GL_TEXTURE_2D);
	} else {
		glEnable(GL_TEXTURE_2D);
		glEnable( GL_DEPTH_TEST );

		glEnable( GL_BLEND );
		glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE ); /// textures with lightning
		//glEnable(GL_POLYGON_SMOOTH);
		glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE );
		glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
	}

	/// st. zil na stVsehZil, gravelius=0, weibull=0, charLength=1,activeLength=starost,nagib=0
	drevca( stUpodobljenihZil, 0, 0, 1, starost, modelMatrixInverse, modelMatrixInversePostGrowthEffects, NULL, 0 );

	if ( serializatorDelckovDrevesa.exportingToPoints ) {
		serializatorDelckovDrevesa.koncajSerializacijo();
		serializatorDelckovDrevesa.exportingToPoints = false;
	}
} //Drevo::upodobiPrikaz

void Drevo::drawSingleTreeCentered( double dol_pomik, double levo_pomik, double z_pomik, double x_zasuk, double y_zasuk) {
	/// izbira ozadja
	if ( pogled.backgroundSky )
		glClearColor( 68 / 255.0f, 121 / 255.0f, 137 / 255.0f, 1.0f ); /// modra
	else
		glClearColor( 1.0, 1.0, 1.0, 1.0 ); /// bela-zajemanje slik

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	GLfloat light_position[] = {
	                               GL_DIFFUSE, 0, 0, 5
	                           }
	                           ; /*0,0,0,1*/
	glLightfv( GL_LIGHT0, GL_POSITION, light_position );
	light_position[ 1 ] -= 3;
	glLightfv( GL_LIGHT0 + 1, GL_POSITION, light_position );
	light_position[ 2 ] += 3;
	glLightfv( GL_LIGHT0 + 2, GL_POSITION, light_position );

	/// postavimo izhodiščno pozicijo
	if ( !serializatorDelckovDrevesa.exportingToPoints ) {
		/// to je razlog, da ne vidim drevesa, ko exportam: saj mora biti na 0, da dobim prave koordinate!!!
		glTranslatef( -levo_pomik, dol_pomik, z_pomik );
		glRotatef( x_zasuk, 1.0, 0.0, 0.0 ); /// zasuk v osi x
		glRotatef( y_zasuk, 0.0, 1.0, 0.0 ); /// zasuk v osi y
		glRotatef( pogled.orbit, 0.f, 1.f, 0.f ); /// orbitiramo po Y
	}

	pogled.lowQuality = 0; /// nariše drevo v polni kvaliteti
	upodobiPrikaz( );
}

void Drevo::drawSingleTreeAwayLOD(int lod) {
	pogled.lowQuality = lod; /// nariše drevo v kvaliteti, v odvisnosti od oddaljenosti od kamere
	upodobiPrikaz();
}
