/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "parametridrevesa.h"
#include "tocka2d.h"

using namespace drevo;

ParametriDrevesa::ParametriDrevesa() {
	defaultTreeParams( );
}

ParametriDrevesa::~ParametriDrevesa() {}

void ParametriDrevesa::defaultTreeParams( ) {
	stVsehZil = 1000;
	dolzinaDebla = 5;

	///privzeto=dolzinaSteblaNaZilo = 1;
	sirinaSteblaNaZilo = 0.01f /*0.06*/;

	listi = true;
	listSize = 0.1f /*0.7*/;
	gostotaListov = 5;
	tipListov = 0;

	graviCentralizem = 0.3;
	phyllotaxisAngle = 85;
	windSpeed = 1.5;
	windSpeedNodirect=50.0 / 20;

	/// distribucija za G lahko gre različno, npr.:
	/// -od 0 do 1 (kot vejitve: se nato množi z 180 stopinj!)
	/// -od 0.5 do 1,(verjetnosti vejitve)
	/// od 0 do 10 (distribucije dolžin vej)
	/// distribucija za W gre vedno od 0 do 1 in se množi z distribucijo za G,
	/// tako da dobimo končno vrednost

	/// verjetnost vejitve
	Tocka2D zacVejitevG( 0, 0.5 );
	Tocka2D konVejitevG( 1, 0.8 );
	Tocka2D zacVejitevW( 0, 0.5 );
	Tocka2D konVejitevW( 1, 0.5 );
	verjetnostiVejitveG.kontrolneTocke.clear();
	verjetnostiVejitveG.kontrolneTocke.push_back( zacVejitevG );
	verjetnostiVejitveG.kontrolneTocke.push_back( konVejitevG );
	verjetnostiVejitveW.kontrolneTocke.clear();
	verjetnostiVejitveW.kontrolneTocke.push_back( zacVejitevW );
	verjetnostiVejitveW.kontrolneTocke.push_back( konVejitevW );

	/// kot vejitve
	Tocka2D zacKotG( 0, 0.25 );
	Tocka2D konKotG( 1, 0.25 );
	Tocka2D zacKotW( 0, 0.5 );
	Tocka2D konKotW( 1, 0.5 );
	kotVejitveG.kontrolneTocke.clear();
	kotVejitveG.kontrolneTocke.push_back( zacKotG );
	kotVejitveG.kontrolneTocke.push_back( konKotG );
	kotVejitveW.kontrolneTocke.clear();
	kotVejitveW.kontrolneTocke.push_back( zacKotW );
	kotVejitveW.kontrolneTocke.push_back( konKotW );

	/// minimalna karakteristična dolžina
	Tocka2D zacCharMinG( 0, 1 / 10.0 ); /// cel čas na 1
	Tocka2D sredCharMinG( 0.5, 0.45 / 10.0 );
	Tocka2D konCharMinG( 1, 1 / 10.0 );
	Tocka2D zacCharMinW( 0, 0.5 ); /// W lahko vedno le od min. 0 do max. 1
	Tocka2D konCharMinW( 1, 0.5 ); /// W lahko vedno le od min. 0 do max. 1
	minCharacteristicLengthG.kontrolneTocke.clear();
	minCharacteristicLengthG.kontrolneTocke.push_back( zacCharMinG );
	minCharacteristicLengthG.kontrolneTocke.push_back( sredCharMinG );
	minCharacteristicLengthG.kontrolneTocke.push_back( konCharMinG );
	minCharacteristicLengthW.kontrolneTocke.clear();
	minCharacteristicLengthW.kontrolneTocke.push_back( zacCharMinW );
	minCharacteristicLengthW.kontrolneTocke.push_back( konCharMinW );

	/// maksimalna karakteristična dolžina
	Tocka2D zacCharMaxG( 0, 1 / 10.0 ); /// omejitev navzgor ni vazna
	Tocka2D konCharMaxG( 1, 1 / 10.0 );
	Tocka2D zacCharMaxW( 0, 0.5 ); /// W lahko vedno le od min. 0 do max. 1
	Tocka2D konCharMaxW( 1, 0.5 ); /// W lahko vedno le od min. 0 do max. 1
	maxCharacteristicLengthG.kontrolneTocke.clear();
	maxCharacteristicLengthG.kontrolneTocke.push_back( zacCharMaxG );
	maxCharacteristicLengthG.kontrolneTocke.push_back( konCharMaxG );
	maxCharacteristicLengthW.kontrolneTocke.clear();
	maxCharacteristicLengthW.kontrolneTocke.push_back( zacCharMaxW );
	maxCharacteristicLengthW.kontrolneTocke.push_back( konCharMaxW );

	/// lengthScaling efekt
	Tocka2D zacLengthScalingG( 0, 1 / 10.0 ); /// ni vpliva
	Tocka2D konLengthScalingG( 1, 1 / 10.0 );
	Tocka2D zacLengthScalingW( 0, 1 / 2.0 );
	Tocka2D konLengthScalingW( 1, 1 / 2.0 );
	lengthScalingG.kontrolneTocke.clear();
	lengthScalingG.kontrolneTocke.push_back( zacLengthScalingG );
	lengthScalingG.kontrolneTocke.push_back( konLengthScalingG );
	lengthScalingW.kontrolneTocke.clear();
	lengthScalingW.kontrolneTocke.push_back( zacLengthScalingW );
	lengthScalingW.kontrolneTocke.push_back( konLengthScalingW );

	/// gravimorphism efekt
	Tocka2D zacGravimorphismG( 0, 0.5 ); /// ni vpliva
	Tocka2D konGravimorphismG( 1, 0.5 );
	Tocka2D zacGravimorphismW( 0, 0.5 );
	Tocka2D konGravimorphismW( 1, 0.5 );
	gravimorphismImpactG.kontrolneTocke.clear();
	gravimorphismImpactG.kontrolneTocke.push_back( zacGravimorphismG );
	gravimorphismImpactG.kontrolneTocke.push_back( konGravimorphismG );
	gravimorphismImpactW.kontrolneTocke.clear();
	gravimorphismImpactW.kontrolneTocke.push_back( zacGravimorphismW );
	gravimorphismImpactW.kontrolneTocke.push_back( konGravimorphismW );

	barkTextureFileName = ":/gui/textures/bark.jpg";
	leafTextureFileName = ":/gui/textures/leaf.png";

	recalculateRelativeParams();
}

void ParametriDrevesa::recalculateRelativeParams() {
	preracunajParametra( STRAND_DISTRIBUTION );
	preracunajParametra( BRANCH_ANGLES_DISTRIBUTION );
	preracunajParametra( MIN_CHARACTERISTIC_LENGTH );
	preracunajParametra( MAX_CHARACTERISTIC_LENGTH );
	preracunajParametra( LENGTH_SCALING );
	preracunajParametra( GRAVIMORPHISM_IMPACT );
}

void ParametriDrevesa::preracunajParametra( int idParametra ) {
	/// določimo parametra glede na id
	Krivulja * pkG = 0;
	Krivulja *pkW = 0;

	switch ( idParametra ) {
	case STRAND_DISTRIBUTION:
		pkG = &verjetnostiVejitveG;
		pkW = &verjetnostiVejitveW;
		break;
	case BRANCH_ANGLES_DISTRIBUTION:
		pkG = &kotVejitveG;
		pkW = &kotVejitveW;
		break;
	case MIN_CHARACTERISTIC_LENGTH:
		pkG = &minCharacteristicLengthG;
		pkW = &minCharacteristicLengthW;
		break;
	case MAX_CHARACTERISTIC_LENGTH:
		pkG = &maxCharacteristicLengthG;
		pkW = &maxCharacteristicLengthW;
		break;
	case LENGTH_SCALING:
		pkG = &lengthScalingG;
		pkW = &lengthScalingW;
		break;
	case GRAVIMORPHISM_IMPACT:
		pkG = &gravimorphismImpactG;
		pkW = &gravimorphismImpactW;
		break;
	};
	if ( !pkW || !pkG )
		return ;

	/// preračunamo parametra
	for ( int g = 0; g < MAX_GRA; g++ ) {
		double poG = interpoliraj( ( double ) g / MAX_GRA, pkG );
		if ( idParametra == STRAND_DISTRIBUTION )
			poG = poG / 2 + 0.5;
		for ( int w = 0; w < MAX_WEI; w++ ) {
			double poW = interpoliraj( ( double ) w / MAX_WEI, pkW );
			double val = poG * poW;
			/// distribution za glavno vejo je vedno več od 50%!
			// if (idParametra == STRAND_DISTRIBUTION && val < 0.5)
			//	val = 0.5;

			/// izračun poteka glede na tip parametra
			switch ( idParametra ) {
			case STRAND_DISTRIBUTION:
				verjetnostVejitve[ g ][ w ] = max2( min2( val * 2, 1.0 ), 0.5 );
				break;
			case BRANCH_ANGLES_DISTRIBUTION:
				kotVejitve[ g ][ w ] = min2( val * 180 * 2, 180.0 );
				break;
			case MIN_CHARACTERISTIC_LENGTH:
				charMin[ g ][ w ] = val * 10 * 2;
				break;
			case MAX_CHARACTERISTIC_LENGTH:
				charMax[ g ][ w ] = val * 10 * 2;
				break;
			case LENGTH_SCALING:
				lengthScaling[ g ][ w ] = val * 10 * 2;
				break;
			case GRAVIMORPHISM_IMPACT:
				graviMorfizem[ g ][ w ] = max2( min2( val * 360 * 2 - 180, 180.0 ), -180 );
				break;
			};
		}
	}
}

double ParametriDrevesa::interpoliraj( double x, Krivulja *parametri ) {
	std::vector<Tocka2D> tocke = parametri->kontrolneTocke;
	int stTock = ( int ) tocke.size();
	/// mediana: poičemo sosednji točki
	int meja = 0;
	for ( int i = 1; i < stTock; i++ ) {
		Tocka2D t = tocke[ i ];
		if ( t.x > x ) {
			meja = i - 1;
			break;
		}
	}
	/// naklon premice, ki gre skozi zahtevan x
	double dx = tocke[ meja + 1 ].x - tocke[ meja ].x;
	double dy = tocke[ meja + 1 ].y - tocke[ meja ].y;
	double k = dx == 0 ? 0 : dy / dx;
	/// izračunamo y za zahtevan x: odsek + k*lokalni x
	double val = ( double ) tocke[ meja ].y + k * ( x - tocke[ meja ].x );

	//double val = ((double)tocke[meja].y + (double)tocke[meja+1].y)/2;
	return val;
}

bool ParametriDrevesa::loadParameters( std::string fileName ) {
	std::ifstream f( fileName.c_str() );
	if ( !f.is_open() )
		return false;

	/// skalarni parametri
	char buf[ 255 ];
	f.getline( buf, sizeof( buf ) );
	barkTextureFileName = buf;
	f.getline( buf, sizeof( buf ) );
	leafTextureFileName = buf;

	f >> stVsehZil;
	f >> dolzinaDebla;
	f >> sirinaSteblaNaZilo;
	f >> phyllotaxisAngle;
	f >> graviCentralizem;
	f >> listi;
	f >> listSize;
	f >> gostotaListov;
	f >> windSpeed;
	f >> tipListov;

	/// porazdelitvene parametre shranimo v obliki njihovih dvojnikov v grafu
	naloziParameter( f, verjetnostiVejitveG );
	naloziParameter( f, verjetnostiVejitveW );

	naloziParameter( f, kotVejitveG );
	naloziParameter( f, kotVejitveW );

	/// major characteristic length
	naloziParameter( f, minCharacteristicLengthG );
	naloziParameter( f, minCharacteristicLengthW );
	naloziParameter( f, maxCharacteristicLengthG );
	naloziParameter( f, maxCharacteristicLengthW );

	naloziParameter( f, lengthScalingG );
	naloziParameter( f, lengthScalingW );

	naloziParameter( f, gravimorphismImpactG );
	naloziParameter( f, gravimorphismImpactW );

	/// osvežimo izračun parametrov in sliko
	recalculateRelativeParams();

	return true;
}

bool ParametriDrevesa::saveParameters( std::string fileName ) {
	std::ofstream f( fileName.c_str() );
	if ( !f.is_open() )
		return false;

	/// skalarni parametri
	f << barkTextureFileName << std::endl;
	f << leafTextureFileName << std::endl;
	f << stVsehZil << '\t';
	f << dolzinaDebla << '\t';
	f << sirinaSteblaNaZilo << '\t';
	f << phyllotaxisAngle << '\t';
	f << graviCentralizem << '\t';
	f << listi << '\t';
	f << listSize << '\t';
	f << gostotaListov << '\t';
	f << windSpeed << '\t';
	f << tipListov << std::endl;

	/// porazdelitvene parametre shranimo v obliki njihovih dvojnikov v grafu
	shraniParameter( f, verjetnostiVejitveG );
	shraniParameter( f, verjetnostiVejitveW );

	shraniParameter( f, kotVejitveG );
	shraniParameter( f, kotVejitveW );

	/// characteristic length
	shraniParameter( f, minCharacteristicLengthG );
	shraniParameter( f, minCharacteristicLengthW );
	shraniParameter( f, maxCharacteristicLengthG );
	shraniParameter( f, maxCharacteristicLengthW );

	shraniParameter( f, lengthScalingG );
	shraniParameter( f, lengthScalingW );

	shraniParameter( f, gravimorphismImpactG );
	shraniParameter( f, gravimorphismImpactW );

	return true;
}

void ParametriDrevesa::shraniParameter( std::ofstream &f, const Krivulja &pk ) {
	int dolzina = ( int ) pk.kontrolneTocke.size();
	f << dolzina << "\t" << std::endl;

	/// shranim vsako točko na grafu kot par (x,y)
	for ( int i = 0; i < dolzina; i++ )
		f << pk.kontrolneTocke[ i ].x << "\t" << pk.kontrolneTocke[ i ].y << std::endl;

	f << std::endl;
}

void ParametriDrevesa::naloziParameter( std::ifstream &f, Krivulja &pk ) {
	pk.kontrolneTocke.clear();

	int dolzina = 0;
	/// stevilo elementov v vektorju parametra
	f >> dolzina;

	for ( int i = 0; i < dolzina; i++ ) {
		double x, y;
		f >> x >> y;
		Tocka2D t( x, y );
		pk.kontrolneTocke.push_back( t );
	}
}
