/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DREVO_POGLED_H
#define DREVO_POGLED_H

#include <qgl.h>

namespace drevo {
	/**
	 * \brief lastnosti pogleda na drevo, nima nič veze z OpenGL
	 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
	 */
	class Pogled {
	public:
		/**
		 * Ustvari parametre pogleda in jih inicializira.
		 * @return
		 */
		Pogled();
		~Pogled();
		/**
		 * Kreira GLU pomožni objekt za delo z kvadratičnimi ploskvami.
		 */
		void createKvadraticnePloskve();

		/**
		 * Starost drevesa (0..1, 1..2=nazaj) pri animaciji,
		 * v odvisnosti od tega postavimo število žil v drevesu pri klicu "drevca".
		 */
		double starostDrevesa;
		/**
		 * Korak (hitrost) rasti starosti pri animaciji rasti v pogledu enega samega drevesa.
		 */
		double korak;
		/**
		 * Moč pihanja vetra, da se drevo pozibava v njem.
		 */
		double windGauss;
		/**
		 * Čas, ki vpliva na pozibavanje v vetru.
		 */
		double cas;
		/**
		 * Določilo, ali naj pobarvamo veje vzdolž G.
		 */
		bool colorMonopodial;

		/**
		 * Določa žični prikaz drevesa.
		 */
		bool wireframe;
		/**
		 * Preklopi med 4 in 15 ogljišč v presekih vej ob upodabljanju.
		 */
		bool highQualityRendering;
		/**
		 * Risanje modrega ozadja pri prikazu enega drevesa.
		 */
		bool backgroundSky;
		/**
		 * Naj se vrši animacija zibanja v vetru.
		 */
		bool animateWindSway;
		/**
		 * Naj se vrši animacija rasti drevesa.
		 */
		bool animateGrow;
		/**
		 * Naj se vrši animacija obračanja drevesa.
		 */
		bool animateOrbitAround;
		/**
		 * Kot zasuka okoli y (centralne osi debla) prvega odseka drevesa.
		 */
		double orbit;

		/**
		 * Namig ob načrtovanju distribucije: od katerega G dalje naj pobarvamo.
		 */
		int barvajGMin;
		/**
		 * Namig ob načrtovanju distribucije: do katerega G naj pobarvamo.
		 */
		int barvajGMax;
		/**
		 * Namig ob načrtovanju distribucije: od katerega W dalje naj pobarvamo.
		 */
		int barvajWMin;
		/**
		 * Namig ob načrtovanju distribucije: do katerega W naj pobarvamo.
		 */
		int barvajWMax;

		/**
		 * LOD kvaliteta izrisa: bolj kot je low, bolj pomanjkljiv je izris.
		 */
		int lowQuality;

		// TODO: spremeni v QGl teksturo
		/*	GLubyte barkTexture[TEXTURES_WY][TEXTURES_WX][4];
			GLubyte leafTexture[TEXTURES_WY][TEXTURES_WX][4];*/
		/**
		 * OpenGL Id-ja za teksturi debla in listov.
		 */
		GLuint textureNames[2];

		/**
		 * pomožni objekti za renderiranje štirikotnikov
		 */
		GLUquadricObj *kvadraticnePloskve;
	};
}
#endif
