/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "serializatordelckovdrevesa.h"
#include "matrika.h"
#include "qgl.h"

using namespace drevo;

SerializatorDelckovDrevesa::SerializatorDelckovDrevesa() {
	presekSpodaj.stTock = 4;
	presekZgoraj.stTock = 4;
	exportingToPoints = false;
} //SerializatorDelckovDrevesa::SerializatorDelckovDrevesa

SerializatorDelckovDrevesa::~SerializatorDelckovDrevesa() {} //SerializatorDelckovDrevesa::~SerializatorDelckovDrevesa

void SerializatorDelckovDrevesa::zacniSerializacijo( const std::string &strFileName, int stTock ) {
	out.open( strFileName.c_str() );
	this->stTock = stTock;
	presekSpodaj.stTock = stTock;
	presekZgoraj.stTock = stTock;
}

void SerializatorDelckovDrevesa::koncajSerializacijo() {
	out.close();
}

void SerializatorDelckovDrevesa::addPointToLWO( Tocka &tocka ) {
	out << tocka.x << " " << tocka.y << " " << tocka.z << std::endl;
} //SerializatorDelckovDrevesa::addPointToLWO

void SerializatorDelckovDrevesa::getPoint( Tocka &tocka, double pomikY, double sirina, double sirinaZ, double kot ) {
	Matrika matrikaRealWorld;
	/// ugotovimo koordinate
	glPushMatrix();
	{
		/// e vedno kreiramo inverzno matriko: kasneje bomo jemali "negativne koordinate"!!!
		if ( pomikY != 0 )
			glTranslatef( 0, pomikY, 0 );
		glRotatef( kot, 0, 1, 0 );
		glTranslatef( sirina, 0, sirinaZ );
		/// shranimo novo inverzno matriko
		glGetFloatv( GL_MODELVIEW_MATRIX, matrikaRealWorld.el );
		tocka.x = matrikaRealWorld.a14;
		tocka.y = matrikaRealWorld.a24;
		tocka.z = matrikaRealWorld.a34;
	}
	glPopMatrix();
} //SerializatorDelckovDrevesa::getPoint

void SerializatorDelckovDrevesa::serializirajVejo( double visina, double sirinaSpodaj, double sirinaZgoraj, Presek *presekSpodaj, int idVeje ) {
	out << "B " << idVeje << std::endl;
	/// spodaj
	if ( presekSpodaj == NULL ) {
		double kot = 0;
		for ( int i = 0; i < stTock; i++ ) {
			if ( i != 0 )
				kot = 360.0 / stTock * i;
			/// nove koordinate pridobimo samo za podvejo!
			getPoint( this->presekSpodaj.tocke[ i ], 0, sirinaSpodaj, sirinaSpodaj, kot ); /// levoSpredajSpodaj
		}
	} else {
		this->presekSpodaj = *presekSpodaj; /// prevzamemo kar prejnje koordinate
	}
	/// zgoraj
	double kot = 0;
	for ( int i = 0; i < stTock; i++ ) {
		if ( i != 0 )
			kot = 360.0 / stTock * i;
		/// nove koordinate pridobimo samo za podvejo!
		getPoint( this->presekZgoraj.tocke[ i ], visina, sirinaZgoraj, sirinaZgoraj, kot ); /// levoSpredajSpodaj
	}

	for ( int i = 0; i < this->presekSpodaj.stTock; i++ ) {
		int naslTocka = ( i + 1 ) % this->presekSpodaj.stTock;
		addPointToLWO( this->presekSpodaj.tocke[ naslTocka ] ); /// spodaj
		addPointToLWO( this->presekZgoraj.tocke[ naslTocka ] ); /// zgoraj
		addPointToLWO( this->presekZgoraj.tocke[ i ] ); /// zgoraj
		addPointToLWO( this->presekSpodaj.tocke[ i ] ); /// spodaj
	}
} //SerializatorDelckovDrevesa::serializirajVejo

Presek SerializatorDelckovDrevesa::getPresekZgoraj() {
	return presekZgoraj;
} //SerializatorDelckovDrevesa::getPresekZgoraj

void SerializatorDelckovDrevesa::serializirajList( double stranica, int idVeje ) {
	out << "L " << idVeje << std::endl;
	glPushMatrix();
	{
		Tocka ogljisca[ 4 ];
		getPoint( ogljisca[ 0 ], 0, stranica, 0, 0 ); //kar ta točka
		getPoint( ogljisca[ 1 ], 0, stranica, 0, 90 );
		getPoint( ogljisca[ 2 ], 0, stranica, 0, 180 );
		getPoint( ogljisca[ 3 ], 0, stranica, 0, 270 );
		for ( int i = 0; i < 4; i++ )
			addPointToLWO( ogljisca[ i ] );
	}
	glPopMatrix();
}
