/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef DREVO_SERIALIZATORDELCKOVDREVESA_H
#define DREVO_SERIALIZATORDELCKOVDREVESA_H

#include <fstream>
#include <string>
#include <vector>

#include "tocka.h"
#include "presek.h"

namespace drevo {
	/**
	 * \brief export drevesa kot 3D točke, npr. za Lightwave, ki prebere s skripto CRS
	 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
	*/
	class SerializatorDelckovDrevesa {
	public:
		/**
		 * Ustvari nov objekt in inicializira število točk v preseku na 4.
		 * @return
		 */
		SerializatorDelckovDrevesa();
		~SerializatorDelckovDrevesa();

		/**
		 * Odpre datoteko za shranjevanje in shrani število točk za obliko preseka.
		 * Inicializira serializator in odpre izhodno datoteko.
		 * @param strFileName
		 * @param stTock
		 */
		void zacniSerializacijo( const std::string &strFileName, int stTock );
		/**
		 * Konča serializacijo, zapre datoteko.
		 */
		void koncajSerializacijo();
		/**
		 * Serializira eno vejo v datoteko CRS, tako
		 * zapie vse točke veje v izhodno datoteko, pri tem si pomaga s pomiki po world koordinatah, z OpenGL.
		 * @param visina višina veje
		 * @param sirinaSpodaj širina veje spodaj
		 * @param sirinaZgoraj širina veje zgoraj
		 * @param presekSpodaj točke preseka spodaj
		 * @param idVeje zaporedna številka prave veje po G, ne le odseka (za združevanje pri branju CRS)
		 */
		void serializirajVejo( double visina, double sirinaSpodaj, double sirinaZgoraj, Presek *presekSpodaj, int idVeje );

		/**
		 * Serializira list v datoteko CRS,
		 *    stranica, 180 stopinj
		 *   /\
		 *  /  \ stranica*koren iz dva / 2, 45 stopinj
		 *  \  /
		 *   \/
		 *
		 * @param stranica velikost stranice kvadrata, na katerem je narisan list.
		 * @param idVeje zaporedna številka prave veje po G, ne le odseka (za združevanje pri branju CRS)
		 */
		void serializirajList( double stranica, int idVeje );
		/**
		 * Vrne zadnji zgornji presek veje.
		 * @return točke krivulje v presek
		 */
		Presek getPresekZgoraj();
		/**
		 * Določilo, ali trenutno izvažamo ogljišča v datoteko.
		 */
		bool exportingToPoints;

	private:
		/**
		 * Število točk v preseku, stopnja podrobnosti prikaza odsekov vej.
		 */
		int stTock;
		/**
		 * Ogljišča v preseku veje na spodnji strani.
		 */
		Presek presekSpodaj;
		/**
		 * Ogljišča v preseku veje na zgornji strani.
		 */
		Presek presekZgoraj;
		/**
		 * Izhodna datoteka.
		 */
		std::ofstream out;

		/**
		 * Doda eno ogljišče v datoteko CRS: shrani (izvozi) novo točko v datoteko za LWO.
		 * @param tocka ogljišče
		 */
		void addPointToLWO( Tocka &tocka );
		/**
		 * S pomočjo premikov v OpenGL ugotovi novo pozicijo točke v world koordinatah,
		 * matriko moramo kreirati, saj jo spreminjamo.
		 * @param tocka nova točka
		 * @param pomikY pomik do sedaj
		 * @param sirina pomik do sedaj
		 * @param sirinaZ širina preseka
		 * @param kot kot v krogu
		 */
		void getPoint( Tocka &tocka, double pomikY, double sirina, double sirinaZ, double kot );
	};
}
#endif
