/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "tocka.h"

#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

using namespace drevo;

Tocka::~Tocka() {}

Tocka Tocka::operator-( const Tocka &t1) const {
	return Tocka( x -t1.x, y - t1.y, z - t1.z );
} //operator-

Tocka Tocka::operator+( const Tocka &t1 ) const {
	return Tocka( x + t1.x, y + t1.y, z + t1.z );
} //operator+

Tocka Tocka::operator*( const float &skalar ) const {
	return Tocka( x * skalar, y * skalar, z * skalar );
} //operator*

Tocka Tocka::vektorskiProd( const Tocka &v1, const Tocka &v2 ) {
	Tocka produkt;
	///		i j k
	///v1:	1 2 3
	///v2:	4 5 6
	produkt.x = v1.y * v2.z - v1.z * v2.y; ///2*6 - 3*5
	produkt.y = -( v1.x * v2.z - v1.z * v2.x ); ///1*6 - 3*4
	produkt.z = v1.x * v2.y - v1.y * v2.x; ///1*5 - 2*4

	return produkt;
} //vektorskiProd

float Tocka::skalarniProdukt( const Tocka &v2 ) const {
	return
	    x * v2.x +
	    y * v2.y +
	    z * v2.z;
} //Tocka::skalarniProdukt

float Tocka::dolzina() const {
	return sqrt( x * x + y * y + z * z );
} //Tocka::dolzina

void Tocka::normalize() {
	float r = sqrt( pow( x, 2 ) + pow( y, 2 ) + pow( z, 2 ) );
	x = x / r;
	y = y / r;
	z = z / r;
} //normalize
