/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DREVO_TOCKA_H
#define DREVO_TOCKA_H

namespace drevo {
	/**
	  * \brief float predstavitev 3D točke
	  * @author Ales Zamuda <ales.zamuda@uni-mb.si>
	*/
	class Tocka {
	public:
		/**
		 * Ustvari točko in je ne inicializira.
		 * @return
		 */
		Tocka() {}
		;
		/**
		 * Inicializira točko.
		 * @param x x komponenta
		 * @param y y komponenta
		 * @param z z komponenta
		 * @return
		 */
		Tocka(float x, float y, float z) : x(x), y(y), z(z) {}

		~Tocka();

		/**
		 * @brief Odteje dve točki.
		 * Odteje točko po komponentah in vrne novo točko t1-t2.
		 * @param t2 druga točka
		 * @return this - t2
		 */
		Tocka operator-(const Tocka &t2) const;
		/**
		 * Vrne setevek točk (vsoto dveh krajevnih vektorjev)
		 * @param t2 druga točka
		 * @return vsota
		 */
		Tocka operator+(const Tocka &t2) const;
		/**
		 * Vrne krajevni vektor pomnožen s skalarjem.
		 * @param skalar konstanta
		 * @return
		 */
		Tocka operator*(const float &skalar) const;

		/**
		 * Izračuna vektorski produkt dveh točk.
		 * @param v1 prva točka
		 * @param v2 druga točka
		 * @return vektorski produkt v1 x v2
		 */
		static Tocka vektorskiProd(const Tocka &v1, const Tocka &v2);
		/**
		 * Izračuna skalarni produkt dveh vektorjev/točk.
		 * @param v2 druga točka
		 * @return this * v2
		 */
		float skalarniProdukt(const Tocka &v2) const;
		/**
		 * Izračuna dolžino trenutnega vektorja/razdaljo točke od izhodišča.
		 * @return dolžina
		 */
		float dolzina() const;
		/**
		 * Normalizira trenutni vektor.
		 */
		void normalize();

	public:
		/** x komponenta točke */
		float x;
		/** y komponenta točke */
		float y;
		/** z komponenta točke */
		float z;
	};
}
#endif
