/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "camera.h"
#include "terraindata.h"

#include <QtOpenGL>
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

namespace ecosystem {
	namespace render {
		Camera::Camera(QObject *parent, data::CTerrainData * terrainData)
				: QObject(parent), terrainData(terrainData) {
			reset();
		}

		Camera::~Camera() {}

		void Camera::reset() {
			z_pomik=-15.0;
			dol_pomik = -13;
			levo_pomik=0;
			x_zasuk=0;
			y_zasuk=0;

			atSurface=false;
			aboveSurface=true;
			lx=ly=0;
			lz=-1;

			x=1243;
			y=1000;
			z=192;
			alfa=180;
			beta=59;
			gama=0;

			saveCamera();

			x=1300;
			y=250;
			z=790;
			alfa=270;
			beta=0;
			gama=0;
			saveCamera();
		}

		void Camera::setupEcosystemCamera() {
			///novo za ecosystem
			lx = sin(alfa/180.0*M_PI);
			ly=0;
			lz = -cos(alfa/180.0*M_PI);

			if (atSurface || aboveSurface)
				hodiPoTerenu();

			glRotatef(beta, 1,0,0);
			glRotatef(gama, 0,0,1);

			gluLookAt(x, y, z,x+lx, y+ly, z+lz, 0, 1, 0);
		}

		void Camera::hodiPoTerenu() {
			float krpaX=x;
			float krpaZ=z;
			float extent= 25;
			if (krpaX<0 || krpaX>extent*terrainData->stTockX || krpaZ<0 || krpaZ>extent*terrainData->stTockY) {
				if (atSurface) {
					if (x<0)
						krpaX=x=0;
					if (x>extent*(terrainData->stTockX-1))
						krpaX=x=extent*(terrainData->stTockX-1);
					if (z<0)
						krpaZ=z=0;
					if (z>extent*(terrainData->stTockY-1))
						krpaZ=z=extent*(terrainData->stTockY-1);
				} else
					return;
			}
			int x = int(krpaX/extent);
			int z = int(krpaZ/extent);
			/// izravnamo na (mod 25)
			krpaX-=x*extent;
			krpaZ-=z*extent;
			///normaliziramo
			krpaX/=extent;
			krpaZ/=extent;

			int x1 = x+1;
			int z1 = z+1;

			float y1 = terrainData->pTerrainVertexData[x+z*terrainData->stTockX].pos.y;
			float y2 = terrainData->pTerrainVertexData[x1+z*terrainData->stTockX].pos.y;
			float y3 = terrainData->pTerrainVertexData[x+z1*terrainData->stTockX].pos.y;
			float y4 = terrainData->pTerrainVertexData[x1+z1*terrainData->stTockX].pos.y;

			/// glede na bližino kateri od točk, linarno interpoliramo med višinami
			float y=y1+(y2-y1)*krpaX+(y3-y1)*krpaZ;
			if (krpaX+krpaZ>=1.0f) {
				y=y4+(y2-y4)*(1-krpaZ)+(y3-y4)*(1-krpaX);
			}
			///dodam še 2.7 metra (oko..) višina človeka
			y+=1.7f;

			if (atSurface || aboveSurface && this->y<y)
				this->y=y;
		}

		void Camera::saveCamera() {
			shranjeneLokacije.push_back(drevo::Tocka(x,y,z));
			shranjeniZasuki.push_back(drevo::Tocka(alfa,beta,gama));
		}

		void Camera::switchEcosystemCamera() {
			currCamera++;
			if (currCamera>=(int)shranjeneLokacije.size()) {
				currCamera=0;
			}
			drevo::Tocka &lok=shranjeneLokacije[currCamera];
			drevo::Tocka &rot=shranjeniZasuki[currCamera];
			x=lok.x;
			y=lok.y;
			z=lok.z;
			alfa=rot.x;
			beta=rot.y;
			gama=rot.z;
		}
	}
}
