/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ECOSYSTEM_RENDERCAMERA_H
#define ECOSYSTEM_RENDERCAMERA_H

#include <QObject>
#include <vector>
#include "tocka.h"

namespace ecosystem {
	namespace data {
		class CTerrainData;
	}

	namespace render {
		/**
			@author Ales Zamuda <ales.zamuda@uni-mb.si>
		*/
		class Camera : public QObject {
			Q_OBJECT
		public:
			/**
			 * Usrvari nov objekt kamere, resetira parametre na privzete.
			 * @param parent oče objekta
			 * @param terrainData povezava na podatke o terenu, za hojo po njem
			 * @return
			 */
			Camera(QObject *parent, data::CTerrainData * terrainData);
			~Camera();

			/**
			 * V obstoječem OpenGL kontekstu nastavi kamero.
			 */
			void setupEcosystemCamera();
			/**
			 * Resetira parametre kamere na začetne.
			 */
			void reset();
			/**
			 * Prestavi na naslednjo predefinirano kamero.
			 */
			void switchEcosystemCamera();
			/**
			 * Shrani parametre kamere.
			 */
			void saveCamera();

		public:
			/**Single tree: x zasuk
			 */
			float x_zasuk;
			/**Single tree: y zasuk
			 */
			float y_zasuk;
			/**Single tree: z zasuk
			 */
			float z_pomik;
			/**Single tree: pomik dol
			 */
			float dol_pomik;
			/**Single tree: pomik levo
			 */
			float levo_pomik;

			/**Ecosystem: alfa zasuk
			 */
			float alfa;
			/**Ecosystem: beta zasuk
			 */
			float beta;
			/**Ecosystem: gama zasuk
			 */
			float gama;
			/**Ecosystem: premik x.
			 */
			float x;
			/**Ecosystem: premik y.
			 */
			float y;
			/**Ecosystem: premik z.
			 */
			float z;
			/**Ecosystem: smer pogleda x.
			 */
			float lx;
			/**Ecosystem: smer pogleda y.
			 */
			float ly;
			/**Ecosystem: smer pogleda z.
			 */
			float lz;

			/**Ecosystem: kretnica hoje po terenu.
			 */
			bool atSurface;
			/**Ecosystem: kretnica omejitve pogleda nad terenom.
			 */
			bool aboveSurface;

		private:
			/**
			 * Povezava na terenske podatke, potrebna zaradi hoje po terenu.
			 */
			data::CTerrainData * terrainData;

			/**
			 * Spremeni nastavitve kamere tako, da ustrezajo hoji po terenu,
			 * s projekcijo y komponente na teren.
			 */
			void hodiPoTerenu();
			/**
			 * Shranjene lokacije kamere za priklic ob Ctrl+0
			 */
			std::vector<drevo::Tocka> shranjeneLokacije;
			/**
			 * Shranjeni zasuki kamere za priklic ob Ctrl+0
			 */
			std::vector<drevo::Tocka> shranjeniZasuki;
			/**
			 * Številka trenutne kamere iz predefiniranih.
			 */
			int currCamera;
		};
	}
}
#endif
