#include "objectculling.h"
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

using namespace ecosystem::render;

void CObjectCulling::setMatrix(drevo::Matrika &m) {
	this->comboMatrix=m;
	extractPlanesGL();
}//CObjectCulling::setMatrix

void CObjectCulling::extractPlanesGL() {
	/// Left clipping plane
	p_planes[0].a = comboMatrix.a41 + comboMatrix.a11;
	p_planes[0].b = comboMatrix.a42 + comboMatrix.a12;
	p_planes[0].c = comboMatrix.a43 + comboMatrix.a13;
	p_planes[0].d = comboMatrix.a44 + comboMatrix.a14;
	/// Right clipping plane
	p_planes[1].a = comboMatrix.a41 - comboMatrix.a11;
	p_planes[1].b = comboMatrix.a42 - comboMatrix.a12;
	p_planes[1].c = comboMatrix.a43 - comboMatrix.a13;
	p_planes[1].d = comboMatrix.a44 - comboMatrix.a14;
	/// Top clipping plane
	p_planes[2].a = comboMatrix.a41 - comboMatrix.a21;
	p_planes[2].b = comboMatrix.a42 - comboMatrix.a22;
	p_planes[2].c = comboMatrix.a43 - comboMatrix.a23;
	p_planes[2].d = comboMatrix.a44 - comboMatrix.a24;
	/// Bottom clipping plane
	p_planes[3].a = comboMatrix.a41 + comboMatrix.a21;
	p_planes[3].b = comboMatrix.a42 + comboMatrix.a22;
	p_planes[3].c = comboMatrix.a43 + comboMatrix.a23;
	p_planes[3].d = comboMatrix.a44 + comboMatrix.a24;
	/// Near clipping plane
	p_planes[4].a = comboMatrix.a41 + comboMatrix.a31;
	p_planes[4].b = comboMatrix.a42 + comboMatrix.a32;
	p_planes[4].c = comboMatrix.a43 + comboMatrix.a33;
	p_planes[4].d = comboMatrix.a44 + comboMatrix.a34;
	/// Far clipping plane
	p_planes[5].a = comboMatrix.a41 - comboMatrix.a31;
	p_planes[5].b = comboMatrix.a42 - comboMatrix.a32;
	p_planes[5].c = comboMatrix.a43 - comboMatrix.a33;
	p_planes[5].d = comboMatrix.a44 - comboMatrix.a34;


	/// normaliziramo ena�o ravnine (normalni vektor)
	normalizePlane(p_planes[0]);
	normalizePlane(p_planes[1]);
	normalizePlane(p_planes[2]);
	normalizePlane(p_planes[3]);
	normalizePlane(p_planes[4]);
	normalizePlane(p_planes[5]);

	/// clip plane je tista, ki jo nastavi uporabnik,
	/// ne view frustum!!!
	//for (int i = 0; i < 6; i++) {
	//	GLdouble k[4];
	//	glGetClipPlane(GL_CLIP_PLANE0 + i, k);
	//	p_planes[i].a = k[0];
	//	p_planes[i].b = k[1];
	//	p_planes[i].c = k[2];
	//	p_planes[i].d = k[3];
	//}
}

void CObjectCulling::normalizePlane(Plane &plane) {
	float mag;
	mag = (float)sqrt(plane.a * plane.a + plane.b * plane.b + plane.c * plane.c);
	plane.a = plane.a / mag;
	plane.b = plane.b / mag;
	plane.c = plane.c / mag;
	plane.d = plane.d / mag;
}

bool CObjectCulling::containsSphere(const drevo::Tocka& centerObjekta, float radijObjekta) const {
	float razdalja;/// izračunamo razdaljo do vsake od ravnin
	for(int i = 0; i < 6; i++) {
		drevo::Tocka n((float)p_planes[i].a, (float)p_planes[i].b, (float)p_planes[i].c);
		razdalja = n.skalarniProdukt(centerObjekta)+(float)p_planes[i].d;

		if(razdalja < -radijObjekta) /// če je zunaj za keterokoli, je zunaj
			return false;	///če je razdalja manjša od -radijKrogle, smo zunaj

		if((float)fabs(razdalja) < radijObjekta)/// če se vsaj z eno seka, je notri!
			return true;	///če je razdalja med +-radij, se objekt seka z ravnino
	}
	return true;///sicer je cel objekt notri v pogledu
}
