#ifndef ECOSYSTEM_RENDER_OBJECT_CULLING_H
#define ECOSYSTEM_RENDER_OBJECT_CULLING_H

#include "tocka.h"
#include "matrika.h"

namespace ecosystem {
	namespace render {
		/*!
		 * \brief izločanje objektov izven scene
		 *@author Aleš Zamuda (ales.zamuda@uni-mb.si)
		 */
		class CObjectCulling {
		public:
			/**
			 * Shrani matriko pogleda in modela (viewmodel matrix).
			 * @param m matrika pogleda in modela
			 */
			void setMatrix(drevo::Matrika &m);
			/**
			 * izračuna enačbe ravnin v viewing frustum
			 */
			void extractPlanesGL();
			/**
			 * Določi, ali je krogla na dani lokaciji z danim radijem vidna v viewing frustum.
			 * @param centerObjekta dana lokacija krogle
			 * @param radijObjekta radij krogle
			 * @return vidnost okrogleda objekta iz kamere
			 */
			bool containsSphere(const drevo::Tocka& centerObjekta, float radijObjekta) const;

			/*!
			 * Enačba ravnine.
			 */
			struct Plane {
				double a, b, c, d;
			};
		private:
			/**
			 * Normalizira enačbo (koeficiente) normale.
			 * @param plane koeficienti enačbe normale
			 */
			void normalizePlane(Plane &plane);
			/**
			 * Izračuna, ali je dana točka na pravi strani (+) ravnine.
			 * @param plane koeficienti enačbe ravnine
			 * @param pt dana točka
			 * @return ali je točka v + delu polravnine
			 */
			bool pointIsOutOfPlane(const Plane & plane, const drevo::Tocka & pt);

			/*!
			 * Viewmodel OpenGL matrika
			 */
			drevo::Matrika comboMatrix;
			/*!
			 * Enačbe testih ravnin, ki sestavljajo viewing frustum
			 */
			Plane p_planes[6];
		};
	}//namespace render
}//namespace ecosystem
#endif // ECOSYSTEM_RENDER_OBJECT_CULLING_H
