/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "sky.h"

namespace ecosystem {

	namespace render {

		Sky::Sky( drevo::Tocka dimenzije ) : dimenzije( dimenzije ) {
			for ( int i = 0; i < 6; i++ )
				idTexturaSky[ i ] = 0;

			tvoriOgljiscaObzorja();
			tvoriTeksturneKoordinate();
			tvoriPloskveKvadraObzorja();
		}

	Sky::~Sky() {}

		void Sky::setTextureNames( GLuint idTexturaSkyN, GLuint idTexturaSkyS,
		                           GLuint idTexturaSkyE, GLuint idTexturaSkyW,
		                           GLuint idTexturaSkyU, GLuint idTexturaSkyD ) {
			idTexturaSky[ 0 ] = idTexturaSkyN;
			idTexturaSky[ 1 ] = idTexturaSkyS;
			idTexturaSky[ 2 ] = idTexturaSkyE;
			idTexturaSky[ 3 ] = idTexturaSkyW;
			idTexturaSky[ 4 ] = idTexturaSkyU;
			idTexturaSky[ 5 ] = idTexturaSkyD;
		}

		GLuint* Sky::getTextureNames() {
			return idTexturaSky;
		}

		void Sky::tvoriOgljiscaObzorja() {
			float radijObzorja = dimenzije.x;
			/// 12,5 km okoli terena (2500 m * 5 = 12,5 km)
			float r = 5;
			drevo::Tocka radijiObzorja( radijObzorja * r,
			                            radijObzorja * r, radijObzorja * r );

			/// določimo ogljišča obzorja
			ogljiscaObzorja[ 0 ] = drevo::Tocka( -radijiObzorja.x, radijiObzorja.y,
			                                     radijiObzorja.z );
			ogljiscaObzorja[ 1 ] = drevo::Tocka( radijiObzorja.x, radijiObzorja.y,
			                                     radijiObzorja.z );
			ogljiscaObzorja[ 2 ] = drevo::Tocka( radijiObzorja.x, radijiObzorja.y,
			                                     -radijiObzorja.z );
			ogljiscaObzorja[ 3 ] = drevo::Tocka( -radijiObzorja.x, radijiObzorja.y,
			                                     -radijiObzorja.z );

			ogljiscaObzorja[ 4 ] = drevo::Tocka( -radijiObzorja.x, 0,
			                                     radijiObzorja.z );
			ogljiscaObzorja[ 5 ] = drevo::Tocka( radijiObzorja.x, 0,
			                                     radijiObzorja.z );
			ogljiscaObzorja[ 6 ] = drevo::Tocka( radijiObzorja.x, 0,
			                                     -radijiObzorja.z );
			ogljiscaObzorja[ 7 ] = drevo::Tocka( -radijiObzorja.x, 0,
			                                     -radijiObzorja.z );

			/// ker je začetek = (0,0,0), dimenzije podajo vso informacijo o dimenzijah
			drevo::Tocka centerTerena( dimenzije.x / 2, 0, dimenzije.z / 2 );
			for ( int i = 0; i < 8; i++ ) {
				ogljiscaObzorja[ i ] = ogljiscaObzorja[ i ] + centerTerena;
			}
		}

		void Sky::tvoriTeksturneKoordinate() {
			/// pol piksla zaradi antialasa:
			float polPiksla=0.5/256.0f;
			///spodaj levo, spodaj desno, zgoraj desno, zgoraj levo
			teksturneKoordinate[ 0 ] = drevo::Tocka2D( 0+polPiksla, 0+polPiksla );
			teksturneKoordinate[ 1 ] = drevo::Tocka2D( 1-polPiksla, 0+polPiksla );
			teksturneKoordinate[ 2 ] = drevo::Tocka2D( 1-polPiksla, 1-polPiksla );
			teksturneKoordinate[ 3 ] = drevo::Tocka2D( 0+polPiksla, 1-polPiksla );

			float magVoda=5.0f;
			teksturneKoordinateVode[ 0 ] = drevo::Tocka2D( 0+polPiksla, 0+polPiksla );
			teksturneKoordinateVode[ 1 ] = drevo::Tocka2D( (1-polPiksla)*magVoda, 0+polPiksla );
			teksturneKoordinateVode[ 2 ] = drevo::Tocka2D( (1-polPiksla)*magVoda, (1-polPiksla)*magVoda );
			teksturneKoordinateVode[ 3 ] = drevo::Tocka2D( 0+polPiksla, (1-polPiksla)*magVoda );
		}

		void Sky::tvoriPloskveKvadraObzorja() {
			/// sever
			indeksiOgljiscObzorja[ 0 ][ 0 ] = 5;
			indeksiOgljiscObzorja[ 0 ][ 1 ] = 4;
			indeksiOgljiscObzorja[ 0 ][ 2 ] = 0;
			indeksiOgljiscObzorja[ 0 ][ 3 ] = 1;

			///  jug
			indeksiOgljiscObzorja[ 1 ][ 0 ] = 7;
			indeksiOgljiscObzorja[ 1 ][ 1 ] = 6;
			indeksiOgljiscObzorja[ 1 ][ 2 ] = 2;
			indeksiOgljiscObzorja[ 1 ][ 3 ] = 3;

			/// vzhod
			indeksiOgljiscObzorja[ 2 ][ 0 ] = 4;
			indeksiOgljiscObzorja[ 2 ][ 1 ] = 7;
			indeksiOgljiscObzorja[ 2 ][ 2 ] = 3;
			indeksiOgljiscObzorja[ 2 ][ 3 ] = 0;

			/// zahod
			indeksiOgljiscObzorja[ 3 ][ 0 ] = 6;
			indeksiOgljiscObzorja[ 3 ][ 1 ] = 5;
			indeksiOgljiscObzorja[ 3 ][ 2 ] = 1;
			indeksiOgljiscObzorja[ 3 ][ 3 ] = 2;

			/// zgoraj: pogledam na sever in se obrnem gor
			indeksiOgljiscObzorja[ 4 ][ 0 ] = 1;
			indeksiOgljiscObzorja[ 4 ][ 1 ] = 0;
			indeksiOgljiscObzorja[ 4 ][ 2 ] = 3;
			indeksiOgljiscObzorja[ 4 ][ 3 ] = 2;

			/// spodaj: pogledam na sever in se obrnem dol
			indeksiOgljiscObzorja[ 5 ][ 0 ] = 5;
			indeksiOgljiscObzorja[ 5 ][ 1 ] = 6;
			indeksiOgljiscObzorja[ 5 ][ 2 ] = 7;
			indeksiOgljiscObzorja[ 5 ][ 3 ] = 4;
		}

		void Sky::render() {
			glEnable( GL_TEXTURE_2D );
			glDisable(GL_BLEND);
			glColor3f(1,1,1);
			/// kvadrati obzorja:
			for ( int i = 0; i < 6; i++ ) {
				glBindTexture( GL_TEXTURE_2D, idTexturaSky[ i ] );
				glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
				glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
				glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
				glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
				glBegin( GL_QUADS );
				{
					/// ogljišča za kvadrat
					for ( int j = 0; j < 4; j++ ) {
						if (i==5)
							glTexCoord2d( teksturneKoordinateVode[ j ].x, teksturneKoordinateVode[ j ].y );
						else
							glTexCoord2d( teksturneKoordinate[ j ].x, teksturneKoordinate[ j ].y );
						
						drevo::Tocka &t = ogljiscaObzorja[ indeksiOgljiscObzorja[ i ][ j ] ];
						glVertex3f( t.x, t.y, t.z );
					}
				}
				glEnd();
			}
		}
	}
}
