/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ECOSYSTEM_RENDERSKY_H
#define ECOSYSTEM_RENDERSKY_H

#include "tocka.h"
#include "tocka2d.h"
#include <QtOpenGL>

namespace ecosystem {

	namespace render {

		/**
		 * Služi risanju neba, oblakov in vsega obzorja, ki obdaja teren.
		 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
		*/
		class Sky {
		public:
			/**
			 * Ustvari objekt in shrani podane dimenzije.
			 * @param dimenzije dimenzije x,y,z terena
			 */
			Sky(drevo::Tocka dimenzije);

			~Sky();

			/**
			 * Shrani teksture terena, ki so dobljene od zunaj.
			 * Od zunaj so te teksture tudi izbrisane.
			 *
			 * @param idTexturaSkyN tekstura sever
			 * @param idTexturaSkyS tekstura jug
			 * @param idTexturaSkyE tekstura vzhod
			 * @param idTexturaSkyW tekstura zahod
			 * @param idTexturaSkyU tekstura zgoraj
			 * @param idTexturaSkyD tekstura spodaj
			 */
			void setTextureNames(GLuint idTexturaSkyN, GLuint idTexturaSkyS,
								 GLuint idTexturaSkyE, GLuint idTexturaSkyW,
								 GLuint idTexturaSkyU, GLuint idTexturaSkyD);
			/**
			 * Izriše obzorje.
			 */
			void render();

			/**
			 * Vrne imena tekstur OpenGL za sky.
			 * @return imena tekstur v OpenGL
			 */
			GLuint* getTextureNames();

		private:
			/**
			 * Tvori koordinate ogljišč kvadra obzorja.
			 */
			void tvoriOgljiscaObzorja();
			/**
			 * Tvori teksturne koordinate ploskev obzorja
			 */
			void tvoriTeksturneKoordinate();
			/**
			 * Tvori indekse ogljišč za kvadre obzorja
			 */
			void tvoriPloskveKvadraObzorja();
			/**
			 * Dimenzije x,y,z terena.
			 */
			drevo::Tocka dimenzije;
			/**
			 * OpenGL tekstura na severu, jugu, vzhodu, zahodu, zgoraj, spodaj
			 */
			GLuint idTexturaSky[6];
			/**
			 * Koordinate za izris obljišč obzorja
			 */
			drevo::Tocka ogljiscaObzorja[8];
			/**
			 * Koordinate teksture ploskev obzorja:
			 * spodaj levo, spodaj desno, zgoraj desno, zgoraj levo
			 */
			drevo::Tocka2D teksturneKoordinate[4];
			/**
			 * Koordinate teksture ploskve vode
			 * spodaj
			 */
			drevo::Tocka2D teksturneKoordinateVode[4];
			/**
			 * Indeksi ogljišč obzorja, za izris ploskev
			 */
			int indeksiOgljiscObzorja[6][4];
		};

	}

}

#endif
