/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "terrainmarkings.h"
#include "tocka.h"
#include "terraindata.h"
#include "sunfactory.h"

namespace ecosystem {

	namespace render {

		TerrainMarkings::TerrainMarkings(data::CTerrainData *terrainData) : terrainData(terrainData) {
			kvadraticnePloskve = gluNewQuadric();
			gluQuadricDrawStyle( kvadraticnePloskve, GLU_FILL );
			gluQuadricTexture( kvadraticnePloskve, GL_TRUE );
		}

		TerrainMarkings::~TerrainMarkings() {}

		void TerrainMarkings::renderDataOnVertices(float *data, int type) {
			glDisable(GL_BLEND);
			glDisable(GL_TEXTURE_2D);
			glDisable(GL_LIGHTING);
			for (int y=0; y<terrainData->stTockY-1; y++) {
				int row = y*terrainData->stTockX;
				for (int x=0; x<terrainData->stTockX-1; x++) {
					int base = row+x;
					if (data == NULL)
						glBegin(GL_LINES);
					{
						glVertex3f(terrainData->pTerrainVertexData[base].pos.x,
								   terrainData->pTerrainVertexData[base].pos.y,
								   terrainData->pTerrainVertexData[base].pos.z);

						drevo::Tocka value = terrainData->terrainNormals[base];
						if (data != NULL) {
							value = value*data[base];
							switch (type) {
								case 1:
									glColor4f(data[base],0,data[base],0.5f);
									break;
								case 2:
									glColor4f(0,data[base],0,0.5f);
									break;
								case 3:
									glColor4f(data[base],0,0,0.5f);
									break;
								case 4:
									glColor4f(data[base],data[base],0,0.5f);
									break;
							}
						}
						value = value* 50;
						drevo::Tocka sum = terrainData->pTerrainVertexData[base].pos + value;

						glVertex3f(sum.x, sum.y, sum.z);
					}
					glEnd();
					if (data != NULL) {
						glDisable( GL_DEPTH_TEST );
						glPushMatrix(); {
							glTranslatef(terrainData->pTerrainVertexData[base].pos.x,
										 terrainData->pTerrainVertexData[base].pos.y,
										 terrainData->pTerrainVertexData[base].pos.z);
							glRotatef(90,1,0,0);
							glRotatef(45,0,0,1);
							gluDisk( kvadraticnePloskve, 0, 1.42*12.5f*data[base], 4, 1 );
						} glPopMatrix();
						glEnable( GL_DEPTH_TEST );
					}
				}
			}
			glEnable(GL_LIGHTING);
			glEnable(GL_BLEND);
			glEnable(GL_TEXTURE_2D);

			glColor4f(1,1,1,1);
		}//TerrainMarkings::renderDataOnVertices

		// draws pre-calculated terrain normals
		void TerrainMarkings::renderNormals() {
			// no additional factor to multiply the normal
			glColor3f(1,0,0);
			renderDataOnVertices(NULL, 0);
		}

		void TerrainMarkings::renderInclination() {
			renderDataOnVertices(terrainData->terrainInclination, 1);
		}

		void TerrainMarkings::renderMoisture() {
			renderDataOnVertices(terrainData->terrainMoisture, 2);
		}

		void TerrainMarkings::renderSunniness() {
			renderDataOnVertices(terrainData->terrainSunniness, 3);
		}

		void TerrainMarkings::renderWindiness() {
			renderDataOnVertices(terrainData->terrainWindiness, 4);
		}

		void TerrainMarkings::renderLandMarkings() {
			//renderDataOnVertices(terrainData->terrainMoisture);
		}

		void TerrainMarkings::renderSun() {
			for (int i = 0; i < terrainData->getSunFactory()->getNrSunSamples(); i++) {
				drevo::Tocka sunPos=terrainData->getSunFactory()->getSunPositionAtSample(i);
				sunPos.x=(0.5f+sunPos.x)*terrainData->dimen.x;
				sunPos.y=sunPos.y*terrainData->dimen.y;
				sunPos.z=(0.5f+sunPos.z)*terrainData->dimen.z;
				renderSunBall(sunPos);
			}
			glColor3f(1,1,1);
		}

		void TerrainMarkings::renderSunBall(drevo::Tocka sunPos) {
			glPushMatrix(); {
				glColor3f(1,1,0);
				glTranslatef(sunPos.x, sunPos.y, sunPos.z);
				glEnable(GL_TEXTURE_2D);
				glBindTexture( GL_TEXTURE_2D, 1 );
				gluSphere( kvadraticnePloskve, 50, 10, 10);
			} glPopMatrix();
		}

	}

}
