/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ECOSYSTEM_RENDERTERRAINMARKINGS_H
#define ECOSYSTEM_RENDERTERRAINMARKINGS_H

namespace drevo {
	class Tocka;
}
#include <QtOpenGL>
namespace ecosystem {
	namespace data {
		class CTerrainData;
	}
	namespace render {

		/**
		 * Prikaže razne oznake na terenu.
		 *
		 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
		*/
		class TerrainMarkings {
		public:
			/**
			 * Kreira nov objekt in shrani povezavo na podatke o terenu.
			 */
			TerrainMarkings(data::CTerrainData *terrainData);
			/**
			 * Pobriše objekt.
			 * @return
			 */
			~TerrainMarkings();
			/*!
			 * Narie vektorje normal.
			 */
			void renderNormals();
			/*!
			 * Narie vektorje naklona.
			 */
			void renderInclination();
			/*!
			 * Narie vektorje vlažnosti.
			 */
			void renderMoisture();
			/*!
			 * Narie vektorje sončnosti.
			 */
			void renderSunniness();
			/*!
			 * Narie vektorje vetrovnosti.
			 */
			void renderWindiness();
			/*!
			 * Izriše lik, ki označuje vrh, strani neba in kompas.
			 */
			void renderLandMarkings();
			/**
			 * Prikaže kroglo na lokaciji sonca.
			 */
			void renderSun();

		private:
			void renderSunBall(drevo::Tocka sunPos);
			/*!
			 * Splona funkcija za risanje vektorjev, ki so skalirani z neko vrednostjo.
			 * To funkcijo uporabljajo npr. izrisi naklona, padavin, sonca in vetrovnosti.
			 * @param data podatki
			 * @param type tip parametra
			 */
			void renderDataOnVertices(float *data, int type);
			/*!
			 * Narie smer pihanja vetra. Narie le enega izmed snopa vektorjev.
			 */
			void drawWindDirection();
			/**
			 * Kazalec na podatke o terenu.
			 */
			data::CTerrainData *terrainData;
			/**
			 * OpenGL objekt za izris GLU objektov.
			 */
			GLUquadricObj *kvadraticnePloskve;
		};
	}
}
#endif
