#include "ecosystemsimulationfactory.h"
#include "entitiesstatistics.h"

#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

using namespace ecosystem;
using namespace ecosystem::simulationFactory;

void CEcosystemSimulationFactory::buildPanonicVegetation(CEcosystemSimulation& ecosystemSimulation, data::CTerrainData *terrainData, statistics::CEntitiesStatistics *entitiesStatistics) {
	using namespace simulationEntities;
	ecosystemSimulation.rastlinskeVrste.reserve(10);


#define D(X) CVrstaRastlin X(terrainData, entitiesStatistics, id++,

	int id=0;

	/// default vrednosti
	float dNov = 5.0f;
	/// komulacijaPrihajajocihRastlin: jih naj nastane 0 dodatno po prvem koraku
	float dK = 0.0f;
	/// delezRastiBlizuSvojeVrste
	float dCl = 0.80f;

	/// dodatne vrednosti
	//	float brezRasti = 0.0f;
	/***********************************************************************************************
				|stNovihRastlinNaSimulacijskiKorak
				|			|komulacijaPrihajajocihRastlin
				|			|	|casZrelosti
				|			|	|		|casStarosti
				|			|	|		|		|casSigurneSmrti
				|			|	|		|		|		|velikostObZrelosti
				|			|	|		|		|		|		|verjetnostUmrtjaObInferiornosti
				|			|	|		|		|		|		|		|potrebaPoVisini
				|			|	|		|		|		|		|		|		|toleriranostSpremembVisine
				|			|	|		|		|		|		|		|		|		|verjetnostSmrtiObNeugodniVisini
				|			|	|		|		|		|		|		|		|		|		|slabVplivVisineNaHitrostRasti
				|			|	|		|		|		|		|		|		|		|		|		|potrebaPoVlagi
				|			|	|		|		|		|		|		|		|		|		|		|		|tolerantnostSpremembVlage
				|			|	|		|		|		|		|		|		|		|		|		|		|		|verjetnostSmrtiObNeugodniVlagi
				|			|	|		|		|		|		|		|		|		|		|		|		|		|		|slabVplivVlageNaHitrostRasti
				|			|	|		|		|		|		|		|		|		|		|		|		|		|		|		|potrebaPoSoncu
				|			|	|		|		|		|		|		|		|		|		|		|		|		|		|		|		|tolerantnostSpremembSonca
				|			|	|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|verjetnostSmrtiObNeugodnemSoncu
				|			|	|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|slabVplivSoncaNaHitrostRasti
				|			|	|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|potrebaPoVetru
				|			|	|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|tolerantnostSpremembVetra
				|			|	|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|verjetnostSmrtiObNeugodnemVetru
				|			|	|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|slabVplivVetraNaHitrostRasti
				|			|	|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|razmerjeMedVelikostjoInRadijemKrosnje: pri 100 letih bo zasedla radij 5 polj: torej 25 polj
				|			|	|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|delezRastiBlizuSvojeVrste
				|			|	|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|skodljivostNaklona
				|			|	|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|		|vplivMoci
				¡			¡	¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		¡		*/
	D(grm)		dNov,		dK,	3,		10,		25,		0.8f,	0.08f,	.4f,	.4f,	.3f,	0.5f,	0.5f,	0.5f,	0.3f,	0.5f,	0.4f,	0.6f,	0.4f,	0.1f,	0.0f,	1.0f,	0.1f,	0.1f,	1.0f,	dCl,	0.01f,	0.05f);
	D(zalujka)	dNov,		dK,	40,		100,	200,	0.8f,	0.075f,	.01f,	.05f,	.5f,	0.9f,	0.5f,	0.3f,	0.3f,	0.5f,	0.6f,	0.2f,	0.4f,	0.1f,	0.0f,	0.3f,	0.2f,	0.1f,	1.f,	dCl,	0.01f,	0.4f);
	D(javor)	dNov,		dK,	30,		100,	250,	0.8f,	0.07f,	.2f,	.3f,	.3f,	0.5f,	0.6f,	0.3f,	0.3f,	0.5f,	0.65f,	0.15f,	0.4f,	0.1f,	0.0f,	0.4f,	0.22f,	0.1f,	1.5f,	dCl,	0.05f,	0.6f);
	D(smreka)	dNov,		dK,	60,		200,	250,	0.8f,	0.01f,	.55f,	.35f,	.3f,	0.5f,	0.2f,	0.6f,	0.3f,	0.5f,	0.4f,	0.5f,	0.4f,	0.1f,	0.0f,	1.0f,	0.15f,	0.1f,	2.3f,	dCl,	0.01f,	0.6f);
	D(bukev)	dNov,		dK,	28,		150,	250,	0.8f,	0.08f,	.2f,	.2f,	.2f,	0.5f,	0.7f,	0.3f,	0.3f,	0.5f,	0.4f,	0.2f,	0.4f,	0.1f,	0.0f,	0.6f,	0.2f,	0.1f,	2.0f,	dCl,	0.05f,	0.7f);

	/************************************************************************************************/
#undef D
	ecosystemSimulation.rastlinskeVrste.push_back(grm);/// grmovje nekoliko vije
	ecosystemSimulation.rastlinskeVrste.push_back(zalujka);/// zalujke spodaj
	ecosystemSimulation.rastlinskeVrste.push_back(javor);
	ecosystemSimulation.rastlinskeVrste.push_back(smreka);
	ecosystemSimulation.rastlinskeVrste.push_back(bukev);

	calculateNighborhoodExtent(ecosystemSimulation, terrainData);
}//CEcosystemSimulationFactory::buildPanonicVegetation

void CEcosystemSimulationFactory::calculateNighborhoodExtent(CEcosystemSimulation& ecosystemSimulation, data::CTerrainData *terrainData) {
	float radijMax = 0;
	/// radij rastline raste ves čas, do starosti, vse do maksimalnega radija (razmerje med starostjo in radijem)
	for (std::vector<simulationEntities::CVrstaRastlin>::iterator vr = ecosystemSimulation.rastlinskeVrste.begin(); vr != ecosystemSimulation.rastlinskeVrste.end(); vr++) {
		simulationEntities::CVrstaRastlin &rastlinskaVrsta = *vr;
		float vrstaMaxRadij = /*rastlinskaVrsta.casStarosti**/rastlinskaVrsta.razmerjeMedVelikostjoInRadijemKrosnje;
		if (vrstaMaxRadij > radijMax)
			radijMax = vrstaMaxRadij;/// result = max(result, r)
	}
#ifndef min
#define min(x,y) ((x<y)?(x):(y))
#endif
	/// velikost kvadrata vzamemo najmanji kvadrat glede x ali y, saj kasneje elimo BOUNDING circle
	float velikostKvadrata =
	    min((float)terrainData->dimen.x/terrainData->stTockX, (float)terrainData->dimen.x/terrainData->stTockX);
#undef min

	/// za največji dobljen radij izralunamo, v koliko kvadratov se razteza, zaokroženo navzgor, da dobimo bounding circle
	int radijKvadratovEneRastline = (int)ceil(radijMax/velikostKvadrata*2);

	/// tevilo kvadratov, da se dve rastlini stikata v sosečini, je vsota obeh kvadratov
	simulationEntities::CVrstaRastlin::maxStKvadratovNajdemRadijSoseda = radijKvadratovEneRastline/*2*/;
}//CEcosystemSimulationFactory::calculateNighborhoodExtent
