#ifndef ECOSYSTEM_SIMULATION_FACTORY_H
#define ECOSYSTEM_SIMULATION_FACTORY_H

#include "ecosystemsimulation.h"

namespace ecosystem {
	namespace statistics {
		class CEntitiesStatistics;
	}

	/*!
	* \namespace ecosystem::simulationFactory
	* \brief določanje parametrov simulacije
	*
	* Hrani tovarne za določanje parametrov simulacije.
	* Zaenkrat je implementirana le hard-coded varianta, ki ima vse parametre zapečene v kodi.
	* Tu naj bi nastala tudi nova tovarna za branje in shranjevanje parametrov v/iz XML datoteke.
	*/
	namespace simulationFactory {
		/*!
		* \brief tovarna za instanciranje nastavitev simulacije
		*
		* Instancira parametre za panonski svet.
		*
		* \sa CRastlina
		* \sa CVrstaRastlin
		* \sa data::CTerrainData
		*
		*@author Aleš Zamuda (ales.zamuda@uni-mb.si)
		*/
		class CEcosystemSimulationFactory {
		public:
			static void buildPanonicVegetation(CEcosystemSimulation& ecosystemSimulation, data::CTerrainData *terrainData, statistics::CEntitiesStatistics *entitiesStatistics);

		private:
			/*!
			* ENG:
			* calculates maximal map extend to look for a neighbor for any plant
			*/
			static void calculateNighborhoodExtent(CEcosystemSimulation& ecosystemSimulation, data::CTerrainData *terrainData);
		};
	}//namespace simulationFactory
}//namespace ecosystem
#endif //ECOSYSTEM_SIMULATION_FACTORY_H
