/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "heighmapfactory.h"
#include "terraindata.h"
#include <QFile>
#include <QTextStream>
#include <QImage>
#include <QMessageBox>
#include <math.h>

namespace ecosystem {
	namespace data {
		CHeighMapFactory::CHeighMapFactory(data::CTerrainData *terrainData) :terrainData(terrainData) {
		/*
			float velikostKrpe;
			velikostKrpe=10;
			for (int y = 0; y < terrainData->stTockY; y++) {
			for (int x = 0; x < terrainData->stTockX; x++) {
			ogliscaTerena[y*terrainData->stTockX+x].x=(100.0f/terrainData->stTockX)*x*velikostKrpe;
			ogliscaTerena[y*terrainData->stTockX+x].z=(100.0f/terrainData->stTockX)*y*velikostKrpe;
			ogliscaTerena[y*terrainData->stTockX+x].y=10+sin(x/100.0f*50)*5+sin(y/100.0f*3)*10+sin(x/100.0f*3)*10;
		}
		}
						*/

			/*
			QString filenameHeightmap=":/gui/textures/heightmap.png";
			QImage heightmap(filenameHeightmap);
			for (int y = 0; y < terrainData->stTockY; y++) {
			for (int x = 0; x < terrainData->stTockX; x++) {
			ogliscaTerena[y*terrainData->stTockX+x].y=qGray(heightmap.pixel(x,y));
		}
		}
			*/
		}

		CHeighMapFactory::~CHeighMapFactory() {}

		void CHeighMapFactory::loadDmrHeighMap(char*fileName) {
			QFile file(fileName);
			if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
				( void ) QMessageBox::information( 0, QFile::tr( "Error" ),
				QFile::tr( "Could not open DEM terrain file." ), QMessageBox::Ok );
				return;
			}

			QTextStream in(&file);
			float velikostKrpe=25;
			float beriX, beriY, beriZ;
			terrainData->dmrMaxX=-1;
			terrainData->dmrMaxY=-1;
			terrainData->dmrMaxZ=-1;
			terrainData->dmrMinX=-1;
			terrainData->dmrMinY=-1;
			terrainData->dmrMinZ=-1;
			for (int y = 0; y < terrainData->stTockY; y++) {
				for (int x = 0; x < terrainData->stTockX; x++) {
					if (in.atEnd()) {
						( void ) QMessageBox::information( 0, QFile::tr( "Error" ),
						QFile::tr( "Error reading DEM terrain file." ), QMessageBox::Ok );
						return;
					}
					terrainData->ogliscaTerena[y*terrainData->stTockX+x].x=x*velikostKrpe;
					terrainData->ogliscaTerena[y*terrainData->stTockX+x].z=y*velikostKrpe;
					in >> beriX >> beriY;
					in >> beriZ;
					terrainData->ogliscaTerena[y*terrainData->stTockX+x].y=beriZ;

					zgradiBoundingBoxDmr(beriX, beriY, beriZ);
				}
			}
			for (int y = 0; y < terrainData->stTockY; y++) {
				for (int x = 0; x < terrainData->stTockX; x++) {
					terrainData->ogliscaTerena[y*terrainData->stTockX+x].y-=terrainData->dmrMinZ;
				}
			}

			for (int y = 0; y < terrainData->stTockY; y++) {
				for (int x = 0; x < terrainData->stTockX; x++) {
					terrainData->pTerrainVertexData[y*terrainData->stTockX+x].pos=terrainData->ogliscaTerena[y*terrainData->stTockX+x];
				}
			}
			terrainData->visinskaRazlikaNaTerenu=terrainData->dmrMaxZ-terrainData->dmrMinZ;
		}

		void CHeighMapFactory::zgradiBoundingBoxDmr(float beriX, float beriY, float beriZ) {
			if (terrainData->dmrMaxX==-1 || terrainData->dmrMaxX < beriX) terrainData->dmrMaxX = beriX;
			if (terrainData->dmrMaxY==-1 || terrainData->dmrMaxY < beriY) terrainData->dmrMaxY = beriY;
			if (terrainData->dmrMaxZ==-1 || terrainData->dmrMaxZ < beriZ) terrainData->dmrMaxZ = beriZ;
			if (terrainData->dmrMinX==-1 || terrainData->dmrMinX > beriX) terrainData->dmrMinX = beriX;
			if (terrainData->dmrMinY==-1 || terrainData->dmrMinY > beriY) terrainData->dmrMinY = beriY;
			if (terrainData->dmrMinZ==-1 || terrainData->dmrMinZ > beriZ) terrainData->dmrMinZ = beriZ;
		}

		QImage *CHeighMapFactory::produceOrtoFotoTexture() {
			QImage h[]={QImage(":/gui/textures/height1.jpg"),
				QImage(":/gui/textures/height2.jpg"),
				QImage(":/gui/textures/height3.jpg"),
				QImage(":/gui/textures/height4.jpg"),
				QImage(":/gui/textures/height5.jpg")};

			int texResX=128;
			int texResY=texResX;
			QImage *orto = new QImage(texResX, texResY, QImage::Format_RGB32);
			for (int y = 0; y < texResY; y++) {
				for (int x = 0; x < texResX; x++) {
					int dmrX=int(float(x)/float(texResX)*terrainData->stTockX);
					int dmrY=int(float(y)/float(texResY)*terrainData->stTockY);
					float height=float(terrainData->ogliscaTerena[dmrY*terrainData->stTockX+dmrX].y)/
							(terrainData->dmrMaxZ-terrainData->dmrMinZ);

					QRgb pixel=getPixelAtNormalized(float(x)/texResX, float(y)/texResY, height, h);
					int ry=(texResY-y-1);
					int rx=x;//(texResX-x-1);
					orto->setPixel(rx,ry,pixel);
				}
			}
			//orto->save("/home/ales/1.png", "PNG");
			return orto;
		}

		QRgb mixPixels(QRgb pix1, QRgb pix2, QRgb pix3, float h) {
			float h0=1-h;
			int r = int(h*qRed(pix1)+h0*(qRed(pix2)+qRed(pix3))/2)/2;
			int g = int(h*qGreen(pix1)+h0*(qGreen(pix2)+qGreen(pix3))/2)/2;
			int b = int(h*qBlue(pix1)+h0*(qBlue(pix2)+qBlue(pix3))/2)/2;
			return qRgb(r,g,b);
		}

		QRgb CHeighMapFactory::getPixelAtNormalized(float x, float y, float height, QImage images[]) {
			/// določimo številko teksture po y
			const int maxTex=5;
			int texNr=int(height*maxTex);
			if (texNr>=maxTex) texNr=maxTex-1;
			QImage &tex=images[texNr];
			QImage &tex0=images[texNr==0?0:texNr-1];
			QImage &tex1=images[texNr==maxTex-1?texNr:texNr+1];

			int texW=tex.width();
			int texH=tex.height();

			int imX=int(x*texW*15);
			int imY=int(y*texH*15);

			imX %=texW;
			imY %=texH;

			float h=height*5;
			h-=floorf(h);
			h=fabs(h-0.5f)*2;

			return mixPixels(tex.pixel(imX,imY),tex0.pixel(imX,imY),tex1.pixel(imX,imY), h);
		}
	}
}
