/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ECOSYSTEM_DATACHEIGHMAPFACTORY_H
#define ECOSYSTEM_DATACHEIGHMAPFACTORY_H
class QImage;
#include <QRgb>

namespace ecosystem {
	namespace data {
		class CTerrainData;

		/**
		 * Naloži višinske podatke v mapo.
		 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
		*/
		class CHeighMapFactory {
		public:
			/**
			 * Kreira objekt, ne naloži še podatkov.
			 * @param terrainData shrani kazalec na shrambo podatkov
			 */
			CHeighMapFactory(data::CTerrainData *terrainData);
			/**
			 * Pobriše objekt, ne odstrani podatkov iz shrambe.
			 * @return
			 */
			~CHeighMapFactory();
			/**
			 * Iz datoteke DMR naloži podatke. Koordinati X in Y pretvori v X in Z, Z pa tu pomeni Y.
			 * Prav tako zgradi bounding box terena, ki ga shrani v CTerrainData.
			 *
			 * Na koncu še odšteje koordinato minY, tako da je najmanjša točka po višini na 0.
			 *
			 * @param fileName ime datoteke, iz katere naj naložimo mapo
			 */
			void loadDmrHeighMap(char*fileName);

			/**
			 * Iz naloženih višin terena generira teksturo terena
			 * @return slika za teksturo terena
			 */
			QImage *produceOrtoFotoTexture();

		private:
			/**
			 * Zgradi bounding box za brane podatke.
			 * @param beriX prebran X točke
			 * @param beriY prebran Y točke
			 * @param beriZ prebran Z točke
			 */
			void zgradiBoundingBoxDmr(float beriX, float beriY, float beriZ);

			/**
			 * Vrne barvo piksla na podanih koordinatah, v 3D teksturi.
			 * @param x x koordinata teksture
			 * @param y y koordinata teksture
			 * @param height določilo, kateri sloj teksture uporabimo
			 * @param images slike 3D teksture: sloji
			 * @return barva piksla
			 */
			QRgb getPixelAtNormalized(float x, float y, float height, QImage images[]);
			
			/**
			 * Kazalec na podatke o terenu
			 */
			data::CTerrainData *terrainData;
		};
	}
}
#endif
