#include "terraindata.h"
#include "randomgenerator.h"

#include "heighmapfactory.h"
#include "terrainnormalsfactory.h"
#include "inclinationfactory.h"
#include "rainfactory.h"
#include "sunfactory.h"
#include "windfactory.h"

using namespace ecosystem::data;

unsigned long long CTerrainData::randomPozicije3DNovihRastlinI=1;
unsigned long long CTerrainData::randomPozicije3DNovihRastlinJ=2;

CTerrainData::CTerrainData(void) {
	pTerrainVertexData=0;
	terrainNormals=0;

	terrainInclination=0;
	terrainMoisture=0;
	terrainSunniness=0;
	terrainWindiness=0;
	waterHeight=0;
	dataIsReady=false;

	origin=drevo::Tocka(0,0,0);
	dimen=drevo::Tocka(2500,1000,2500);

	visinskaRazlikaNaTerenu=500;

	stTockX=100;
	stTockY=100;
	spawnLupa();

	int resolution = stTockX*stTockY;
	pTerrainVertexData = new TerrainVertexData[resolution];

	ogliscaTerena=new drevo::Tocka[stTockX*stTockY];
	terrainNormals = new drevo::Tocka[resolution];
	terrainInclination = new float[resolution];
	terrainMoisture = new float[resolution];
	terrainWindiness = new float[resolution];
	terrainSunniness = new float[resolution];

	for (int i = 0; i < resolution; i++) {
		terrainInclination[i] = 0;
		terrainMoisture[i] =  0;
		terrainWindiness[i] = 0;
		terrainSunniness[i] = 0;
	}
	heighMapFactory=new CHeighMapFactory(this);
	heighMapFactory->loadDmrHeighMap(":/gui/textures/100x100.dmr");
	ortofotoTexture=heighMapFactory->produceOrtoFotoTexture();
	terrainNormalsFactory=0;
	inclinationFactory=0;
	rainFactory=0;
	sunFactory=0;
	windFactory=0;
}

CTerrainData::~CTerrainData(void) {
	if (heighMapFactory)
		delete heighMapFactory;
	if (terrainNormalsFactory)
		delete terrainNormalsFactory;
	if (inclinationFactory)
		delete inclinationFactory;
	if (rainFactory)
		delete rainFactory;
	if (sunFactory)
		delete sunFactory;
	if (windFactory)
		delete windFactory;

	delete []pTerrainVertexData;
	delete []terrainNormals;

	delete []terrainInclination;
	delete []terrainMoisture;
	delete []terrainSunniness;
	delete []terrainWindiness;
}

void CTerrainData::spawnLupa() {
	const bool lupa=false;
	const bool lupaKvadrat=false;
	const bool lupaBiggerArea=true;

	if (lupa) {
		lupaXmin=53;
		lupaXmax=66;
		lupaYmin=28;
		lupaYmax=32;
	} else if (lupaKvadrat) {
		lupaXmin=49;
		lupaXmax=57;
		lupaYmin=28;
		lupaYmax=36;
	} else if (lupaBiggerArea) {
		lupaXmin=stTockX/4;
		lupaXmax=stTockX/4*3;
		lupaYmin=stTockY/4;
		lupaYmax=stTockY/4*3;
	} else {
		lupaXmin=1;
		lupaXmax=stTockX-1;
		lupaYmin=1;
		lupaYmax=stTockY-1;
	}
}

void CTerrainData::buildTerrainData() {
	buildNonwaterIndexes();

	terrainNormalsFactory = new CTerrainNormalsFactory(*this);
	inclinationFactory = new CInclinationFactory(*this);
	rainFactory = new CRainFactory(*this);
	sunFactory = new CSunFactory(*this);
	windFactory = new CWindFactory(*this);

	terrainNormalsFactory->buildTriangleNormals();
	inclinationFactory->buildInclination();
	rainFactory->buildRain();
	sunFactory->buildSunniness();
	windFactory->buildWindiness();
	dataIsReady=true;
}

void CTerrainData::reloadDMR(char* fileName) {
	if (heighMapFactory)
		delete heighMapFactory;
	if (terrainNormalsFactory)
		delete terrainNormalsFactory;
	if (inclinationFactory)
		delete inclinationFactory;
	if (rainFactory)
		delete rainFactory;
	if (sunFactory)
		delete sunFactory;
	if (windFactory)
		delete windFactory;

	delete []pTerrainVertexData;
	delete []terrainNormals;

	delete []terrainInclination;
	delete []terrainMoisture;
	delete []terrainSunniness;
	delete []terrainWindiness;


	stTockX=100;
	stTockY=100;
	int resolution = stTockX*stTockY;
	pTerrainVertexData = new TerrainVertexData[resolution];

	ogliscaTerena=new drevo::Tocka[stTockX*stTockY];
	terrainNormals = new drevo::Tocka[resolution];
	terrainInclination = new float[resolution];
	terrainMoisture = new float[resolution];
	terrainWindiness = new float[resolution];
	terrainSunniness = new float[resolution];

	heighMapFactory=new CHeighMapFactory(this);
	heighMapFactory->loadDmrHeighMap(fileName);
	ortofotoTexture=heighMapFactory->produceOrtoFotoTexture();

	/// ponovno poženemo simulacije
	buildTerrainData();
}

bool CTerrainData::isDataReady() {
	return dataIsReady;
}

void CTerrainData::buildNonwaterIndexes() {
	nonWaterIndexes.clear();

	float waterLevel = waterHeight*visinskaRazlikaNaTerenu;

	for (int y = lupaYmin; y < lupaYmax; y++)
		for (int x = lupaYmin; x < lupaYmax; x++) {
			int index = x+y*stTockY;
			if ((pTerrainVertexData+index)->pos.y>waterLevel &&
			        (x==stTockX-1 || (pTerrainVertexData+index+1)->pos.y>waterLevel) &&
			        (y==stTockY-1 || (pTerrainVertexData+index+stTockY)->pos.y>waterLevel)) {
				nonWaterIndexes.push_back(index);
			}
		}
}//CTerrainData::buildNonwaterIndexes

drevo::Tocka CTerrainData::vrniNakljucnoPozicijo(drevo::Tocka &A, drevo::Tocka &i, drevo::Tocka &j) {
	///generiramo naključna tevila od 0 do 1
	int resolucija = 1<<12;
	float randI = float(CRandomGenerator::randomGenerator(resolucija, randomPozicije3DNovihRastlinI)) / resolucija;
	float randJ = float(CRandomGenerator::randomGenerator(resolucija, randomPozicije3DNovihRastlinJ)) / resolucija;
	CRandomGenerator::randomGenerator(resolucija, randomPozicije3DNovihRastlinI);///reseed
	
	drevo::Tocka result;
	drevo::Tocka pristejI;
	pristejI = i-A;
	pristejI.x *= randI;
	pristejI.y *= randI;
	pristejI.z *= randI;

	drevo::Tocka pristejJ;
	pristejJ = j-A;
	pristejJ.x *= randJ;
	pristejJ.y *= randJ;
	pristejJ.z *= randJ;
	
	result=A+pristejI+pristejJ;

	return result;
}//CTerrainData::vrniNakljucnoPozicijo

void CTerrainData::dolociPozicijoNoveRastline3D(const ecosystem::Tocka2i &pozicija2D, drevo::Tocka &pozicija3d) {
	drevo::Tocka A, B, C;
	///linearna interpolacija
	int index = pozicija2D.x+pozicija2D.y*stTockX;
	A.x = pTerrainVertexData[index].pos.x;
	A.y = pTerrainVertexData[index].pos.y;
	A.z = pTerrainVertexData[index].pos.z;
	if (pozicija2D.x + 1 < (long)stTockX) {
		B.x = pTerrainVertexData[index+1].pos.x;
		B.y = pTerrainVertexData[index+1].pos.y;
		B.z = pTerrainVertexData[index+1].pos.z;
	} else
		B = A;
	if (pozicija2D.y + 1 < (long)stTockY) {
		C.x = pTerrainVertexData[index+stTockY].pos.x;
		C.y = pTerrainVertexData[index+stTockY].pos.y;
		C.z = pTerrainVertexData[index+stTockY].pos.z;
	} else
		C = A;

	pozicija3d = vrniNakljucnoPozicijo(A, B, C);
}//CTerrainData::dolociPozicijoNoveRastline3D

bool CTerrainData::jeNadVodo(const drevo::Tocka &pozicija3D) {
	return
	    (float)pozicija3D.y
	    /
		visinskaRazlikaNaTerenu

	    >

	    waterHeight;
}//CTerrainData::jeNadVodo

bool CTerrainData::jeNadVodo(const ecosystem::Tocka2i &pozicija2D) {
	//#ifdef LUPA
	if (pozicija2D.y < lupaYmin || pozicija2D.y > lupaYmax ||
	        pozicija2D.x < lupaXmin || pozicija2D.x > lupaXmax)
		return false;
	//#endif
	return
	    (float)pTerrainVertexData[pozicija2D.x+pozicija2D.y*stTockX].pos.y
	    /
		visinskaRazlikaNaTerenu

	    >

	    waterHeight;
}//CTerrainData::jeNadVodo

const CSunFactory* CTerrainData::getSunFactory() const {
	return sunFactory;
}
