#ifndef ECOSYSTEM_DATA_TERRAIN_NORMALS_FACTORY_H
#define ECOSYSTEM_DATA_TERRAIN_NORMALS_FACTORY_H

#include <vector>
#include "terraindata.h"
#include "tocka.h"

namespace ecosystem {
	namespace data {
		/*!
		 * \brief določitev normal krp terena
		 *
		 * Normale se določijo z vektorskim produktom vektorjev na stranicah krp.
		 *@author Aleš Zamuda (ales.zamuda@uni-mb.si)
		 */
		class CTerrainNormalsFactory {
		public:
			/*!
			 * Zgradi objekt in shrani povezavo na shrambo faktorjev
			 * @param terrainData shramba faktorjev
			 */
			CTerrainNormalsFactory(::ecosystem::data::CTerrainData &terrainData);
			/**
			 * Destruktor, ne naredi nič
			 */
			~CTerrainNormalsFactory(void);

			/**
			 * Shrani normale po spodnji razlagi.
			 * Trikotnik iz zvezdic kaže stranice.
			 *
			 * ENG:
			 * Here we extract triangles from the pre-built vertex array.
			 * Vertices are located in a linear array, the array adressing is shown below.
			 * Vertices give the triangles to calculate normals.
			 *	|  base+verXRes*2
			 *
			 *	*             number of vertices per  triangle = 2
			 *	*	*		TRIANGLE(v1[base],v2[base+1],v3[base+vertXRes*2]):
			 *	*   *    *
			 *	^		 ^
			 *	|base	 |base+1
			 */
			void buildTriangleNormals();

		private:
			/*!
			 * Vrne normalizirano normalo iz treh podanih vektorjev, drugi je izhodišče
			 */
			drevo::Tocka getNormal(drevo::Tocka &v1, drevo::Tocka &v2, drevo::Tocka &v3);

			/*!
			 * Povezava na shrambo faktorjev, kamor vpisujemo podakte.
			 */
			CTerrainData &terrainData;
		};
	}//namespace data
}//namespace ecosystem
#endif // ECOSYSTEM_DATA_TERRAIN_NORMALS_FACTORY_H
