#include "ecosystemsimulation.h"
#include "ecosystemsimulationfactory.h"

#include "rastlina.h"
#include "sejalecrastlinnateren.h"
#include "sejaleckjerkolirastlinnateren.h"

#include <cassert>
#include <sstream>
#include <QTime>
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

#include <iostream>
#include <fstream>

using namespace ecosystem;

CEcosystemSimulation *CEcosystemSimulation::myStaticPtr;
int CEcosystemSimulation::casPobijanjaDominiranihRastlin = 0;

CEcosystemSimulation::CEcosystemSimulation( void ) : lifeFactors( &rastlinskeVrste ) {
	simulationStepNr = -1;
	simulationRunning=false;
	rastlinePoTerenu = NULL;
	myStaticPtr = this;
	timer.start();

	///zasejemo random generatorje:
	createSeeds();
} //CEcosystemSimulation::CEcosystemSimulation

CEcosystemSimulation::~CEcosystemSimulation( void ) {
	delete [] rastlinePoTerenu;
} //CEcosystemSimulation::~CEcosystemSimulation

void CEcosystemSimulation::createSeeds() {
	std::vector<unsigned long long> seeds;
	for (int j = 1; j < 13+1; j++)
		seeds.push_back(j);
	int i = 0;
	ecosystem::CSejalecRastlinNaTeren::randomRastiBlizu=seeds[i++];
	ecosystem::CSejalecRastlinNaTeren::randomIzbireBliznjeRastline=seeds[i++];
	ecosystem::CSejalecRastlinNaTeren::randomIzbireStraniRastiObBliznjiRastlini=seeds[i++];
	ecosystem::CSejalecKjerkoliRastlinNaTeren::randomPozicijeNovihRastlinKjerkoli=seeds[i++];

	data::CTerrainData::randomPozicije3DNovihRastlinI=seeds[i++];
	data::CTerrainData::randomPozicije3DNovihRastlinJ=seeds[i++];

	lifeFactors.randomSmrtiZaradiDominiranosti=seeds[i++];
	lifeFactors.randomSmrtiZaradiNeugodnostiVisine=seeds[i++];
	lifeFactors.randomSmrtiZaradiNeugodnostiVlage=seeds[i++];
	lifeFactors.randomSmrtiZaradiNeugodnostiSonca=seeds[i++];
	lifeFactors.randomSmrtiZaradiNeugodnostiVetra=seeds[i++];
	lifeFactors.randomSmrtiZaradiNeugodnostiNaklona=seeds[i++];

	ecosystem::simulationEntities::CRastlina::randomCasSmrti=seeds[i++];
}

statistics::CEntitiesStatistics* CEcosystemSimulation::getEntitiesStatistics() {
	return & entitiesStatistics;
}

void CEcosystemSimulation::setTerrainData( ::ecosystem::data::CTerrainData &terrainData ) {
	this->terrainData = &terrainData;

	// pripravimo mapo rastlin po terenu
	int mapSize = terrainData.stTockX * terrainData.stTockY;
	rastlinePoTerenu = new std::vector<simulationEntities::CRastlina*>[ mapSize ];

	lifeFactors.setRastlinePoTerenu( rastlinePoTerenu );
} //CEcosystemSimulation::setTerrainData

void CEcosystemSimulation::simulateNextStep() {
	zagotoviInicializiranostSimulacije();
	simulationStepNr++; //povečamo uro

	casPobijanjaDominiranihRastlin = 0;
	int casOstalo = timer.elapsed();
	timer.start();
	dodajRastlineVrstam();
	int casDodajal = timer.elapsed();

	timer.start();
	zrastiVseRastlineInDolociMoci();
	int casZrastel = timer.elapsed();

	timer.start();
	pobijRastline();
	int casPobijal = timer.elapsed();
	timer.start();

	saveStatistics( casOstalo, casDodajal, casZrastel, casPobijal );
} //CEcosystemSimulation::simulateNextStep

void CEcosystemSimulation::zagotoviInicializiranostSimulacije() {
	if ( simulationStepNr < 0 ) {
		simulationRunning=true;
		simulationStepNr = 0;
		simulationFactory::CEcosystemSimulationFactory::buildPanonicVegetation( *this, terrainData, &entitiesStatistics );
		terrainData->buildTerrainData();

		najvecjaMoznaStarostRastline = 0;
		for ( int i = 0; i < ( int ) rastlinskeVrste.size(); i++ )
			najvecjaMoznaStarostRastline = najvecjaMoznaStarostRastline > rastlinskeVrste[ i ].casSigurneSmrti ? najvecjaMoznaStarostRastline : rastlinskeVrste[ i ].casSigurneSmrti;
	}
} //CEcosystemSimulation::zagotoviInicializiranostSimulacije

void CEcosystemSimulation::dodajRastlineVrstam() {
	for ( std::vector<simulationEntities::CVrstaRastlin>::iterator v = rastlinskeVrste.begin(); v != rastlinskeVrste.end(); v++ ) {
		simulationEntities::CVrstaRastlin &vrstaRastlin = *v;
		/// več kot jih je, bolj se irijo, vendar s korenom
		vrstaRastlin.dodajKomuliraneRastline( vrstaRastlin.stNovihRastlinNaSimulacijskiKorak * vrstaRastlin.komulacijaSemen, rastlinePoTerenu );
	}
} //CEcosystemSimulation::dodajRastlineVrstam

/// od prej:
/// else:
/// rastlina.umreNaKoraku*=(1-rastlinskaVrsta.verjetnostUmrtjaObDominiranosti);
/// umre eno leto prej, saj je bila sedaj dominirana
void CEcosystemSimulation::zrastiVseRastlineInDolociMoci() {
	for ( std::vector<simulationEntities::CVrstaRastlin>::iterator vr = rastlinskeVrste.begin(); vr != rastlinskeVrste.end(); vr++ ) {
		simulationEntities::CVrstaRastlin &rastlinskaVrsta = *vr;
		/// semena so tu že od nekdaj ali pa jih sedaj prinesejo živali
		rastlinskaVrsta.komulacijaSemen = 1.0f / rastlinskaVrsta.stNovihRastlinNaSimulacijskiKorak;
		for ( simulationEntities::CVrstaRastlin::ziviPrimerkiTeVrsteIter r = rastlinskaVrsta.ziviPrimerkiTeVrste.begin(); r != rastlinskaVrsta.ziviPrimerkiTeVrste.end(); r++ ) {
			simulationEntities::CRastlina &rastlina = *r;
			if ( !rastlina.jeDominirana )
				zrastiRastlino( rastlina, rastlinskaVrsta );
			else
				rastlina.prodornost = rastlina.prodornost - 1.0f > 0 ? rastlina.prodornost - 1.0f : 0;
			rastlina.trenutnaStarost++;
		}
	}
} //CEcosystemSimulation::zrastiVseRastlineInDolociMoci

void CEcosystemSimulation::zrastiRastlino( simulationEntities::CRastlina& rastlina, simulationEntities::CVrstaRastlin &rastlinskaVrsta ) {
	povecajVelikostRastline( rastlina );

	/// približek
	rastlina.trenutniRadij = rastlina.trenutnaVelikost * rastlina.vrstaRastline->razmerjeMedVelikostjoInRadijemKrosnje;
	/// moč
	dolociMocRastline( rastlina );

	/// rastlina daje več semen, ko je stareja, to vrednost bomo pozneje korenili
	if ( rastlina.trenutnaStarost >= rastlina.vrstaRastline->casZrelosti )
		rastlinskaVrsta.komulacijaSemen += rastlina.trenutnaMoc;
} //CEcosystemSimulation::zrastiRastlino

void CEcosystemSimulation::povecajVelikostRastline( simulationEntities::CRastlina& rastlina ) {
	float ugodnostPogojev = izracunajUgodnostPogojev( rastlina );
	if ( rastlina.trenutnaStarost < rastlina.vrstaRastline->casZrelosti ) {
		/// na začetku raste hitreje
		rastlina.trenutnaVelikost +=
		    ( float ) rastlina.vrstaRastline->velikostObZrelosti / rastlina.vrstaRastline->casZrelosti * ugodnostPogojev;
	} else {
		/// sicer raste preostali del do začetka starosti
		if ( rastlina.trenutnaStarost < rastlina.vrstaRastline->casStarosti )
			rastlina.trenutnaVelikost +=
			    ( float ) ( 1 - rastlina.vrstaRastline->velikostObZrelosti )
			    /
			    ( rastlina.vrstaRastline->casStarosti - rastlina.vrstaRastline->casZrelosti )
			    * ugodnostPogojev;
	}
} //CEcosystemSimulation::povecajVelikostRastline

float CEcosystemSimulation::izracunajUgodnostPogojev( simulationEntities::CRastlina & /*rastlina*/ ) {
	//TODO: ugodnost pogojev je vedno 1, izklopil, ne upoštevam vpliva na hitrost rasti
	return 1.0f;
} //CEcosystemSimulation::izracunajUgodnostPogojev

void CEcosystemSimulation::dolociMocRastline( simulationEntities::CRastlina &rastlina ) {
	float maxDozivetaMoc = float( rastlina.vrstaRastline->casSigurneSmrti ) / najvecjaMoznaStarostRastline;
	assert( rastlina.prodornost >= 0 );
	rastlina.trenutnaMoc = rastlina.prodornost * rastlina.trenutnaVelikost * maxDozivetaMoc;
} //CEcosystemSimulation::dolociMocRastline

void CEcosystemSimulation::pobijRastline() {
	for ( std::vector<simulationEntities::CVrstaRastlin>::iterator vr = rastlinskeVrste.begin(); vr != rastlinskeVrste.end(); vr++ ) {
		simulationEntities::CVrstaRastlin &rastlinskaVrsta = *vr;

		rastlinskaVrsta.ziviPrimerkiTeVrste.remove_if( umreSedaj );
	}
} //CEcosystemSimulation::pobijVrste

bool CEcosystemSimulation::umreSedaj( simulationEntities::CRastlina &rastlina ) {
	/// kličemo dejansko preverjanje v CLifeFactors, kasneje bomo le postorili morebitno odstranjevanje
	QTime iskanjeDominiranosti;
	iskanjeDominiranosti.start();
	bool result = myStaticPtr->lifeFactors.umreSedaj( rastlina );
	if ( result ) {
		int terrainIndex = rastlina.pozicijaMap2D.x + rastlina.pozicijaMap2D.y * myStaticPtr->terrainData->stTockX;
		// TODO: v najslabšem primeru O(N^2): vendar ni tak problem: na enem indeksu ni kaj dosti rastlin,
		// pa tudi pri dominiranosti ugotavljamo kar več kot en indeks (tam je območje v mreži, več indeksov)
		std::vector<simulationEntities::CRastlina*> &seznam = myStaticPtr->rastlinePoTerenu[ terrainIndex ];
		int size = ( int ) seznam.size();
		bool odstranil = false;
		for ( int i = 0; i < size; i++ ) {
			simulationEntities::CRastlina &rastlinaSeznam = *seznam[ i ];
			if ( &rastlinaSeznam == &rastlina ) {
				seznam.erase( seznam.begin() + i );
				odstranil = true;
				break;
			}
		}
		//		if (!odstranil)
		//			std::cerr<< "nisem odstranil rastline";
	}
	casPobijanjaDominiranihRastlin += iskanjeDominiranosti.elapsed();
	return result;
} //CEcosystemSimulation::umreSedaj

void CEcosystemSimulation::saveStatistics( int casOstalo, int casDodajal, int casZrastel, int casPobijal ) {
	std::vector<int> stevilaZivihRastlin;
	for ( std::vector<simulationEntities::CVrstaRastlin>::iterator vr = rastlinskeVrste.begin(); vr != rastlinskeVrste.end(); vr++ ) {
		stevilaZivihRastlin.push_back( vr->ziviPrimerkiTeVrste.size() );
	}
	entitiesStatistics.addLiveSpecies( stevilaZivihRastlin );

	entitiesStatistics.saveSimulationRutinesTimings( simulationStepNr, casOstalo, casDodajal, casZrastel, casPobijal, casPobijanjaDominiranihRastlin );
}

void CEcosystemSimulation::saveSimulation(const std::string &fileName) {
	/// shrani lastnosti rastlin
	std::ofstream f(fileName.c_str());

	f << simulationStepNr <<std::endl;
	f << int(rastlinskeVrste.size()) << std::endl;
	for ( std::vector<simulationEntities::CVrstaRastlin>::iterator vr = rastlinskeVrste.begin(); vr != rastlinskeVrste.end(); vr++ ) {
		simulationEntities::CVrstaRastlin &rastlinskaVrsta = *vr;
		f << int(rastlinskaVrsta.ziviPrimerkiTeVrste.size()) << std::endl;

		if ( ( int ) rastlinskaVrsta.ziviPrimerkiTeVrste.size() == 0 )
			continue;

		for ( simulationEntities::CVrstaRastlin::ziviPrimerkiTeVrsteIter r = rastlinskaVrsta.ziviPrimerkiTeVrste.begin();
		        r != rastlinskaVrsta.ziviPrimerkiTeVrste.end(); r++ ) {
			simulationEntities::CRastlina &rastlina = *r;
			f << rastlina.pozicijaMap2D.x << "\t" << rastlina.pozicijaMap2D.y << "\t";
			f << rastlina.pozicija3D.x << "\t" << rastlina.pozicija3D.y << "\t" << rastlina.pozicija3D.z <<"\t";

			f << rastlina.id << "\t";
			f << rastlina.trenutnaStarost<< "\t";
			f << rastlina.trenutnaVelikost<< "\t";
			f << rastlina.trenutniRadij << "\t";
			f << rastlina.prodornost <<"\t";
			f << rastlina.trenutnaMoc <<"\t";

			f << rastlina.strminaTukaj <<"\t";
			f << rastlina.vlaznostTukaj <<"\t";
			f << rastlina.sonceTukaj <<"\t";
			f << rastlina.veterTukaj <<"\t";
			f << rastlina.visinaTukajNormNadVodo <<"\t";
			f << rastlina.umreNaKoraku <<std::endl;
		}
	}

	/// shrani komulacije rastlinskih vrst
	for ( std::vector<simulationEntities::CVrstaRastlin>::iterator vr = rastlinskeVrste.begin(); vr != rastlinskeVrste.end(); vr++ ) {
		f << vr->komulacijaPrihajajocihRastlin << "\t";
	}

	/// shrani semena random generatorjev
	f << ecosystem::CSejalecRastlinNaTeren::randomRastiBlizu << "\t";
	f << ecosystem::CSejalecRastlinNaTeren::randomIzbireBliznjeRastline << "\t";
	f << ecosystem::CSejalecRastlinNaTeren::randomIzbireStraniRastiObBliznjiRastlini << "\t";
	f << ecosystem::CSejalecKjerkoliRastlinNaTeren::randomPozicijeNovihRastlinKjerkoli << "\t";

	f << data::CTerrainData::randomPozicije3DNovihRastlinI << "\t";
	f << data::CTerrainData::randomPozicije3DNovihRastlinJ << "\t";

	f << lifeFactors.randomSmrtiZaradiDominiranosti << "\t";
	f << lifeFactors.randomSmrtiZaradiNeugodnostiVisine << "\t";
	f << lifeFactors.randomSmrtiZaradiNeugodnostiVlage << "\t";
	f << lifeFactors.randomSmrtiZaradiNeugodnostiSonca << "\t";
	f << lifeFactors.randomSmrtiZaradiNeugodnostiVetra << "\t";
	f << lifeFactors.randomSmrtiZaradiNeugodnostiNaklona << "\t";

	f << ecosystem::simulationEntities::CRastlina::randomCasSmrti << "\t"<<std::endl;

	///shrani nastavitve lupe
	f << terrainData->lupaXmin<<"\t";
	f << terrainData->lupaXmax<<"\t";
	f << terrainData->lupaYmin<<"\t";
	f << terrainData->lupaYmax<<std::endl;

	std::cout << "Simulation @ step " << simulationStepNr << "successfouly saved"<<std::endl;
}


void CEcosystemSimulation::loadSimulation(const std::string &filename) {
	/// naloži lastnosti rastlin
	std::ifstream f(filename.c_str());

	int newSimmStep=0;
	f >> newSimmStep;
	int sz=0;
	f >> sz;

	if (sz!=(int)rastlinskeVrste.size()) {
		std::cerr << "Number of species size mismatch!"<<std::endl;
		return;
	}
	simulationStepNr=newSimmStep;
	int stVsehRastlin=0;
	for ( std::vector<simulationEntities::CVrstaRastlin>::iterator vr = rastlinskeVrste.begin(); vr != rastlinskeVrste.end(); vr++ ) {
		simulationEntities::CVrstaRastlin &rastlinskaVrsta = *vr;
		int nrZivih=0;
		f >> nrZivih;
		stVsehRastlin+=nrZivih;
		rastlinskaVrsta.ziviPrimerkiTeVrste.clear();

		if ( nrZivih==0 )
			continue;

		for (int i = 0; i < nrZivih; i++) {
			Tocka2i pozicija2d;
			drevo::Tocka pozicija3d;
			f >> pozicija2d.x  >> pozicija2d.y ;
			f >> pozicija3d.x  >> pozicija3d.y  >> pozicija3d.z;
			simulationEntities::CRastlina rastlina(rastlinskaVrsta, pozicija2d, pozicija3d);

			f >> rastlina.id;
			f >> rastlina.trenutnaStarost;
			f >> rastlina.trenutnaVelikost;
			f >> rastlina.trenutniRadij ;
			f >> rastlina.prodornost;
			f >> rastlina.trenutnaMoc;

			f >> rastlina.strminaTukaj;
			f >> rastlina.vlaznostTukaj;
			f >> rastlina.sonceTukaj;
			f >> rastlina.veterTukaj;
			f >> rastlina.visinaTukajNormNadVodo;
			f >> rastlina.umreNaKoraku;

			rastlinskaVrsta.ziviPrimerkiTeVrste.push_back(rastlina);
		}
	}

	/// naloži komulacije rastlinskih vrst
	for ( std::vector<simulationEntities::CVrstaRastlin>::iterator vr = rastlinskeVrste.begin(); vr != rastlinskeVrste.end(); vr++ ) {
		f >> vr->komulacijaPrihajajocihRastlin;
	}

	/// naloži semena random generatorjev
	f >> ecosystem::CSejalecRastlinNaTeren::randomRastiBlizu;
	f >> ecosystem::CSejalecRastlinNaTeren::randomIzbireBliznjeRastline;
	f >> ecosystem::CSejalecRastlinNaTeren::randomIzbireStraniRastiObBliznjiRastlini;
	f >> ecosystem::CSejalecKjerkoliRastlinNaTeren::randomPozicijeNovihRastlinKjerkoli;

	f >> data::CTerrainData::randomPozicije3DNovihRastlinI;
	f >> data::CTerrainData::randomPozicije3DNovihRastlinJ;

	f >> lifeFactors.randomSmrtiZaradiDominiranosti;
	f >> lifeFactors.randomSmrtiZaradiNeugodnostiVisine;
	f >> lifeFactors.randomSmrtiZaradiNeugodnostiVlage;
	f >> lifeFactors.randomSmrtiZaradiNeugodnostiSonca;
	f >> lifeFactors.randomSmrtiZaradiNeugodnostiVetra;
	f >> lifeFactors.randomSmrtiZaradiNeugodnostiNaklona;

	f >> ecosystem::simulationEntities::CRastlina::randomCasSmrti;

	f >> terrainData->lupaXmin;
	f >> terrainData->lupaXmax;
	f >> terrainData->lupaYmin;
	f >> terrainData->lupaYmax;

	/// tvorimo pointerje živih rastlin po terenu kot id vrste in id rastline
	for (int i = 0; i < terrainData->stTockX*terrainData->stTockY; i++)
		rastlinePoTerenu[i].clear();
	for ( std::vector<simulationEntities::CVrstaRastlin>::iterator vr = rastlinskeVrste.begin(); vr != rastlinskeVrste.end(); vr++ ) {
		simulationEntities::CVrstaRastlin &rastlinskaVrsta = *vr;
		for ( simulationEntities::CVrstaRastlin::ziviPrimerkiTeVrsteIter r = rastlinskaVrsta.ziviPrimerkiTeVrste.begin();
		        r != rastlinskaVrsta.ziviPrimerkiTeVrste.end(); r++ ) {
			simulationEntities::CRastlina &rastlina = *r;
			rastlinePoTerenu[rastlina.pozicijaMap2D.x +rastlina.pozicijaMap2D.y*terrainData->stTockX].push_back(&rastlina);
		}
	}

	std::cout << "Simulation @ step " << simulationStepNr << " with "
	<< stVsehRastlin << " plants successfouly loaded"<<std::endl;
	saveStatistics( 0, 0, 0, 0 );
}


std::vector<unsigned long long> CEcosystemSimulation::getRandomGeneratorSeeds() {
	std::vector<unsigned long long> rands;
	rands.push_back(ecosystem::CSejalecRastlinNaTeren::randomRastiBlizu);
	rands.push_back(ecosystem::CSejalecRastlinNaTeren::randomIzbireBliznjeRastline);
	rands.push_back(ecosystem::CSejalecRastlinNaTeren::randomIzbireStraniRastiObBliznjiRastlini);
	rands.push_back(ecosystem::CSejalecKjerkoliRastlinNaTeren::randomPozicijeNovihRastlinKjerkoli);

	rands.push_back(data::CTerrainData::randomPozicije3DNovihRastlinI);
	rands.push_back(data::CTerrainData::randomPozicije3DNovihRastlinJ);

	rands.push_back(lifeFactors.randomSmrtiZaradiDominiranosti);
	rands.push_back(lifeFactors.randomSmrtiZaradiNeugodnostiVisine);
	rands.push_back(lifeFactors.randomSmrtiZaradiNeugodnostiVlage);
	rands.push_back(lifeFactors.randomSmrtiZaradiNeugodnostiSonca);
	rands.push_back(lifeFactors.randomSmrtiZaradiNeugodnostiVetra);
	rands.push_back(lifeFactors.randomSmrtiZaradiNeugodnostiNaklona);

	rands.push_back(ecosystem::simulationEntities::CRastlina::randomCasSmrti);
	return rands;
}

void CEcosystemSimulation::setRandomGeneratorSeeds(const std::vector<unsigned long long>& seeds) {
	int i = 0;
	ecosystem::CSejalecRastlinNaTeren::randomRastiBlizu=seeds[i++];
	ecosystem::CSejalecRastlinNaTeren::randomIzbireBliznjeRastline=seeds[i++];
	ecosystem::CSejalecRastlinNaTeren::randomIzbireStraniRastiObBliznjiRastlini=seeds[i++];
	ecosystem::CSejalecKjerkoliRastlinNaTeren::randomPozicijeNovihRastlinKjerkoli=seeds[i++];

	data::CTerrainData::randomPozicije3DNovihRastlinI=seeds[i++];
	data::CTerrainData::randomPozicije3DNovihRastlinJ=seeds[i++];

	lifeFactors.randomSmrtiZaradiDominiranosti=seeds[i++];
	lifeFactors.randomSmrtiZaradiNeugodnostiVisine=seeds[i++];
	lifeFactors.randomSmrtiZaradiNeugodnostiVlage=seeds[i++];
	lifeFactors.randomSmrtiZaradiNeugodnostiSonca=seeds[i++];
	lifeFactors.randomSmrtiZaradiNeugodnostiVetra=seeds[i++];
	lifeFactors.randomSmrtiZaradiNeugodnostiNaklona=seeds[i++];

	ecosystem::simulationEntities::CRastlina::randomCasSmrti=seeds[i++];
}
