#ifndef ECOSYSTEM_SIMULATION_ENTITIES_POGOJ_H
#define ECOSYSTEM_SIMULATION_ENTITIES_POGOJ_H

namespace ecosystem {
	namespace simulationEntities {
		/*!
		* @brief zahtevan pogoj za faktor terena
		*
		* Izračun zadostnosti pogoja iz potrebe in tolerirane spremembe.
		* Določa tudi verjetnost smrti ob neugodnem pogoju in slab vpliv na rast
		* ob neugodnosti pogoja.
		*@author Aleš Zamuda (ales.zamuda@uni-mb.si)
		*/
		class CPogoj {
		public:
			CPogoj() {}
			/**
			 * Kreira pogoj, shrani model za izračunavanje tega pogoja.
			 * @param potreba optimalna vrednost parametra
			 * @param toleriranaSprememba tolerirano odstopenje od optimalne (potrebne) vrednosti
			 * @param verjetnostSmrtiObNeugodnosti vetjetnost, da ob neugodnosti (prevelikem odstopanju) umre na vsakem koraku
			 * @param slabVplivNaHitrostRasti slab vpliv (odstotki), ki ga ima pogoj v vsakem koraku, če je pogoj neugoden
			 * @return
			 */
			CPogoj(float potreba,
			       float toleriranaSprememba,
			       float verjetnostSmrtiObNeugodnosti,
			       float slabVplivNaHitrostRasti);

			/*!
			* Vrne, ali rastlina umre v tem koraku zaradi neugodnosti pogoja
			* @param randomSmrtiZaradiNeugodnegaPogoja random generator seed
			* @param pogojTukaj pogoj tukaj pri rastlini
			* @return izpolnjenost pogoja
			*/
			bool umreZaradiNeugodnegaPogoja(unsigned long long &randomSmrtiZaradiNeugodnegaPogoja, float pogojTukaj);

			/*!
			* vrne ugodnost faktorja: 1, če je OK, sicer proti 0, če je pogoj neugoden
			* @param pogojTukaj pogoj tukaj
			* @return ugodnost
			*/
			float getFaktorRasti(float pogojTukaj);

		private:
			/**
			 * Optimalna vrednost parametra.
			 */
			float potreba;
			/**
			 * Tolerirano odstopenje od optimalne (potrebne) vrednosti
			 */
			float toleriranaSprememba;
			/**
			 * vetjetnost, da ob neugodnosti (prevelikem odstopanju) umre na vsakem koraku
			 */
			float verjetnostSmrtiObNeugodnosti;
			/**
			 * Slab vpliv (odstotki), ki ga ima pogoj v vsakem koraku, če je pogoj neugoden
			 */
			float slabVplivNaHitrostRasti;
		};
	}//namespace simulationEntities
}//namespace ecosystem
#endif
