#include "rastlina.h"
#include "randomgenerator.h"
#include "terraindata.h"
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

using namespace ecosystem::simulationEntities;

unsigned long long ecosystem::simulationEntities::CRastlina::randomCasSmrti = 1;
int ecosystem::simulationEntities::CRastlina::lastID = 0;

CRastlina::CRastlina(CVrstaRastlin &vrstaRastlin, const ecosystem::Tocka2i &pozicija2d, const drevo::Tocka &pozicija3d) : vrstaRastline(&vrstaRastlin) {
	this->pozicijaMap2D = pozicija2d;
	this->pozicija3D = pozicija3d;

	trenutnaStarost = 0;
	///velikost povečamo takoj v sledeči zrastiVseRastlineInDolociMoci
	trenutnaVelikost = 0;
	trenutniRadij = 0;
	vlaznostTukaj = dolociVlaznostNoveRastline(pozicijaMap2D);
	dolociSmrt();
	///velikost=0->moč=0
	trenutnaMoc = 0;

	visinaTukajNormNadVodo = dolociVisinoTukajNadVodoNormalizirano();
	strminaTukaj = getStrminaTerena(pozicijaMap2D);
	int index = pozicijaMap2D.x+pozicijaMap2D.y*vrstaRastline->terrainData->stTockX;
	sonceTukaj = vrstaRastlin.terrainData->terrainSunniness[index];
	veterTukaj = vrstaRastlin.terrainData->terrainWindiness[index];

	jeDominirana = false;
	dolociProdornostRastline();

	lastID++;
	id = lastID;
}//CRastlina::CRastlina

void CRastlina::dolociProdornostRastline() {
#ifndef max
#define max(x,y) ((x>y)?(x):(y))
#define min(x,y) ((x<y)?(x):(y))
#endif
	float vlagaOK = min(1.0f, max(0, vrstaRastline->vlaga.getFaktorRasti(vlaznostTukaj)));
	float sonceOK = min(1.0f, max(0, vrstaRastline->sonce.getFaktorRasti(sonceTukaj)));
	float veterOK = min(1.0f, max(0, vrstaRastline->veter.getFaktorRasti(veterTukaj)));
	float visinaOK = min(1.0f, max(0, vrstaRastline->visina.getFaktorRasti(visinaTukajNormNadVodo)));
	float strminaOK = 1.0f - pow(vrstaRastline->skodljivostNaklona*strminaTukaj, 2);
#undef min
#undef max

	prodornost = (vlagaOK*sonceOK*veterOK*visinaOK*strminaOK);
	return;
}//CRastlina::dolociProdornostRastline

float CRastlina::dolociVlaznostNoveRastline(ecosystem::Tocka2i &pozicijaMap2D) {
	int index = pozicijaMap2D.x+pozicijaMap2D.y*vrstaRastline->terrainData->stTockX;
	/// normala kaže ven, ok
	return vrstaRastline->terrainData->terrainMoisture[index];
}//CRastlina::dolociVlaznostNoveRastline

float CRastlina::dolociVisinoTukajNadVodoNormalizirano() {
	float visinaNorm =
			(float)pozicija3D.y*vrstaRastline->terrainData->visinskaRazlikaNaTerenu;
	float visinaDeNormNadVodo = visinaNorm-vrstaRastline->terrainData->waterHeight;
	float visinaNormNadVodo = visinaDeNormNadVodo/(1-vrstaRastline->terrainData->waterHeight);

	return visinaNormNadVodo;
}//CRastlina::dolociVisinoTukajNadVodoNormalizirano

void CRastlina::dolociSmrt() {
	this->umreNaKoraku = float(vrstaRastline->casStarosti+
	                           CRandomGenerator::randomGenerator(vrstaRastline->casSigurneSmrti-vrstaRastline->casStarosti, randomCasSmrti));
}//CRastlina::dolociSmrt

float CRastlina::getStrminaTerena(ecosystem::Tocka2i &pozicijaMap2D) {
	int index = pozicijaMap2D.x+pozicijaMap2D.y*vrstaRastline->terrainData->stTockX;
	/// normala kaže ven, ok
	return vrstaRastline->terrainData->terrainInclination[index];
}//CRastlina::getStrminaTerena

bool CRastlina::presekaRadijRastline(CRastlina &rastlinaSovraznik) {
	/// se sekata? razdalja=pitagora
	float d = sqrt(
	              pow((this->pozicija3D.x - rastlinaSovraznik.pozicija3D.x), 2) +
	              pow((this->pozicija3D.y - rastlinaSovraznik.pozicija3D.y), 2) +
	              pow((this->pozicija3D.z - rastlinaSovraznik.pozicija3D.z), 2));
	if (d < this->trenutniRadij + rastlinaSovraznik.trenutniRadij) {
		return true;
	}

	return false;
}//CRastlina::presekaRadijRastline
