#include "vrstarastlin.h"
#include "sejalecrastlinnateren.h"
#include "terraindata.h"
#include "tocka.h"
#include "tocka2i.h"

#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

using namespace ecosystem::simulationEntities;

int CVrstaRastlin::maxStKvadratovNajdemRadijSoseda=0;

CVrstaRastlin::CVrstaRastlin(ecosystem::data::CTerrainData *terrainData,
                             ecosystem::statistics::CEntitiesStatistics *entitiesStatistics,
                             int tip,
                             float stNovihRastlinNaSimulacijskiKorak,
                             float komulacijaPrihajajocihRastlin,

                             int casZrelosti,
                             int casStarosti,
                             int casSigurneSmrti,

                             float velikostObZrelosti,
                             float verjetnostUmrtjaObDominiranosti,

                             float potrebaPoVisini,
                             float tolerantnostSpremembVisine,
                             float verjetnostSmrtiObNeugodniVisini,
                             float slabVplivVisineNaHitrostRasti,

                             float potrebaPoVlagi,
                             float tolerantnostSpremembVlage,
                             float verjetnostSmrtiObNeugodniVlagi,
                             float slabVplivVlageNaHitrostRasti,

                             float potrebaPoSoncu,
                             float tolerantnostSpremembSonca,
                             float verjetnostSmrtiObNeugodnemSoncu,
                             float slabVplivSoncaNaHitrostRasti,

                             float potrebaPoVetru,
                             float tolerantnostSpremembVetra,
                             float verjetnostSmrtiObNeugodnemVetru,
                             float slabVplivVetraNaHitrostRasti,

                             float razmerjeMedVelikostjoInRadijemKrosnje,
                             float delezRastiBlizuSvojeVrste,
                             float skodljivostNaklona,
                             float vplivMoci) : entitiesStatistics(entitiesStatistics) {
	this->terrainData = terrainData;
	this->tip = tip;
	this->stNovihRastlinNaSimulacijskiKorak = stNovihRastlinNaSimulacijskiKorak;
	this->komulacijaPrihajajocihRastlin = komulacijaPrihajajocihRastlin;

	this->casZrelosti = casZrelosti;
	this->casStarosti = casStarosti;
	this->casSigurneSmrti = casSigurneSmrti;

	this->velikostObZrelosti = velikostObZrelosti;
	this->verjetnostUmrtjaObDominiranosti = verjetnostUmrtjaObDominiranosti;

	this->visina = CPogoj(potrebaPoVisini, tolerantnostSpremembVisine, verjetnostSmrtiObNeugodniVisini, slabVplivVisineNaHitrostRasti);
	this->vlaga = CPogoj(potrebaPoVlagi, tolerantnostSpremembVlage, verjetnostSmrtiObNeugodniVlagi, slabVplivVlageNaHitrostRasti);
	this->sonce = CPogoj(potrebaPoSoncu, tolerantnostSpremembSonca, verjetnostSmrtiObNeugodnemSoncu, slabVplivSoncaNaHitrostRasti);
	this->veter = CPogoj(potrebaPoVetru, tolerantnostSpremembVetra, verjetnostSmrtiObNeugodnemVetru, slabVplivVetraNaHitrostRasti);

	this->razmerjeMedVelikostjoInRadijemKrosnje = razmerjeMedVelikostjoInRadijemKrosnje;
	this->delezRastiBlizuSvojeVrste = delezRastiBlizuSvojeVrste;
	this->skodljivostNaklona = skodljivostNaklona;

	this->vplivMoci = vplivMoci;

	this->komulacijaSemen = 1;
}

void CVrstaRastlin::dodajKomuliraneRastline(float dodanaKomulacija, std::vector<CRastlina*> *rastlinePoTerenu) {
	/// to je onemogočeno, za skalabilnost dodaj:
	//if (dodanaKomulacija>10) dodanaKomulacija=10;

	komulacijaPrihajajocihRastlin += dodanaKomulacija;

	/// dodamo nekaj rastlin
	std::vector<Tocka2i> pozicije2d;
	std::vector<drevo::Tocka> pozicije3d;
	int res = (int)ceil(komulacijaPrihajajocihRastlin);
	pozicije2d.reserve(res);
	pozicije3d.reserve(res);

	CSejalecRastlinNaTeren sejalecRastlinNaTeren(*terrainData, pozicije2d, pozicije3d, *this);
	sejalecRastlinNaTeren.dolociPozicijeRastlin();

	int stNovih = (int)pozicije2d.size();
	for (int i = 0; i < stNovih; i++)
		dodajNovPrimerek(rastlinePoTerenu, pozicije2d[i], pozicije3d[i]);
}//CVrstaRastlin::dodajKomuliraneRastline

void CVrstaRastlin::dodajNovPrimerek(std::vector<CRastlina*> *rastlinePoTerenu, const ecosystem::Tocka2i &pozicija2d, const drevo::Tocka &pozicija3d) {
	CRastlina rastlina(*this, pozicija2d, pozicija3d);
	/// sedaj so vsi uspešno dodani.
	ziviPrimerkiTeVrste.push_back(rastlina);
	CRastlina &rastlinaNotri = ziviPrimerkiTeVrste.back();

	int terrainIndex = rastlinaNotri.pozicijaMap2D.x + rastlinaNotri.pozicijaMap2D.y * terrainData->stTockX;
	rastlinePoTerenu[terrainIndex].push_back(&rastlinaNotri);
}//CVrstaRastlin::oziviNovoRastlino
