#include "randomgenerator.h"
#include "lifefactors.h"
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

using namespace ecosystem;

LifeFactors::LifeFactors(std::vector<simulationEntities::CVrstaRastlin> *rastlinskeVrste) {
	this->rastlinskeVrste = rastlinskeVrste;
	/// randomSmrtiZaradiStarosti = 1; // zamenjan z randomCasSmrti
	randomSmrtiZaradiDominiranosti = 1;
	randomSmrtiZaradiNeugodnostiVisine = 1;
	randomSmrtiZaradiNeugodnostiVlage = 1;
	randomSmrtiZaradiNeugodnostiSonca = 1;
	randomSmrtiZaradiNeugodnostiVetra = 1;
	randomSmrtiZaradiNeugodnostiNaklona = 1;
}

void LifeFactors::setRastlinePoTerenu(std::vector<simulationEntities::CRastlina*> *rastlinePoTerenu) {
	this->rastlinePoTerenu = rastlinePoTerenu;
}

bool LifeFactors::umreSedaj(simulationEntities::CRastlina &rastlina) {
	bool starost, dominiranost;
	starost = umreZaradiStarosti(rastlina) || umreZaradiSibkosti(rastlina);
	umreZaradiVisine(rastlina) ||
	umreZaradiNeugodneVlage(rastlina) ||
	umreZaradiNeugodnegaSonca(rastlina) ||
	umreZaradiNeugodnegaVetra(rastlina) ||

	umreZaradiStrmine_oz_Prsti_oz_kislostiTal_Ph(rastlina);

	if (starost)
		rastlina.vrstaRastline->entitiesStatistics->dodajSmrtOstalo();
	else {
		dominiranost = umreZaradiDominiranosti(rastlina);
		if (dominiranost)
			rastlina.vrstaRastline->entitiesStatistics->dodajSmrtZaradiDominiranosti();
	}

	bool result = starost || dominiranost;
	//if (result) {
	//	std::cout<<"Umre %i\t@%i\n"<<rastlina.id<<rastlina.pozicijaMap2D.x+rastlina.pozicijaMap2D.y*101;
	//}
	return result;
}

bool LifeFactors::umreZaradiStarosti(simulationEntities::CRastlina &rastlina) {
	return rastlina.trenutnaStarost >= rastlina.umreNaKoraku;
}//LifeFactors::umreZaradiStarosti

//@bug - konstanta neparametrična
bool LifeFactors::umreZaradiSibkosti(simulationEntities::CRastlina &rastlina) {
	//if (rastlina.trenutnaStarost<=1) return false;
	return rastlina.prodornost <= 0.1f;
	//return rastlina.trenutnaMoc <= rastlina.trenutnaStarost/rastlina.umreNaKoraku*0.001f;
}//LifeFactors::umreZaradiSibkosti

bool LifeFactors::umreZaradiVisine(simulationEntities::CRastlina &rastlina) {
	return rastlina.vrstaRastline->visina.umreZaradiNeugodnegaPogoja(randomSmrtiZaradiNeugodnostiVisine,
	        rastlina.visinaTukajNormNadVodo);
}//LifeFactors::umreZaradiVisine

bool LifeFactors::umreZaradiNeugodneVlage(simulationEntities::CRastlina &rastlina) {
	return rastlina.vrstaRastline->vlaga.umreZaradiNeugodnegaPogoja(randomSmrtiZaradiNeugodnostiVlage,
	        rastlina.vlaznostTukaj);
}//LifeFactors::umreZaradiNeugodneVlage

bool LifeFactors::umreZaradiNeugodnegaSonca(simulationEntities::CRastlina &rastlina) {
	return rastlina.vrstaRastline->sonce.umreZaradiNeugodnegaPogoja(randomSmrtiZaradiNeugodnostiSonca,
	        rastlina.sonceTukaj);
}//LifeFactors::umreZaradiNeugodnegaSonca

bool LifeFactors::umreZaradiNeugodnegaVetra(simulationEntities::CRastlina &rastlina) {
	return rastlina.vrstaRastline->veter.umreZaradiNeugodnegaPogoja(randomSmrtiZaradiNeugodnostiVetra,
	        rastlina.veterTukaj);
}//LifeFactors::umreZaradiNeugodnegaVetra

bool LifeFactors::umreZaradiStrmine_oz_Prsti_oz_kislostiTal_Ph(simulationEntities::CRastlina &rastlina) {
	bool izidNeugoden = CRandomGenerator::randomGenerator(100000, randomSmrtiZaradiNeugodnostiNaklona)/100000.f
	                    < pow(rastlina.vrstaRastline->skodljivostNaklona*rastlina.strminaTukaj,2);
	if (izidNeugoden)
		return true;/// umre
	return false;
}//LifeFactors::umreZaradiStrmine_oz_Prsti_oz_kislostiTal_Ph

bool LifeFactors::umreZaradiDominiranosti(simulationEntities::CRastlina &rastlina) {
	//preverimo inferiornost glede na ostale rastline
	rastlina.jeDominirana = rastlinaJeDominirana(rastlina);

	if (rastlina.jeDominirana) {
		//je verjetnosti izid neugoden za ivljenje trenutne rastline?
		bool izidNeugoden = CRandomGenerator::randomGenerator(100000, randomSmrtiZaradiDominiranosti)/100000.f<rastlina.vrstaRastline->verjetnostUmrtjaObDominiranosti;
		if (izidNeugoden)
			return true;//umre
	}
	return false;
}//LifeFactors::umreZaradiDominiranosti

bool LifeFactors::rastlinaJeDominirana(simulationEntities::CRastlina &rastlina) {
	/// določimo koordinate polja kvadratov na mapi, ki bi lahko vsebovali sovržne rastline
	Tocka2i startPoint, endPoint;
	int maxR = simulationEntities::CVrstaRastlin::maxStKvadratovNajdemRadijSoseda;
#ifndef max
#define max(x,y) ((x>y)?(x):(y))
#define min(x,y) ((x<y)?(x):(y))
#endif

	startPoint.x = max(0,rastlina.pozicijaMap2D.x - maxR);
	startPoint.y = max(0,rastlina.pozicijaMap2D.y - maxR);

	endPoint.x = min((int)(*rastlinskeVrste)[0].terrainData->stTockX, rastlina.pozicijaMap2D.x + maxR);
	endPoint.y = min((int)(*rastlinskeVrste)[0].terrainData->stTockY, rastlina.pozicijaMap2D.y + maxR);
#undef max
#undef min

	bool result = obstajaMocnejsiSovraznikNaPoljuSosescine(startPoint, endPoint, rastlina);
	return result;
}//LifeFactors::rastlinaJeDominirana

bool LifeFactors::obstajaMocnejsiSovraznikNaPoljuSosescine(Tocka2i &startPoint, Tocka2i &endPoint, simulationEntities::CRastlina &rastlina) {
	int resX = (*rastlinskeVrste)[0].terrainData->stTockX;
	for (int y = startPoint.y; y <= endPoint.y; y++) {
		for (int x = startPoint.x; x <= endPoint.x; x++) {
			int index = x+resX*y;
			std::vector<simulationEntities::CRastlina*> &polje = rastlinePoTerenu[index];
			/// za ta kvadrat gremo skozi vse tukaj rastoče rastline in preverimo, ali se sekajo s podano rastlino
			for (std::vector<simulationEntities::CRastlina*>::iterator r = polje.begin(); r != polje.end(); r++) {
				simulationEntities::CRastlina &rastlinaSovraznik = **r;
				if (&rastlinaSovraznik==&rastlina)
					continue;//za samo sebe ne ugotavlja

				if (rastlina.trenutnaMoc * rastlina.vrstaRastline->vplivMoci <=
				        rastlinaSovraznik.trenutnaMoc * rastlinaSovraznik.vrstaRastline->vplivMoci)

					if (rastlina.presekaRadijRastline(rastlinaSovraznik))
						return true;
			}
		}
	}
	return false;
}//LifeFactors::obstajaMocnejsiSovraznikNaPoljuSosescine
