#ifndef ECOSYSTEM_SEJALEC_KJERKOLI_RASTLIN_NA_TEREN_H
#define ECOSYSTEM_SEJALEC_KJERKOLI_RASTLIN_NA_TEREN_H

#include "tocka.h"
#include "tocka2i.h"
#include "vrstarastlin.h"
#include "terraindata.h"

namespace ecosystem {
	/*!
	* \brief postavljanje rastlin na naključno krpo
	*@author Aleš Zamuda (ales.zamuda@uni-mb.si)
	*/
	class CSejalecKjerkoliRastlinNaTeren {
	public:
		/**
		 * Kreira sejalca rastlin podane vrste (seje samo to, podano vrsto!).
		 *
		 * @param terrainData
		 * @param pozicije2d
		 * @param pozicije3d
		 * @return
		 */
		CSejalecKjerkoliRastlinNaTeren(data::CTerrainData &terrainData, std::vector<Tocka2i> &pozicije2d, std::vector<drevo::Tocka> &pozicije3d);
		/*!
		* Določi pozicije za VSAKO novo rastlino te vrste na tem koraku, ki lahko raste kjerkoli
		* algoritem:
		* B. rast kjerkoli
		*   1)sproti vodimo rastline, ki lahko rastejo kjerkoli in jih generiramo na koncu
		*   2)te rastline dodamo v seznam 2d in 3d, za nad vodo preverjamo samo 3d pozicijo
		* @param stRastlinRastocihKjerkoli število dodanih rastlin te vrste na trenutnem koraku
		*/
		void dodajRastlineKjerkoli(int stRastlinRastocihKjerkoli);

	private:
		/**
		 * Določi pozicijo (indeks krpe) za novo posejano rastlino.
		 * @param pozicija2d
		 */
		void dolociPozicijoNoveRastline2Dkjerkoli(Tocka2i &pozicija2d);
		/**
		 * Doda rastlino v seznam 2d in 3d lokacij, če je rastlina nad vodo.
		 * @param pozicija2d seznam 2d pozicij rastlin
		 * @param pozicija3d seznam 3d pozicij rastlin
		 */
		void dodajRastlinoVSeznama(const Tocka2i &pozicija2d, const drevo::Tocka &pozicija3d);

		/**
		 * Podatki o terenu, potrebni za ugotavljanje, ali je nad vodo.
		 */
		data::CTerrainData &terrainData;
		/**
		 * Seznam pozicij (indeksov krp) živečih rastlin te vrste, ki jo sejemo.
		 */
		std::vector<Tocka2i> &pozicije2d;
		/**
		 * Pozicije 3D živečih rastlin vrste, ki jo sejemo.
		 */
		std::vector<drevo::Tocka> &pozicije3d;

		/**
		 * Seme za random generator.
		 */
		static unsigned long long randomPozicijeNovihRastlinKjerkoli;
		friend class CEcosystemSimulation;/// za pobiranje random generatorjev
	};
}//namespace ecosystem
#endif // ECOSYSTEM_SEJALEC_KJERKOLI_RASTLIN_NA_TEREN_H
