#include "sejalecrastlinnateren.h"
#include "randomgenerator.h"
#include <algorithm>

using namespace ecosystem;

unsigned long long ecosystem::CSejalecRastlinNaTeren::randomRastiBlizu = 1;
unsigned long long ecosystem::CSejalecRastlinNaTeren::randomIzbireBliznjeRastline = 1;
unsigned long long ecosystem::CSejalecRastlinNaTeren::randomIzbireStraniRastiObBliznjiRastlini = 1;

CSejalecRastlinNaTeren::CSejalecRastlinNaTeren(data::CTerrainData &terrainData,
        std::vector<Tocka2i> &pozicije2d, std::vector<drevo::Tocka> &pozicije3d,
        simulationEntities::CVrstaRastlin &vrstaRastlin) :
		terrainData(terrainData), pozicije2d(pozicije2d), pozicije3d(pozicije3d), vrstaRastlin(vrstaRastlin),
sejalecKjerkoliRastlinNaTeren(terrainData, pozicije2d, pozicije3d) {}//CSejalecRastlinNaTeren::CSejalecRastlinNaTeren

void CSejalecRastlinNaTeren::dolociPozicijeRastlin() {
	/// kreiramo 2d indekse
	int stRastlinRastocihKjerkoli = 0;
	std::vector<int> indeksiBlizu;
	kreirajIndekseRastlinBlizu(indeksiBlizu, stRastlinRastocihKjerkoli);
	std::sort(indeksiBlizu.begin(), indeksiBlizu.end());
	/// iz indeksov doda rastline, ki naj rastejo blizu
	dodajRastlineBlizu(indeksiBlizu);

	/// doda preostale rastline, ki jih lahko doda kjerkoli
	sejalecKjerkoliRastlinNaTeren.dodajRastlineKjerkoli(stRastlinRastocihKjerkoli);
}//CSejalecRastlinNaTeren::dolociPozicijeRastlin

void CSejalecRastlinNaTeren::kreirajIndekseRastlinBlizu(std::vector<int> &indeksiBlizu,
        int &stRastlinRastocihKjerkoli) {
	int stBliznjih = (int)vrstaRastlin.ziviPrimerkiTeVrste.size();
	while (vrstaRastlin.komulacijaPrihajajocihRastlin >= 1) {
		/// označimo, da smo dodali novo rastlino
		vrstaRastlin.komulacijaPrihajajocihRastlin -= 1;
		if (rasteBlizu()) {
			/// @alert: če ne bo nič blinžjih, rastlin pač ne generiramo
			if (stBliznjih > 0) {
				int indexBliznjaRastlina = CRandomGenerator::randomGenerator(stBliznjih, randomIzbireBliznjeRastline);
				indeksiBlizu.push_back(indexBliznjaRastlina);
			}
		} else
			stRastlinRastocihKjerkoli++;
	}
	return;
}//CRastlina::kreirajIndekseRastlinBlizu

bool CSejalecRastlinNaTeren::rasteBlizu() {
	return CRandomGenerator::randomGenerator(100000, randomRastiBlizu)/100000.f
	       < vrstaRastlin.delezRastiBlizuSvojeVrste;
}//CSejalecRastlinNaTeren::rasteBlizu

void CSejalecRastlinNaTeren::dodajRastlineBlizu(std::vector<int> &indeksiBlizu) {
	int indeksStarih = 0, indeksNovih = 0;
	int stNovih = (int)indeksiBlizu.size();
	for (simulationEntities::CVrstaRastlin::ziviPrimerkiTeVrsteIter i = vrstaRastlin.ziviPrimerkiTeVrste.begin();
	        i != vrstaRastlin.ziviPrimerkiTeVrste.end() && indeksNovih < stNovih;) {
		if (indeksStarih == indeksiBlizu[indeksNovih]) {
			Tocka2i pozicija2d;
			dolociPozicijoNoveRastline2Dblizu((*i).pozicijaMap2D, pozicija2d);

			/// mora biti brez vode!!
			if (terrainData.jeNadVodo(pozicija2d)) {
				drevo::Tocka pozicija3d;
				terrainData.dolociPozicijoNoveRastline3D(pozicija2d, pozicija3d);
				dodajRastlinoVSeznama(pozicija2d, pozicija3d);
			}

			indeksNovih++;
		} else
			i++, indeksStarih++;
	}
}//CSejalecRastlinNaTeren::dodajRastlineBlizu

void CSejalecRastlinNaTeren::dolociPozicijoNoveRastline2Dblizu(Tocka2i &pozicija2dObstojeceRastline, Tocka2i &pozicija2dNova) {
	pozicija2dNova = pozicija2dObstojeceRastline;
	/// naj v vsako stran rastejo z enako verjetnostjo:
	/// če bo pač v izbrano stran voda, kumulativno raste 3/4 glede onih na suhem.
	/// ena ostane enako
	int stran = (int)(CRandomGenerator::randomGenerator(100000, randomIzbireStraniRastiObBliznjiRastlini)/100000.f*4);
	switch(stran) {
	case 0:
		pozicija2dNova.y--;
		break;/// gor
	case 1:
		pozicija2dNova.y++;
		break;/// dol
	case 2:
		pozicija2dNova.x--;
		break;/// levo
	case 3:
		pozicija2dNova.x++;
		break;/// desno
	}
}//CSejalecRastlinNaTeren::dolociPozicijoNoveRastline2Dblizu

void CSejalecRastlinNaTeren::dodajRastlinoVSeznama(const Tocka2i &pozicija2d, const drevo::Tocka &pozicija3d) {
	if (terrainData.jeNadVodo(pozicija3d)) {
		pozicije2d.push_back(pozicija2d);
		pozicije3d.push_back(pozicija3d);
	}
}//CSejalecRastlinNaTeren::dodajRastlinoVSeznama
