#ifndef ECOSYSTEM_STATISTICS_ENTITIES_STATISTICS_H
#define ECOSYSTEM_STATISTICS_ENTITIES_STATISTICS_H
#include "krivulja.h"

#include <string>

namespace ecosystem {
	/*!
	* \namespace ecosystem::statistics
	* \brief vodenje statistike za simulacijo
	*
	* Prostor je namenjen vodenju, obdelavi in shranjevanju in želenih statistik o simulaciji.
	* Prikazovanje statistike je v imenskem prostoru gui.
	* \sa gui
	*/
	namespace statistics {
		/*!
		* \brief vodenje statistike za število rastlin
		*
		*@author Aleš Zamuda (ales.zamuda@uni-mb.si)
		*/
		class CEntitiesStatistics {
		public:
			/**
			 * Kreira in inicializira objekt.
			 */
			CEntitiesStatistics();
			/**
			 * Pobriše shranjene grafe za statistiko
			 */
			~CEntitiesStatistics();
			/**
			 * Shrani število trenutno živih rastlin.
			 * @param stTrenutnoZivih
			 */
			void setStTrenutnoZivih(int stTrenutnoZivih);
			/**
			 * Prišteje smrt zaradi dominiranosti.
			 */
			void dodajSmrtZaradiDominiranosti();
			/**
			 * Prišteje smrt zaradi nekega neugodnega pogoja ali postaranja do smrti.
			 */
			void dodajSmrtOstalo();
			/**
			 * Doda trenutna števila rastlin v pripadajoče grafe.
			 * @param value seznam trenutno živih rastlin po rastlinskih vrstah
			 */
			void addLiveSpecies(const std::vector<int> &);
			/**
			 * Doda trenutna števila upodobljenih rastlin v pripadajoče grafe.
			 * @param value seznam trenutno živih rastlin po rastlinskih vrstah
			 */
			void addRenderedSpecies(const std::vector<int> &);

			/**
			 * Vrne grafe števila rastlin v simulaciji
			 * @return št. rastlin
			 */
			std::vector<drevo::Krivulja*>* getGrafiStevilaRastlin();

			/**
			 * Vrne grafe števila upodobljenih rastlin v simulaciji
			 * @return št. rastlin
			 */
			std::vector<drevo::Krivulja*>* getGrafiStevilaUpodobljenihRastlin();

			/**
			 * Vrne log izpis statistike timerjev in ostalega
			 * @return log izpis, primeren za konzolo
			 */
			std::string getLogText();
			/**
			 * Shrani čase trajanja posameznih simulacijskih sklopov rutin, tvori log izpis.
			 * @param stepNr zaporedna št. simulacijskega koraka
			 * @param casOstalo čas renderiranja in ostalih rutin
			 * @param casDodajal trajanje dodajanja novih rastlin
			 * @param casZrastel trajanje zrasti novih rastlin, določanja moči, ...
			 * @param casPobijal trajanje odstranjevanja rastlin, ki umrejo na tem koraku
			 * @param casPobijanjaDominiranihRastlin čas iskanja remove_if, ali so rastline dominirane
			 */
			void saveSimulationRutinesTimings(int stepNr, int casOstalo, int casDodajal, int casZrastel, int casPobijal, int casPobijanjaDominiranihRastlin);
		private:
			/**
			 * Število trenutno živih rastlin
			 */
			int stTrenutnoZivih;
			/**
			 * Število umrlih rastlin zaradi dominiranosti.
			 */
			int umrliZaradiDominiranosti;
			/**
			 * Število umrlih rastlin, ki niso umrle zaradi dominiranosti (ostalo).
			 */
			int umrliOstalo;
			/**
			 * Število umrlih rastlin vse skupaj.
			 */
			int umrliSkupaj;
			/**
			* Grafi števil rastlin skozi čas, po rastlinskih vrstah
			*/
			std::vector<drevo::Krivulja*> grafiStevilaRastlin;
			/**
			* Grafi števil upodobljenih rastlin skozi čas, po rastlinskih vrstah
			*/
			std::vector<drevo::Krivulja*> grafiStevilaUpodobljenihRastlin;
			/**
			 * Do sedaj skupaj porabljen čas za renderiranje.
			 */
			int sumRenderTime;
			/**
			 * Do sedaj skupaj porabljen čas za simulacijo.
			 */
			int sumSimmTime;
			/**
			 * Izpis log - texta, ki ga lahko narišemo nad ekranom.
			 */
			std::string logText;
		};
	}//namespace statistics
}//namespace ecosystem
#endif // ECOSYSTEM_STATISTICS_ENTITIES_STATISTICS_H
