/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "advancedtreeproperties.h"
#include "drevo.h"
#include "parametridrevesa.h"
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

using namespace gui;

AdvancedTreeProperties::AdvancedTreeProperties( QWidget *parent, drevo::Drevo *drevo )
		: QDialog( parent ), drevo(drevo) {
	setupUi(this);

	pdUndo = new drevo::ParametriDrevesa(drevo->pdPrepared);

	editNumberOfStrands->setValue(drevo->pdPrepared.stVsehZil);
	sliderNumberOfStrands->setValue(drevo->pdPrepared.stVsehZil);
	editTrinkHeight->setValue(drevo->pdPrepared.dolzinaDebla);
	sliderTrinkHeight->setValue(int(drevo->pdPrepared.dolzinaDebla*10000.0/10));
	editBranchThicknessPerStrand->setValue(drevo->pdPrepared.sirinaSteblaNaZilo);
	sliderBranchThicknessPerStrand->setValue(int(drevo->pdPrepared.sirinaSteblaNaZilo*10000.0/0.05));
	editGravicentralism->setValue(drevo->pdPrepared.graviCentralizem);
	sliderGravicentralism->setValue(int(drevo->pdPrepared.graviCentralizem*10000.0));
	editPhyllotaxisAngle->setValue(drevo->pdPrepared.phyllotaxisAngle);
	sliderPhyllotaxisAngle->setValue((int)drevo->pdPrepared.phyllotaxisAngle);
	editLeafType->setCurrentIndex(drevo->pdPrepared.tipListov);
	editLeafDensity->setValue(drevo->pdPrepared.gostotaListov);
	sliderLeafDensity->setValue(drevo->pdPrepared.gostotaListov);
	editLeafSize->setValue(drevo->pdPrepared.listSize);
	sliderLeafSize->setValue(int(drevo->pdPrepared.listSize*10000.0/0.3));
	editWindSpeed->setValue(drevo->pdPrepared.windSpeed);
	sliderWindSpeed->setValue(int(drevo->pdPrepared.windSpeed*10000.0/10.0));
	editWindSpeedNodirect->setValue(drevo->pdPrepared.windSpeedNodirect);
	sliderWindSpeedNodirect->setValue(int(drevo->pdPrepared.windSpeedNodirect*10000.0/10.0));

	connect(this, SIGNAL(rejected()), this, SLOT(makeUndo()));
	connect(buttonApply, SIGNAL(clicked()), this, SLOT(applyParameters()));
}

AdvancedTreeProperties::~AdvancedTreeProperties() {
	delete pdUndo;
}

void AdvancedTreeProperties::makeUndo() {
#define undo(x) drevo->pdPrepared.x=pdUndo->x;
	undo(stVsehZil);
	undo(dolzinaDebla);
	undo(sirinaSteblaNaZilo);
	undo(graviCentralizem);
	undo(phyllotaxisAngle);
	undo(tipListov);
	undo(gostotaListov);
	undo(listSize);
	undo(windSpeed);
#undef undo
}

void AdvancedTreeProperties::applyParameters() {
#define apply(x,y) drevo->pdPrepared.x=y;
	apply(stVsehZil, editNumberOfStrands->value());
	apply(dolzinaDebla, editTrinkHeight->value());
	apply(sirinaSteblaNaZilo, editBranchThicknessPerStrand->value());
	apply(graviCentralizem, editGravicentralism->value());
	apply(phyllotaxisAngle, editPhyllotaxisAngle->value());
	apply(tipListov, editLeafType->currentIndex());
	apply(gostotaListov, editLeafDensity->value());
	apply(listSize, editLeafSize->value());
	apply(windSpeed, editWindSpeed->value());
#undef apply

	*pdUndo = drevo->pdPrepared;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_editNumberOfStrands_valueChanged(int value) {
	drevo->pdPrepared.stVsehZil=value;
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_sliderNumberOfStrands_valueChanged(int value) {
	drevo->pdPrepared.stVsehZil=value;
	editNumberOfStrands->setValue(drevo->pdPrepared.stVsehZil);
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_editTrinkHeight_valueChanged(double value) {
	drevo->pdPrepared.dolzinaDebla=value;
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_sliderTrinkHeight_valueChanged(int value) {
	drevo->pdPrepared.dolzinaDebla=value/10000.0*10;
	editTrinkHeight->setValue(drevo->pdPrepared.dolzinaDebla);
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_editBranchThicknessPerStrand_valueChanged(double value) {
	drevo->pdPrepared.sirinaSteblaNaZilo=value;
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_sliderBranchThicknessPerStrand_valueChanged(int value) {
	drevo->pdPrepared.sirinaSteblaNaZilo=value/10000.0*0.05;
	editBranchThicknessPerStrand->setValue(drevo->pdPrepared.sirinaSteblaNaZilo);
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_editGravicentralism_valueChanged(double value) {
	drevo->pdPrepared.graviCentralizem=value;
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_sliderGravicentralism_valueChanged(int value) {
	drevo->pdPrepared.graviCentralizem=value/10000.0;
	editGravicentralism->setValue(drevo->pdPrepared.graviCentralizem);
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_editPhyllotaxisAngle_valueChanged(double value) {
	drevo->pdPrepared.phyllotaxisAngle=value;
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_sliderPhyllotaxisAngle_valueChanged(int value) {
	drevo->pdPrepared.phyllotaxisAngle=value;
	editPhyllotaxisAngle->setValue(value);
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_editLeafType_currentIndexChanged(int value) {
	drevo->pdPrepared.tipListov=value;
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_editLeafDensity_valueChanged(int value) {
	drevo->pdPrepared.gostotaListov=value;
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_sliderLeafDensity_valueChanged(int value) {
	drevo->pdPrepared.gostotaListov=value;
	editLeafDensity->setValue(drevo->pdPrepared.gostotaListov);
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_editLeafSize_valueChanged(double value) {
	drevo->pdPrepared.listSize=value;
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_sliderLeafSize_valueChanged(int value) {
	drevo->pdPrepared.listSize=value/10000.0*0.3;
	editLeafSize->setValue(drevo->pdPrepared.listSize);
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_editWindSpeed_valueChanged(double value) {
	drevo->pdPrepared.windSpeed=value;
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_sliderWindSpeedNodirect_valueChanged(int value) {
	drevo->pdPrepared.windSpeedNodirect=value/10000.0*10.0;
	editWindSpeedNodirect->setValue(drevo->pdPrepared.windSpeedNodirect);
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}


void AdvancedTreeProperties::on_editWindSpeedNodirect_valueChanged(double value) {
	drevo->pdPrepared.windSpeedNodirect=value;
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}

void AdvancedTreeProperties::on_sliderWindSpeed_valueChanged(int value) {
	drevo->pdPrepared.windSpeed=value/10000.0*10.0;
	editWindSpeed->setValue(drevo->pdPrepared.windSpeed);
	if (cbAutoProof->checkState()!=Qt::Checked)
		return;
	emit forceRefresh();
}
