/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef ADVANCEDTREEPROPERTIES_H
#define ADVANCEDTREEPROPERTIES_H

#include "ui_advancedtreeproperties.h"
namespace drevo {
	class Drevo;
	class ParametriDrevesa;
}

namespace gui {
	/**
	 * Prikaže dialog za nastavljanje podrobnih lastnosti drevesa in jih vrši.
	 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
	 */
	class AdvancedTreeProperties: public QDialog, private Ui::AdvancedTreeProperties {
		Q_OBJECT
	public:
		/**
		 * Shrani povezavo na lastnika in urejan model drevesa.
		 * @param parent lastnik
		 * @param drevo urejan model drevesa
		 */
		AdvancedTreeProperties( QWidget *parent, drevo::Drevo *drevo);
		/**
		 * Pobriše objekt, undo parametre.
		 */
		~AdvancedTreeProperties();

	signals:
		/**
		 * Sprožen, ko naj se prikaz modela drevesa osveži, saj smo spremenili parametre.
		 */
		void forceRefresh();

	public slots:
		/**
		 * Vrne parametre na prejšnje nastavitve.
		 */
		void makeUndo();
		/**
		 * Shrani število žil ob vpisu.
		 * @param value novo število žil
		 */
		void on_editNumberOfStrands_valueChanged(int);
		/**
		 * Shrani število žil.
		 * @param value novo število žil
		 */
		void on_sliderNumberOfStrands_valueChanged(int);
		/**
		 * Shrani velikost prvega odseka osnovnega debla.
		 * @param value nova velikost
		 */
		void on_editTrinkHeight_valueChanged(double);
		/**
		 * Shrani velikost prvega odseka osnovnega debla.
		 * @param value nova velikost
		 */
		void on_sliderTrinkHeight_valueChanged(int);
		/**
		 * Shrani debelino vej.
		 * @param value nova debelina
		 */
		void on_editBranchThicknessPerStrand_valueChanged(double);
		/**
		 * Shrani debelino vej
		 * @param value nova debelina
		 */
		void on_sliderBranchThicknessPerStrand_valueChanged(int);
		/**
		 * Shrani nastavitve kota gravicentralizma.
		 * @param value nov kot
		 */
		void on_editGravicentralism_valueChanged(double);
		/**
		 * Shrani nastavitve kota gravicentralizma.
		 * @param value nov kot
		 */
		void on_sliderGravicentralism_valueChanged(int);
		/**
		 * Shrani kot filotakse
		 * @param value nov kot
		 */
		void on_editPhyllotaxisAngle_valueChanged(double);
		/**
		 * Shrani kot filotakse
		 * @param value nov kot
		 */
		void on_sliderPhyllotaxisAngle_valueChanged(int);
		/**
		 * Shrani tip razporeditve listov
		 * @param value nova razporeditev
		 */
		void on_editLeafType_currentIndexChanged(int);
		/**
		 * Shrani gostoto listov
		 * @param value gostota listov
		 */
		void on_editLeafDensity_valueChanged(int);
		/**
		 * Shrani gostoto listov
		 * @param value gostota listov
		 */
		void on_sliderLeafDensity_valueChanged(int);
		/**
		 * Shrani velikost listov
		 * @param value nova velikost
		 */
		void on_editLeafSize_valueChanged(double);
		/**
		 * Shrani velikost listov
		 * @param value nova velikost
		 */
		void on_sliderLeafSize_valueChanged(int);
		/**
		 * Shrani hitrost vetra
		 * @param value nova hitrost vetra
		 */
		void on_editWindSpeed_valueChanged(double);
		/**
		 * Shrani hitrost vetra
		 * @param value nova hitrost vetra
		 */
		void on_sliderWindSpeed_valueChanged(int);
		/**
		 * Shrani hitrost vetra
		 * @param value nova hitrost vetra
		 */
		void on_editWindSpeedNodirect_valueChanged(double);
		/**
		 * Shrani hitrost vetra
		 * @param value nova hitrost vetra
		 */
		void on_sliderWindSpeedNodirect_valueChanged(int);

		/**
		 * Shrani vse nastavljanje parametre v ParametreDrevesa
		 */
		void applyParameters();

	private:
		/**
		 * Kazalec na drevo, katerega parametre nastavljamo s tem dialogom.
		 */
		drevo::Drevo *drevo;
		/**
		 * Undo za spreminjane parametre drevesa
		 */
		drevo::ParametriDrevesa *pdUndo;
	};
}
#endif
