/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "designhandler.h"
#include "drevo.h"

#include "render/ecosystemrender.h"
#include "krivulja.h"
#include "distributiondesigner.h"
#include "parametridrevesa.h"
#include "advancedtreeproperties.h"

#include <QPolygonF>
#include <QRectF>
#include <QPointF>
#include <vector>

namespace gui {

	namespace actionhandling {

		DesignHandler::DesignHandler(QObject *parent, QWidget *parentWindow, ecosystem::render::EcosystemRender * model)
				: QObject(parent), parentWindow(parentWindow), model(model) {}

		DesignHandler::~DesignHandler() {}

		void DesignHandler::actionAdvanced_tree_properties() {
			AdvancedTreeProperties *advancedTreeProperties =
			    new AdvancedTreeProperties ( parentWindow, model->getIzbranoDrevo() );
			connect(advancedTreeProperties, SIGNAL(forceRefresh()), this, SIGNAL(forceRepaint()));
			advancedTreeProperties->show();
		}

		void DesignHandler::actionDesign_strand_distribution() {
			std::vector<int> tipParametra;
			std::vector<drevo::Krivulja*> porazdelitevG, porazdelitevW;

			tipParametra.push_back(drevo::ParametriDrevesa::STRAND_DISTRIBUTION);
			porazdelitevG.push_back(&model->getIzbranoDrevo()->pdPrepared.verjetnostiVejitveG);
			porazdelitevW.push_back(&model->getIzbranoDrevo()->pdPrepared.verjetnostiVejitveW);
			const QString *labeleG[]= {
			                              new QString("50%"), new QString("62,5%"), new QString("75%"),
			                              new QString("87,5%"),new QString("100%")
			                          };
			const QString *labeleW[]= {
			                              new QString("0%"), new QString("50%"), new QString("100%"),
			                              new QString("150%"),new QString("200%")
			                          };
			createDesigner(tipParametra,porazdelitevG,porazdelitevW, labeleG, labeleW, tr(" for strand distribution"));
		}

		void DesignHandler::actionDesign_branching_angles() {
			std::vector<int> tipParametra;
			std::vector<drevo::Krivulja*> porazdelitevG, porazdelitevW;

			tipParametra.push_back(drevo::ParametriDrevesa::BRANCH_ANGLES_DISTRIBUTION);
			porazdelitevG.push_back(&model->getIzbranoDrevo()->pdPrepared.kotVejitveG);
			porazdelitevW.push_back(&model->getIzbranoDrevo()->pdPrepared.kotVejitveW);

			const QString *labeleG[]= {
			                              new QString(QString::fromUtf8("0°")), new QString(QString::fromUtf8("45°")), new QString(QString::fromUtf8("90°")),
			                              new QString(QString::fromUtf8("135°")),new QString(QString::fromUtf8("180°"))
			                          };
			const QString *labeleW[]= {
			                              new QString("0%"), new QString("50%"), new QString("100%"),
			                              new QString("150%"),new QString("200%")
			                          };
			createDesigner(tipParametra,porazdelitevG,porazdelitevW, labeleG, labeleW, tr(" for branching angles distribution"));
		}

		void DesignHandler::actionDesign_branch_absolute_minimal_length() {
			std::vector<int> tipParametra;
			std::vector<drevo::Krivulja*> porazdelitevG, porazdelitevW;

			tipParametra.push_back(drevo::ParametriDrevesa::MIN_CHARACTERISTIC_LENGTH);
			porazdelitevG.push_back(&model->getIzbranoDrevo()->pdPrepared.minCharacteristicLengthG);
			porazdelitevW.push_back(&model->getIzbranoDrevo()->pdPrepared.minCharacteristicLengthW);

			const QString *labeleG[]= {
			                              new QString("0%"), new QString("250%"), new QString("500%"),
			                              new QString("750%"),new QString("1000%")
			                          };
			const QString *labeleW[]= {
			                              new QString("0%"), new QString("50%"), new QString("100%"),
			                              new QString("150%"),new QString("200%")
			                          };
									  createDesigner(tipParametra,porazdelitevG,porazdelitevW, labeleG, labeleW,
											  tr(" for minimal absolute branch lengths distribution"));
		}

		void DesignHandler::actionDesign_branch_absolute_maximal_length() {
			std::vector<int> tipParametra;
			std::vector<drevo::Krivulja*> porazdelitevG, porazdelitevW;

			tipParametra.push_back(drevo::ParametriDrevesa::MAX_CHARACTERISTIC_LENGTH);
			porazdelitevG.push_back(&model->getIzbranoDrevo()->pdPrepared.maxCharacteristicLengthG);
			porazdelitevW.push_back(&model->getIzbranoDrevo()->pdPrepared.maxCharacteristicLengthW);

			const QString *labeleG[]= {
			                              new QString("0%"), new QString("250%"), new QString("500%"),
			                              new QString("750%"),new QString("1000%")
			                          };
			const QString *labeleW[]= {
			                              new QString("0%"), new QString("50%"), new QString("100%"),
			                              new QString("150%"),new QString("200%")
			                          };
			createDesigner(tipParametra,porazdelitevG,porazdelitevW, labeleG, labeleW,
						   tr(" for maximal absolute branch lengths distribution"));
		}

		void DesignHandler::actionDesign_branch_lenght_scaling() {
			std::vector<int> tipParametra;
			std::vector<drevo::Krivulja*> porazdelitevG, porazdelitevW;

			tipParametra.push_back(drevo::ParametriDrevesa::LENGTH_SCALING);
			porazdelitevG.push_back(&model->getIzbranoDrevo()->pdPrepared.lengthScalingG);
			porazdelitevW.push_back(&model->getIzbranoDrevo()->pdPrepared.lengthScalingW);

			const QString *labeleG[]= {
			                              new QString("0%"), new QString("250%"), new QString("500%"),
			                              new QString("750%"),new QString("1000%")
			                          };
			const QString *labeleW[]= {
			                              new QString("0%"), new QString("50%"), new QString("100%"),
			                              new QString("150%"),new QString("200%")
			                          };
			createDesigner(tipParametra,porazdelitevG,porazdelitevW, labeleG, labeleW,
						   tr(" for branch lengths scaling distribution"));
		}

		void DesignHandler::actionDesign_gravimorphism_impact_distribution() {
			std::vector<int> tipParametra;
			std::vector<drevo::Krivulja*> porazdelitevG, porazdelitevW;

			tipParametra.push_back(drevo::ParametriDrevesa::GRAVIMORPHISM_IMPACT);
			porazdelitevG.push_back(&model->getIzbranoDrevo()->pdPrepared.gravimorphismImpactG);
			porazdelitevW.push_back(&model->getIzbranoDrevo()->pdPrepared.gravimorphismImpactW);

			const QString *labeleG[]= {
			                              new QString(QString::fromUtf8("-180°")), new QString(QString::fromUtf8("-90°")), new QString(QString::fromUtf8("0°")),
			                              new QString(QString::fromUtf8("90°")),new QString(QString::fromUtf8("180°"))
			                          };
			const QString *labeleW[]= {
			                              new QString("0%"), new QString("50%"), new QString("100%"),
			                              new QString("150%"),new QString("200%")
			                          };
			createDesigner(tipParametra,porazdelitevG,porazdelitevW, labeleG, labeleW,
						   tr(" for gravimorphism impact distribution"));
		}

		void DesignHandler::createDesigner(const std::vector<int>&tipParametra,
		                                   const std::vector<drevo::Krivulja*>&porazdelitevG,
		                                   const std::vector<drevo::Krivulja*>&porazdelitevW,
		                                   const QString *labeleG[], const QString *labeleW[], const QString &title) {
			DistributionDesigner *distributionDesigner =
			    new DistributionDesigner( parentWindow, tipParametra, porazdelitevG, porazdelitevW, labeleG, labeleW, title);
			distributionDesigner->show();
			distributionDesigner->setKrivulje();
			connect(distributionDesigner, SIGNAL(pointsUpdated(const std::vector<int>&, const std::vector<drevo::Krivulja*>&, const std::vector<drevo::Krivulja*>&)),
			        this, SLOT(updateKrivulja(const std::vector<int>&, const std::vector<drevo::Krivulja*>&, const std::vector<drevo::Krivulja*>&)));

			connect(distributionDesigner, SIGNAL(hoverGChanged(int,int)), this, SLOT(applyHoverGChanged(int,int)));
			connect(distributionDesigner, SIGNAL(hoverWChanged(int,int)), this, SLOT(applyHoverWChanged(int,int)));
			for (int i = 0; i < 5; i++) {
				delete labeleG[i];
				delete labeleW[i];
			}
		}

		///apliciramo krivulje, pobrišemo za sabo
		void DesignHandler::updateKrivulja(const std::vector<int>& tipParametra, const std::vector<drevo::Krivulja*>& novaKrivuljaG, const std::vector<drevo::Krivulja*>& novaKrivuljaW) {
			drevo::ParametriDrevesa *pd=&model->getIzbranoDrevo()->pdPrepared;

			for (int i = 0; i < (int)tipParametra.size(); i++) {
				switch (tipParametra[i]) {
				case drevo::ParametriDrevesa::STRAND_DISTRIBUTION:
					pd->verjetnostiVejitveG=*novaKrivuljaG[i];
					pd->verjetnostiVejitveW=*novaKrivuljaW[i];
					break;
				case drevo::ParametriDrevesa::BRANCH_ANGLES_DISTRIBUTION:
					pd->kotVejitveG=*novaKrivuljaG[i];
					pd->kotVejitveW=*novaKrivuljaW[i];
					break;
				case drevo::ParametriDrevesa::MIN_CHARACTERISTIC_LENGTH:
					pd->minCharacteristicLengthG=*novaKrivuljaG[i];
					pd->minCharacteristicLengthW=*novaKrivuljaW[i];
					break;
				case drevo::ParametriDrevesa::MAX_CHARACTERISTIC_LENGTH:
					pd->maxCharacteristicLengthG=*novaKrivuljaG[i];
					pd->maxCharacteristicLengthW=*novaKrivuljaW[i];
					break;
				case drevo::ParametriDrevesa::LENGTH_SCALING:
					pd->lengthScalingG=*novaKrivuljaG[i];
					pd->lengthScalingW=*novaKrivuljaW[i];
					break;
				case drevo::ParametriDrevesa::GRAVIMORPHISM_IMPACT:
					pd->gravimorphismImpactG=*novaKrivuljaG[i];
					pd->gravimorphismImpactW=*novaKrivuljaW[i];
					break;
				}

				///prevežemo kazalce, stare krivulje pobrišemo
				for (int x =0; x < (int)novaKrivuljaG.size(); x++) {
					delete novaKrivuljaG[x];
					delete novaKrivuljaW[x];
				}
				pd->preracunajParametra( tipParametra[i]);
			}
			emit forceRepaint();
		}

		void DesignHandler::applyHoverGChanged(int indexMin, int indexMax) {
			model->getIzbranoDrevo()->pogled.barvajWMin=-1;
			model->getIzbranoDrevo()->pogled.barvajWMax=-1;

			model->getIzbranoDrevo()->pogled.barvajGMin=indexMin;
			model->getIzbranoDrevo()->pogled.barvajGMax=indexMax;
			emit forceRepaint();
		}

		void DesignHandler::applyHoverWChanged(int indexMin, int indexMax) {
			model->getIzbranoDrevo()->pogled.barvajGMin=-1;
			model->getIzbranoDrevo()->pogled.barvajGMax=-1;

			model->getIzbranoDrevo()->pogled.barvajWMin=indexMin;
			model->getIzbranoDrevo()->pogled.barvajWMax=indexMax;
			emit forceRepaint();
		}
	}
}
