/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "distributiondesigner.h"
#include "hoverpoints.h"
#include "krivulja.h"
#include "tocka2i.h"
#include "tocka2d.h"

#include <vector>
#include <QPointF>
#include <QPolygonF>


using namespace gui;

DistributionDesigner::DistributionDesigner( QWidget *parent, std::vector<int> tipParametra, std::vector<drevo::Krivulja*> porazdelitevG,
											std::vector<drevo::Krivulja*> porazdelitevW,
											const QString *labeleG[], const QString *labeleW[], const QString &title
										  ) : QDialog( parent ), tipParametra(tipParametra), porazdelitevG(porazdelitevG), porazdelitevW(porazdelitevW) {
	setupUi( this );

	porazdelitevGUndo=porazdelitevG;
	for (int i = 0; i < (int)porazdelitevGUndo.size(); i++) {
		porazdelitevGUndo[i]=new drevo::Krivulja;
		porazdelitevGUndo[i]->kontrolneTocke=porazdelitevG[i]->kontrolneTocke;
	}
	porazdelitevWUndo=porazdelitevW;
	for (int i = 0; i < (int)porazdelitevWUndo.size(); i++) {
		porazdelitevWUndo[i]=new drevo::Krivulja;
		porazdelitevWUndo[i]->kontrolneTocke=porazdelitevW[i]->kontrolneTocke;
	}

	connect(this, SIGNAL(rejected()), this, SLOT(ponastaviParametreDrevesa()));

	connect(graphG->hoverPoints(), SIGNAL(pointsChanged( const QPolygonF &)), this, SLOT(fireKrivuljeChanged()));
	connect(graphW->hoverPoints(), SIGNAL(pointsChanged( const QPolygonF &)), this, SLOT(fireKrivuljeChanged()));

	connect(graphG->hoverPoints(), SIGNAL(hoverChanged(const QPointF &)), this, SLOT(applyHoverGChanged(const QPointF &)));
	connect(graphW->hoverPoints(), SIGNAL(hoverChanged(const QPointF &)), this, SLOT(applyHoverWChanged(const QPointF &)));

	this->g_0->setText(*labeleG[0]);
	this->g_1->setText(*labeleG[1]);
	this->g_2->setText(*labeleG[2]);
	this->g_3->setText(*labeleG[3]);
	this->g_4->setText(*labeleG[4]);
	this->w_0->setText(*labeleW[0]);
	this->w_1->setText(*labeleW[1]);
	this->w_2->setText(*labeleW[2]);
	this->w_3->setText(*labeleW[3]);
	this->w_4->setText(*labeleW[4]);

	this->setWindowTitle(this->windowTitle()+title);
}

DistributionDesigner::~DistributionDesigner() {
	for (int i = 0; i < (int)porazdelitevGUndo.size(); i++) {
		delete porazdelitevGUndo[i];
	}
	for (int i = 0; i < (int)porazdelitevWUndo.size(); i++) {
		delete porazdelitevWUndo[i];
	}
}

void DistributionDesigner::ponastaviParametreDrevesa() {
	for (int i = 0; i < (int)porazdelitevGUndo.size(); i++) {
		porazdelitevG[i]->kontrolneTocke=porazdelitevGUndo[i]->kontrolneTocke;
	}
	for (int i = 0; i < (int)porazdelitevWUndo.size(); i++) {
		porazdelitevW[i]->kontrolneTocke=porazdelitevWUndo[i]->kontrolneTocke;
	}

	std::vector<drevo::Krivulja*> pkG=porazdelitevG;
	for (int i = 0; i < (int)porazdelitevG.size(); i++) {
		pkG[i]=new drevo::Krivulja;
		pkG[i]->kontrolneTocke=porazdelitevG[i]->kontrolneTocke;
	}
	std::vector<drevo::Krivulja*> pkW=porazdelitevW;
	for (int i = 0; i < (int)porazdelitevW.size(); i++) {
		pkW[i]=new drevo::Krivulja;
		pkW[i]->kontrolneTocke=porazdelitevW[i]->kontrolneTocke;
	}

	emit pointsUpdated(tipParametra, pkG, pkW);
}

void DistributionDesigner::setKrivulje() {
	for (int param=0; param < (int)tipParametra.size(); param++) {
		{
			QPolygonF tockeG;
			QRectF rect=graphG->hoverPoints()->boundingRect();
			for (int i=0; i < (int)porazdelitevG[param]->kontrolneTocke.size(); i++) {
				float x=porazdelitevG[param]->kontrolneTocke[i].x*rect.width();
				float y=(1.0f-porazdelitevG[param]->kontrolneTocke[i].y)*rect.height();
				QPointF tocka(x,y);
				tockeG<<tocka;
			}
			graphG->hoverPoints()->setPoints(tockeG);
			graphG->hoverPoints()->setPointLock(0, HoverPoints::LockToLeft);
			graphG->hoverPoints()->setPointLock(tockeG.size() - 1, HoverPoints::LockToRight);
			graphG->update();
		}

		{
			QPolygonF tockeW;
			QRectF rect=graphW->hoverPoints()->boundingRect();
			for (int i=0; i < (int)porazdelitevW[param]->kontrolneTocke.size(); i++) {
				float x=porazdelitevW[param]->kontrolneTocke[i].x*rect.width();
				float y=(1.0f-porazdelitevW[param]->kontrolneTocke[i].y)*rect.height();
				QPointF tocka(x,y);
				tockeW<<tocka;
			}
			graphW->hoverPoints()->setPoints(tockeW);
			graphW->hoverPoints()->setPointLock(0, HoverPoints::LockToLeft);
			graphW->hoverPoints()->setPointLock(tockeW.size() - 1, HoverPoints::LockToRight);
			graphW->update();
		}
	}
}

void DistributionDesigner::fireKrivuljeChanged() {
	if (autoProof->checkState()!=Qt::Checked) return;

	std::vector<drevo::Krivulja*> kG=createKrivuljeG();
	std::vector<drevo::Krivulja*> kW=createKrivuljeW();

	emit pointsUpdated(tipParametra, kG, kW);
}

std::vector<drevo::Krivulja*> DistributionDesigner::createKrivuljeG() const {
	std::vector<drevo::Krivulja*> krivulje;
	drevo::Krivulja *k = new drevo::Krivulja;
	krivulje.push_back(k);

	QPolygonF pol(graphG->points());
	QRectF rect=graphG->hoverPoints()->boundingRect();
	for (int i = 0; i < pol.size(); i++) {
		double x =pol[i].x()/rect.width();
		double y =1.0f-pol[i].y()/rect.height();
		krivulje[0]->kontrolneTocke.push_back(drevo::Tocka2D(x,y));
	}

	return krivulje;
}

std::vector<drevo::Krivulja*> DistributionDesigner::createKrivuljeW() const {
	std::vector<drevo::Krivulja*> krivulje;
	drevo::Krivulja *k = new drevo::Krivulja;
	krivulje.push_back(k);

	QPolygonF pol(graphW->points());
	QRectF rect=graphW->hoverPoints()->boundingRect();
	for (int i = 0; i < pol.size(); i++) {
		double x =pol[i].x()/rect.width();
		double y =1.0f-pol[i].y()/rect.height();
		krivulje[0]->kontrolneTocke.push_back(drevo::Tocka2D(x,y));
	}

	return krivulje;
}

QRectF DistributionDesigner::boundingRectG() const {
	return graphG->hoverPoints()->boundingRect();
}

QRectF DistributionDesigner::boundingRectW() const {
	return graphW->hoverPoints()->boundingRect();
}

void DistributionDesigner::applyHoverGChanged(const QPointF &point) {
	if (colorLevels->checkState()!=Qt::Checked) return;
	QRectF rect=graphG->hoverPoints()->boundingRect();
	int hovering=graphG->hoverPoints()->getCurrentIndex();
	int indexMin,indexMax;

	if (hovering>=0) {
		QPolygonF points(graphG->points());

		int hoverMin=hovering-1>0?hovering-1:0;
		int hoverMax=hovering+1<points.size()?hovering+1:points.size();

		indexMin = int(points[hoverMin].x()/rect.width()*15);
		indexMax = int(points[hoverMax].x()/rect.width()*15);
	} else {
		float x=1.0f*point.x()/rect.width();
		indexMin =indexMax = int(x * 15);
	}
	hoverIndex->setText(indexMin==indexMax?QString("G=%1").arg(indexMin):QString("G=%1-%2").arg(indexMin).arg(indexMax));

	emit hoverGChanged(indexMin, indexMax);
}

void DistributionDesigner::applyHoverWChanged(const QPointF &point) {
	if (colorLevels->checkState()!=Qt::Checked) return;
	QRectF rect=graphW->hoverPoints()->boundingRect();
	int hovering=graphW->hoverPoints()->getCurrentIndex();
	int indexMin,indexMax;

	if (hovering>=0) {
		QPolygonF points(graphW->points());

		int hoverMin=hovering-1>0?hovering-1:0;
		int hoverMax=hovering+1<points.size()?hovering+1:points.size();

		indexMin = int(points[hoverMin].x()/rect.width()*50);
		indexMax = int(points[hoverMax].x()/rect.width()*50);
	} else {
		float x=1.0f*point.x()/rect.width();
		indexMin =indexMax = int(x * 50);
	}
	hoverIndex->setText(indexMin==indexMax?QString("W=%1").arg(indexMin):QString("W=%1-%2").arg(indexMin).arg(indexMax));

	emit hoverWChanged(indexMin, indexMax);
}
