/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "ecosystemstatistics.h"
#include "krivulja.h"
#include <QPolygonF>
#include <QRectF>
#include <QSizeF>
#include <vector>
#include "hoverpoints.h"
#include "tocka2d.h"
#include "simulation/statistics/entitiesstatistics.h"
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

using namespace gui;

EcosystemStatistics::EcosystemStatistics( QWidget *parent, ecosystem::statistics::CEntitiesStatistics *entitiesStatistics )
		: QDialog( parent ), entitiesStatistics(entitiesStatistics) {
	setupUi( this );

	izbranaVrstaIndex=0;
	connect(izbranaVrsta, SIGNAL(currentIndexChanged(int)),
	        this, SLOT(izberiVrsto(int)));

	connect(graf1->hoverPoints(), SIGNAL(hoverChanged(const QPointF &)),
	        this, SLOT(hoverLivingSpeciesChanged(const QPointF &)));
	graf1->hoverPoints()->setPointSize(QSizeF(0,0));
	graf1->hoverPoints()->setEditable(false);

	nrPlantsLivingChanged();
}

EcosystemStatistics::~EcosystemStatistics() {}

void EcosystemStatistics::hoverLivingSpeciesChanged(const QPointF &point) {
	QRectF rect=graf1->hoverPoints()->boundingRect();

	float x=point.x();
	float y=exp((graf1->height()-point.y())/20.0f);
	currentValue->setText(QString("(%1, %2)").arg(x).arg(y));
}

void EcosystemStatistics::nrPlantsLivingChanged() {
	if (showOnlyRenderedOnes->checkState()==Qt::Checked)
		return;
	std::vector<drevo::Krivulja*> &grafiStevilaRastlin = *entitiesStatistics->getGrafiStevilaRastlin();

	QPolygonF tockeGrafa;
	QRectF rect=graf1->hoverPoints()->boundingRect();
	float lastX=0,lastY=0;
	for (int i=0; i < (int)grafiStevilaRastlin[izbranaVrstaIndex]->kontrolneTocke.size(); i++) {
		float x=grafiStevilaRastlin[izbranaVrstaIndex]->kontrolneTocke[i].x;
		float y=rect.height()-log(grafiStevilaRastlin[izbranaVrstaIndex]->kontrolneTocke[i].y)*20.0f;
		QPointF tocka(x,y);
		tockeGrafa<<tocka;

		lastX=x;
		lastY=grafiStevilaRastlin[izbranaVrstaIndex]->kontrolneTocke[i].y;
	}
	graf1->hoverPoints()->setPoints(tockeGrafa);

	currentValue->setText(QString("(%1, %2)").arg(lastX).arg(lastY));
	refreshStatistics();
}

void EcosystemStatistics::nrPlantsRenderedChanged() {
	if (showOnlyRenderedOnes->checkState()!=Qt::Checked)
		return;
	std::vector<drevo::Krivulja*> &grafiStevilaRastlin = *entitiesStatistics->getGrafiStevilaUpodobljenihRastlin();

	QPolygonF tockeGrafa;
	QRectF rect=graf1->hoverPoints()->boundingRect();
	float lastX=0,lastY=0;
	for (int i=0; i < (int)grafiStevilaRastlin[izbranaVrstaIndex]->kontrolneTocke.size(); i++) {
		float x=grafiStevilaRastlin[izbranaVrstaIndex]->kontrolneTocke[i].x;
		float y=rect.height()-log(grafiStevilaRastlin[izbranaVrstaIndex]->kontrolneTocke[i].y)*20.0f;

		QPointF tocka(x,y);
		tockeGrafa<<tocka;

		lastX=x;
		lastY=grafiStevilaRastlin[izbranaVrstaIndex]->kontrolneTocke[i].y;
	}
	graf1->hoverPoints()->setPoints(tockeGrafa);
	currentValue->setText(QString("(%1, %2)").arg(lastX).arg(lastY));

	refreshStatistics();
}

void EcosystemStatistics::refreshStatistics() {
	///zaenkrat skopiramo samo 1 vrsto
	///TODO: dodaj več grafov na widget!
	QPen pen;
	int t = izbranaVrstaIndex;
	int r = 0, g = 0, b = 0;
	switch ( t ) {
	case 0:
		r = 255;
		break;
	case 1:
		g = 255;
		break;
	case 2:
		b = 255;
		break;
	case 3:
		r = 255;
		b = 255;
		break;
	case 4:
		r = 255;
		g = 255;
		break;
	default:
		r = 255;
		g = 255;
		b = 255;
		break;
	}
	pen=QPen( QColor( r, g, b, 200 ), 2 );

	graf1->hoverPoints()->setConnectionPen( pen );
	graf1->update();
}

void EcosystemStatistics::izberiVrsto(int value) {
	izbranaVrstaIndex=value;

	/// osvežimo statistiko
	nrPlantsLivingChanged();
	nrPlantsRenderedChanged();
}
