/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef ECOSYSTEMSTATISTICS_H
#define ECOSYSTEMSTATISTICS_H

#include "ui_ecosystemstatistics.h"

#include <vector>

namespace drevo {
	class Krivulja;
}
namespace ecosystem {
	namespace statistics {
		class CEntitiesStatistics;
	}
}

namespace gui {
	/**
	 * Skrbi za prikaz statistike o simulaciji ekosistema.
	 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
	 */
	class EcosystemStatistics: public QDialog, private Ui::EcosystemStatistics {
		Q_OBJECT
	public:
		/**
		 * Ustvari nov objekt in inicializira prikazovano št. frejma na 0.
		 *
		 * @param parent očetovski objekt
		 */
		EcosystemStatistics( QWidget *parent, ecosystem::statistics::CEntitiesStatistics *CEntitiesStatistics);
		/**
		 * Pobriše dialog: pobriše tudi krivulje za grafe.
		 * @return
		 */
		virtual ~EcosystemStatistics();

	public slots:
		/**
		 * Osveži grafe za statistiko, brez dodajanja novih točk.
		 */
		void refreshStatistics();
		/**
		 * Izbere določeno vrsto za prikaz na grafu.
		 * @param value določena vrsta
		 */
		void izberiVrsto(int);
		/**
		 * Prikaže število rastlin na tem mestu, kjer smo hover.
		 */
		void hoverLivingSpeciesChanged(const QPointF &);
		/**
		 * Klican ob spremembi števila živečih rastlin v ekosistemu
		 */
		void nrPlantsLivingChanged();
		/**
		 * Klican ob spremembi števila upodobljenih rastlin
		 */
		void nrPlantsRenderedChanged();

	private:
		/**
		 * Indeks izbrane rastlinske vrste.
		 */
		int izbranaVrstaIndex;
		/**
		 * Kazalec na objekt za zbiranje statistike
		 */
		ecosystem::statistics::CEntitiesStatistics *entitiesStatistics;
	};
}
#endif
