/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "filehandler.h"
#include <QFileDialog>
#include <QMessageBox>
#include <QImage>
#include <QPixmap>
#include <QTextStream>
#include <QFile>
#include <QLocale>
#include "oglwidget.h"
#include "drevo.h"
#include "pixmapexportdialog.h"
#include "render/ecosystemrender.h"
#include "krivulja.h"
#include "skytextures.h"

#include <string>
#include <sstream>
#include <iomanip>


namespace gui {
	namespace actionhandling {

		FileHandler::FileHandler( QWidget *parentWindow, ecosystem::render::EcosystemRender * model, QGLWidget *qglWidget ) :
		parentWindow( parentWindow ), model( model ), qglWidget( qglWidget ) {}

		FileHandler::~FileHandler() {}

		void FileHandler::actionSet_sky_textures() {
			SkyTextures skyTextures(parentWindow, model);
			skyTextures.exec();
		}
		
		void FileHandler::actionLoad_DEM_terrain_model() {
			QString s = QFileDialog::getOpenFileName(
			                parentWindow,
			                tr( "Choose a file to open" ), tr( "" ),
			                tr( "DEM model (*.dmr)" ) );
			if ( s == 0 )
				return ;

			std::string name = s.toStdString();
			model->reloadDMR(name);
		}

		void FileHandler::actionLoad_parameters() {
			QString s = QFileDialog::getOpenFileName(
			                parentWindow,
			                tr( "Choose a file to open" ), tr( "" ),
			                tr( "Tree version 3.x parameters (*.tree.xml)" ) );
			if ( s == 0 )
				return ;

			std::string name = s.toStdString();
			bool result = model->getIzbranoDrevo() ->pdPrepared.loadParameters( name );
			if ( !result ) {
				( void ) QMessageBox::information( 0, tr( "Error" ),
				                                   tr( "Could not open file." ), QMessageBox::Ok );
			}

			emit reloadCurrentTreeTextures();
		}

		void FileHandler::actionImport_tree_2_x_parameters() {
			QString s = QFileDialog::getOpenFileName(
			                parentWindow,
			                tr( "Choose a file to import" ), tr( "" ),
			                tr( "Tree version 2.x parameters (*.tree)" ) );
			if ( s == 0 )
				return ;

			( void ) QMessageBox::information( 0, tr( "Information" ),
			                                   tr( "Not yet implemented." ), QMessageBox::Ok );
		}

		void FileHandler::actionSave_parameters() {
			QString s = QFileDialog::getSaveFileName(
			                parentWindow,
			                tr( "Choose a filename to save under" ), tr( "" ),
			                tr( "Tree version 3.x parameters (*.tree.xml)" ) );

			if ( s == 0 )
				return ;

			std::string name = s.toStdString();
			bool result = model->getIzbranoDrevo() ->pdPrepared.saveParameters( name );
			if ( !result ) {
				( void ) QMessageBox::information( 0, tr( "Error" ),
				                                   tr( "Could not open file." ), QMessageBox::Ok );
				return ;
			}
		}

		void FileHandler::actionExport_current_tree_geometry_as_vertices() {
			QString s = QFileDialog::getSaveFileName(
			                parentWindow,
			                tr( "Choose a filename to export under" ), tr( "" ),
			                tr( "Tree geometry vertices (*.crs)" ) );
			if ( s == 0 )
				return ;

			std::string name = s.toStdString();

			model->getIzbranoDrevo() ->markSerializingToFile( name );

			lastFileNameVertex = s;
		}

		void FileHandler::actionExport_window_image() {
			QString s = QFileDialog::getSaveFileName(
			                parentWindow,
			                tr( "Choose a filename to export under" ), tr( "" ),
			                tr( "Images (*.png *.xpm *.jpg *.bmp)" ) );
			if ( s == 0 )
				return ;

			//slika z Alpha kanalom
			QImage image = qglWidget->grabFrameBuffer ( true );
			image.save( s, "PNG" );

			lastFileNameImage = s;
		}

		void FileHandler::actionExport_custom_resolution_image() {
			QString s = QFileDialog::getSaveFileName(
			                parentWindow,
			                tr( "Choose a filename to export under" ), tr( "" ),
			                tr( "Images (*.png *.xpm *.jpg *.bmp)" ) );
			if ( s == 0 )
				return ;

			PixmapExportDialog pixmapExportDialog( parentWindow );
			if ( pixmapExportDialog.exec() == QDialog::Rejected )
				return ;
			int width = pixmapExportDialog.sbWidth->value();
			int height = pixmapExportDialog.sbHeight->value();
			int quality = pixmapExportDialog.sliderQuality->value();

			//ne uporabimo trenutnega GL konteksta (sistem Windows tega sploh ne omogoča)
			QPixmap pixmap = qglWidget->renderPixmap ( width, height, false );
			pixmap.save( s, "PNG", quality );
		}

		void FileHandler::actionAutosave_simulation() {
			model->setAutosave(30);
		}

		void FileHandler::grabAnimationFrame( int frameNr ) {
			if ( lastFileNameImage == "" )
				return ;
			///vzamemo sliko
			std::stringstream s;
			s << lastFileNameImage.toStdString();
			s << std::setfill( '0' ) << std::setw( 5 ) << frameNr;
			s << ".png";

			std::string imeDatoteke;
			imeDatoteke = s.str();
			///slika z Alpha kanalom
			QImage image = qglWidget->grabFrameBuffer ( true );
			QString ime;
			ime = imeDatoteke.c_str();
			image.save( ime, "PNG" );
		}

		void FileHandler::grabGeometryFrame( int frameNr ) {
			if ( lastFileNameVertex == "" )
				return ;
			///vzamemo točke
			std::stringstream s;
			s << lastFileNameVertex.toStdString();
			s << std::setfill( '0' ) << std::setw( 5 ) << frameNr;
			s << ".crs";

			std::string imeDatoteke;
			imeDatoteke = s.str();
			model->getIzbranoDrevo() ->markSerializingToFile( imeDatoteke );
		}

		void FileHandler::resetLastNames() {
			lastFileNameVertex = "";
			lastFileNameImage = "";
		}

		void FileHandler::actionLoad_bark_texture() {
			QString s = QFileDialog::getOpenFileName(
			                parentWindow,
			                tr( "Choose a file to open" ), tr( "" ),
			                tr( "Images (*.png *.xpm *.jpg *.bmp)" ) );
			if ( s == 0 )
				return ;

			std::string name = s.toStdString();
			model->getIzbranoDrevo() ->pdPrepared.barkTextureFileName = name;

			emit reloadCurrentTreeTextures();
		}

		void FileHandler::actionLoadleaf_texture() {
			QString s = QFileDialog::getOpenFileName(
			                parentWindow,
			                tr( "Choose a file to open" ), tr( "" ),
			                tr( "Images (*.png *.xpm *.jpg *.bmp)" ) );
			if ( s == 0 )
				return ;

			std::string name = s.toStdString();
			model->getIzbranoDrevo() ->pdPrepared.leafTextureFileName = name;

			emit reloadCurrentTreeTextures();
		}

		void FileHandler::actionSave_simulation() {
			if (!model->isSimulationRedyAndRunning()) {
				( void ) QMessageBox::information( 0, tr( "Error" ),
				                                   tr( "First run simulation please." ), QMessageBox::Ok );
				return;
			}
			QString s = QFileDialog::getSaveFileName(
			                parentWindow,
			                tr( "Choose a filename to save under" ), tr( "" ),
			                tr( "Simulation files (*.sim)" ) );
			if ( s == 0 )
				return ;

			std::string name = s.toStdString();
			model->saveSimulation(name);
		}

		void FileHandler::actionLoad_simulation() {
			if (!model->isSimulationRedyAndRunning()) {
				( void ) QMessageBox::information( 0, tr( "Error" ),
				                                   tr( "First run simulation please." ), QMessageBox::Ok );
				return;
			}
			QString s = QFileDialog::getOpenFileName(
			                parentWindow,
			                tr( "Choose a simulation file to open" ), tr( "" ),
			                tr( "Simulation files (*.sim)" ) );
			if ( s == 0 )
				return ;

			std::string name = s.toStdString();
			model->loadSimulation(name);
		}

		void FileHandler::actionExport_current_tree_as_Latex_formulas() {
			QString s = QFileDialog::getSaveFileName(
			                parentWindow,
			                tr( "Choose a filename to export under" ), tr( "" ),
			                tr( "Latex files (*.tex)" ) );
			if ( s == 0 )
				return ;

			QFile file( s );
			if ( !file.open( QIODevice::WriteOnly | QIODevice::Text ) ) {
				( void ) QMessageBox::information( 0, tr( "Error" ),
				                                   tr( "Could not save file." ), QMessageBox::Ok );
				return ;
			}

			QTextStream out( &file );
			out.setRealNumberPrecision(4);

			exportTreeAsLatexFormulas( out );
		}


		void FileHandler::exportTreeAsLatexFormulas( QTextStream &f ) {
			drevo::ParametriDrevesa & pd = model->getIzbranoDrevo() ->pdPrepared;

			f << "%\\documentclass[slovene,a4paper,12pt,twoside]{report}" << endl;
			f << "%\\usepackage[slovene]{babel}" << endl;
			f << "%\\selectlanguage{slovene}" << endl;
			f << "%\\usepackage[utf8]{inputenc}" << endl;
			f << "%\\inputencoding{utf8}" << endl;


			f << "%\\begin{document}" << endl;
			f << tr( "To model the tree we have chosen following scalar parameters: " ) << endl;

			f << "$S=" << replacePoint(pd.stVsehZil) << "$, ";
			f << "$L_0=" << replacePoint(pd.dolzinaDebla) << "$, ";
			f << "$k_d=" << replacePoint(pd.sirinaSteblaNaZilo) << "$, ";
			f << "$\\alpha_p=" << replacePoint(pd.phyllotaxisAngle) << "$, ";
			f << "$k_c=" << replacePoint(pd.graviCentralizem) << "$, ";
			f << "$l_l=" << replacePoint(pd.listSize) << "$, ";
			f << "$\\rho_l=" << replacePoint(pd.gostotaListov) << "$, " << endl;
			f << "$l_{type}=\\textit{";
			switch ( pd.tipListov ) {
			case 0:
				f << "Spiral";
				break;
			case 1:
				f << "Stacked";
				break;
			case 2:
				f << "Staggered";
				break;
			case 3:
				f << "Bunched";
				break;
			case 4:
				f << "Coniferous";
				break;
			}
			f << "}$, ";
			f << "$w_g=" << replacePoint(pd.windSpeed) << "$, ";
			f << "${\\bf w}=" << "[0\\;\\;1\\;\\;0]^T" << "$, ";
			f << "$k_f=" << "1" << "$ " << tr( " and " );
			f << "$l_{LOD}=" << "0" << "$." << endl << endl;


			/// vektorski parametri

			/// porazdelitvene parametre shranimo v obliki njihovih dvojnikov v grafu
			f << tr( "Branch distribution" )
			<< " $k_s^{g,w}$ " << tr( "is determined for all orders" ) << " ($g,w$)," << endl
			<< tr( "where" ) << " $g \\in [0,15]$, $w \\in [0,50]$ ($g,w$ "
			<< tr( "are nonnegative whole numbers)," ) << endl
			<< tr( "with equation \\ref{eqn:porazdelitevZilGW}. Values" )
			<< " $k_s^{g}$" << " in " << "$k_s^{w}$" << endl
			<< tr( "are calculated from both graphs, that are given with polylines" )
			<< " $k_s^g";
			shraniParameterKotLatex( f, pd.verjetnostiVejitveG, true, drevo::ParametriDrevesa::STRAND_DISTRIBUTION );
			f << tr( " and " ) << "$k_s^w";
			shraniParameterKotLatex( f, pd.verjetnostiVejitveW, false, drevo::ParametriDrevesa::STRAND_DISTRIBUTION );
			f << "." << endl << endl;

			f << tr( "Branching angle between dividing subbranches," ) << " $\\alpha^{g,w}$ "
			<< tr( ", is determined for all orders" ) << endl
			<< "$(g,w)$ " << tr( "using equation \\ref{eqn:kotVejitveGW}. Needed parameters" ) << endl
			<< tr( "in mentioned equation" ) << " $\\alpha^g$" << " in " << "$\\alpha^w$ "
			<< tr( "are defined using two graphs," ) << endl
			<< tr( "given as polylines" )
			<< " $\\alpha^g";
			shraniParameterKotLatex( f, pd.kotVejitveG, true, drevo::ParametriDrevesa::BRANCH_ANGLES_DISTRIBUTION );
			f << tr( " and " ) << "$\\alpha^w";
			shraniParameterKotLatex( f, pd.kotVejitveW, false, drevo::ParametriDrevesa::BRANCH_ANGLES_DISTRIBUTION );
			f << "." << endl << endl;

			f << tr( "Minimum relative branch length," ) << " $m^{g,w}$"
			<< tr( ", is determined for all orders" ) << endl
			<< " $(g,w)$" << tr( " using equation \\ref{eqn:minCharacteristicLengthGW}, substituting" ) << endl
			<< "$m^g";
			shraniParameterKotLatex( f, pd.minCharacteristicLengthG, true, drevo::ParametriDrevesa::MIN_CHARACTERISTIC_LENGTH );
			f << tr( " and " ) << "$m^w";
			shraniParameterKotLatex( f, pd.minCharacteristicLengthW, false, drevo::ParametriDrevesa::MIN_CHARACTERISTIC_LENGTH );
			f << "." << endl << endl;

			f << tr( "Maximum relative branch length," ) << " $M^{g,w}$"
			<< tr( ", is determined for all orders" ) << endl
			<< "$(g,w)$ " << tr( "using equation \\ref{eqn:maxCharacteristicLengthGW}, substituting" ) << endl
			<< "$M^g";
			shraniParameterKotLatex( f, pd.maxCharacteristicLengthG, true, drevo::ParametriDrevesa::MAX_CHARACTERISTIC_LENGTH );
			f << tr( " and " ) << "$M^w ";
			shraniParameterKotLatex( f, pd.maxCharacteristicLengthW, false, drevo::ParametriDrevesa::MAX_CHARACTERISTIC_LENGTH );
			f << "." << endl << endl;

			f << tr( "Branch length scaling factor," ) << " $k_l^{g,w}$"
			<< tr( ", is determined for all orders" ) << endl
			<< "$(g,w)$ " << tr( "using equation \\ref{eqn:lengthScalingGW}, substituting" ) << endl
			<< "$k_l^g";
			shraniParameterKotLatex( f, pd.lengthScalingG, true, drevo::ParametriDrevesa::LENGTH_SCALING );
			f << tr( " and " ) << "$k_l^w";
			shraniParameterKotLatex( f, pd.lengthScalingW, false, drevo::ParametriDrevesa::LENGTH_SCALING );
			f << "." << endl << endl;

			f << tr( "Gravimorphism impact," ) << " $\\alpha_m^{g,w}$"
			<< tr( ", is determined for all orders" ) << endl
			<< "$(g,w)$ " << tr( "using equation \\ref{eqn:gravimorphismGW}, substituting" ) << endl
			<< "$\\alpha_m^g";
			shraniParameterKotLatex( f, pd.gravimorphismImpactG, true, drevo::ParametriDrevesa::GRAVIMORPHISM_IMPACT );
			f << tr( " and " ) << "$\\alpha_m^w";
			shraniParameterKotLatex( f, pd.gravimorphismImpactW, false, drevo::ParametriDrevesa::GRAVIMORPHISM_IMPACT );
			f << "." << endl << endl;

			f << "%\\end{document}" << endl;
		}

		void FileHandler::shraniParameterKotLatex( QTextStream &f, const drevo::Krivulja &pk, bool isG, int paramType ) {
			int dolzina = ( int ) pk.kontrolneTocke.size();
			f << "=\\left\\lbrace\\right.";

			/// shranim vsako točko na grafu kot par (x,y)
			for ( int i = 0; i < dolzina; i++ ) {
				float index=0;

				if (isG)
					index=pk.kontrolneTocke[ i ].x*15;
				else
					index=pk.kontrolneTocke[ i ].x*50;
				float value=pk.kontrolneTocke[ i ].y;
				if (isG)
					value=getRealParameterMeaning(value, paramType);
				else
					value*=2;

				f << "(" << replacePoint(index) << ",\\;\\;" << replacePoint(value) ;

				if (isG && (paramType==drevo::ParametriDrevesa::BRANCH_ANGLES_DISTRIBUTION || paramType==drevo::ParametriDrevesa::GRAVIMORPHISM_IMPACT))
					f << "^\\circ";
				f << ")";
				if ( i != dolzina - 1 ) {
					f << "$," << endl << "$";
				}
			}

			f << "\\left.\\right\\rbrace $";
		}

		float FileHandler::getRealParameterMeaning(float value, int paramType) {
			if ( paramType == drevo::ParametriDrevesa::STRAND_DISTRIBUTION )
				value = value / 2 + 0.5;

			/// izračun poteka glede na tip parametra
			switch ( paramType ) {
			case drevo::ParametriDrevesa::STRAND_DISTRIBUTION:
				value=drevo::ParametriDrevesa::max2( drevo::ParametriDrevesa::min2( value, 1.0 ), 0.5 );
				break;
			case drevo::ParametriDrevesa::BRANCH_ANGLES_DISTRIBUTION:
				value=drevo::ParametriDrevesa::min2( value * 180, 180.0 );
				break;
			case drevo::ParametriDrevesa::MIN_CHARACTERISTIC_LENGTH:
				value=value * 10;
				break;
			case drevo::ParametriDrevesa::MAX_CHARACTERISTIC_LENGTH:
				value=value * 10;
				break;
			case drevo::ParametriDrevesa::LENGTH_SCALING:
				value=value * 10;
				break;
			case drevo::ParametriDrevesa::GRAVIMORPHISM_IMPACT:
				value=drevo::ParametriDrevesa::max2( drevo::ParametriDrevesa::min2( value * 360  - 180, 180.0 ), -180 );
				break;
			};
			return value;
		}

		QString FileHandler::replacePoint(float value) {
			QString str=QString::number(double(value), 'g', 4);

			QString loc=QLocale::system().name();
			if (loc=="sl_SI")
				str.replace(".", ",");

			return str;
		}
	}
}
