/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GUI_ACTIONHANDLINGFILEHANDLER_H
#define GUI_ACTIONHANDLINGFILEHANDLER_H

#include <QObject>
#include <QGLWidget>
#include <string>
class QString;
class QTextStream;

namespace drevo {
	class Krivulja;
}
namespace ecosystem {
namespace render{
	class EcosystemRender;
}
}

namespace gui {
	class QMainWindow;
	namespace actionhandling {
		/**
		 * Kontrolira akcije, vezane z obravnavno datotek.
		 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
		*/
		class FileHandler : public QObject {
			Q_OBJECT
		public:
			/**
			 * Ustvari objekt in shrani kazalec na urejan model.
			 * @param parentWindow Parent za modalna okna, npr. MainWindow
			 * @param model Model aplikacije -- komponenta za renderiranje ekosistema
			 * @param qglWidget Potreben pri dostopu do GLContexta za renderiranje v pixmap in zajem slike okna.
			 */
			FileHandler(QWidget *parentWindow, ecosystem::render::EcosystemRender * model, QGLWidget *qglWidget);
			/**
			 * Odstrani objekt, pobriše dialog.
			 * @return
			 */
			virtual ~FileHandler();

		signals:
			/**
			 * Sprožen, da naj se osvežijo teksture drevesa, saj so bile znova naložene.
			 */
			void reloadCurrentTreeTextures();

		public slots:
			/**
			 * Naloži teksture obzorja.
			 */
			void actionSet_sky_textures();
			/**
			 * Naloži DMR model terena iz datoteke.
			 */
			void actionLoad_DEM_terrain_model();
			/**
			 * Nalaganje parametrov drevesa iz datoteke
			 */
			void actionLoad_parameters();
			/**
			 * Shranjevanje parametrov drevesa v datoteko
			 */
			void actionSave_parameters();
			/**
			 * Importiranje 2.x parametrov (če nebodo ostali XML)
			 */
			void actionImport_tree_2_x_parameters();
			/**
			 * Izvozi sliko ekrana.
			 */
			void actionExport_window_image();
			/**
			 * Izvozi off-DC renderirano sliko.
			 */
			void actionExport_custom_resolution_image();
			/**
			 * Izvozi geometrijo trenutno izbranega drevesa
			 */
			void actionExport_current_tree_geometry_as_vertices();
			/**
			 * Shrani sliko v datoteko, z zaporedno številko
			 * @param frameNr zaporedna številka
			 */
			void grabAnimationFrame(int frameNr);
			/**
			 * Shrani geometrijo v datoteko z zaporedno številko
			 * @param frameNr zaporedna številka
			 */
			void grabGeometryFrame(int frameNr);
			/**
			 * Resetira imena zadnjih exportanih slik animacije
			 */
			void resetLastNames();

			/**
			 * Naloži teksturo skorje
			 */
			void actionLoad_bark_texture();
			/**
			 * Naloži teksturo listov
			 */
			void actionLoadleaf_texture();
			/**
			 * Izvozi parametre drevesa v datoteko kot Latex formule
			 */
			void actionExport_current_tree_as_Latex_formulas();

			/**
			 * Shrani simulacijo: lokacije in lastnosti rastlin in random generatorje.
			 */
			void actionSave_simulation();
			/**
			 * Naloži simulacijo: lokacije in lastnosti rastlin in random generatorje.
			 */
			void actionLoad_simulation();
			/**
			 * Določi samodejno shranjevanje simulacije.
			 */
			void actionAutosave_simulation();

		private:
			/**
			* Izvozi parametre drevesa v datoteko v obliki Latex formul.
			* @param f ime datoteke
			*/
			void exportTreeAsLatexFormulas(QTextStream &f);
			/**
			* Shrani kontrolne točke izbrane krivulje v izhodno datoteko, v obliki Latex točk.
			* @param f izhodna datoteka
			* @param pk izbrana krivulja
			* @param isG ali shranjujem parametre tipa G
			* @param paramType tip parametra
			*/
			void shraniParameterKotLatex(QTextStream &f, const drevo::Krivulja &pk, bool isG, int paramType);
			/**
			 * Vrne pravo (absolutno) številko, ki pomeni parameter
			 * @param value
			 * @param paramType
			 * @return
			 */
			float getRealParameterMeaning(float value, int paramType);

			/**
			 * Zamenja pike z vejicami v float številu.
			 * @param value float število
			 */
			QString replacePoint(float value);

			/**
			 * Ime datoteke za export geometrije kot ogljišč
			 */
			QString lastFileNameVertex;
			/**
			 * Ime datoteke za export rastrskih slik
			 */
			QString lastFileNameImage;

		private:
			/** Parent za modalna okna, npr. MainWindow */
			QWidget *parentWindow;

			/** Model aplikacije -- komponenta za renderiranje ekosistema */
			ecosystem::render::EcosystemRender * model;

			/** Potreben pri dostopu do GLContexta za renderiranje v pixmap in zajem slike okna. */
			QGLWidget *qglWidget;
		};
	}
}

#endif
