/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "mainwindow.h"
#include "oglwidget.h"
#include "about.h"
#include "help.h"
#include "ecosystemstatistics.h"
#include "render/ecosystemrender.h"

#include "filehandler.h"
#include "timekeeper.h"
#include "designhandler.h"
#include "viewhandler.h"
#include "navigationhandler.h"
#include "plantarea.h"
#include "randomgeneratorseedseditor.h"

#include <QtGui>
using namespace gui;

/**
* Creates.
* @param parent
* @return
*/
MainWindow::MainWindow( QWidget *parent )
		: QMainWindow( parent ) {
	setupUi( this );
	fileHandler=0;
	currentModel=0;
	currentModelNr=0;

	//widget naj sam kreira nov model
	oglWidget = new gui::OglWidget( dockWidget, 0 );
	connect(oglWidget, SIGNAL(newModelCreated(ecosystem::render::EcosystemRender
*)), this, SLOT(bindNewModel(ecosystem::render::EcosystemRender *)));
	connect(oglWidget, SIGNAL(newGlContextInitialized(OglWidget*)),
			this,SLOT(newGlContextInitialized(OglWidget*)));

	///odstranil dockwidget:
	//setCentralWidget( oglWidget );
	dockWidget->setWidget(oglWidget);
}

MainWindow::~MainWindow() {
	for (int i = 0; i < (int)models.size(); i++)
		delete models[i];
}

void MainWindow::newGlContextInitialized(OglWidget *oglWidget) {
	connect(timeKeeper, SIGNAL(updateOnlyGL()), oglWidget, SLOT(updateGL()));
	connect(designHandler, SIGNAL(forceRepaint()), oglWidget, SLOT(updateGL()));
	connect(viewHandler, SIGNAL(forceRepaint()), oglWidget, SLOT(updateGL()));
	connect(navigationHandler, SIGNAL(forceRefresh()), oglWidget,
SLOT(updateGL()));
}

void MainWindow::bindNewModel(ecosystem::render::EcosystemRender *model) {
	models.push_back(model);
	currentModel = model;

	///FILE MENU
	///ne izbrišemo: if (fileHandler) delete fileHandler;
	/// parentwindow=mainwindow, model=ecosystemRender, qglWidget=OglWidget
	fileHandler = new actionhandling::FileHandler(this, currentModel, oglWidget);
	connect(fileHandler, SIGNAL(reloadCurrentTreeTextures()), currentModel, SLOT(reloadCurrentTreeTextures()));

#define trigg(x,y) connect( x, SIGNAL( triggered() ), y, SLOT( x()  ) );
	trigg(actionLoad_parameters,fileHandler);
	trigg(actionSave_parameters,fileHandler);
	trigg(actionImport_tree_2_x_parameters,fileHandler);
	trigg(actionLoad_DEM_terrain_model,fileHandler);
	trigg(actionExport_window_image,fileHandler);
	trigg(actionExport_custom_resolution_image,fileHandler);
	trigg(actionExport_current_tree_geometry_as_vertices,fileHandler);

	trigg(actionLoad_bark_texture,fileHandler);
	trigg(actionLoadleaf_texture,fileHandler);
	trigg(actionExport_current_tree_as_Latex_formulas,fileHandler);

	trigg(actionLoad_simulation,fileHandler);
	trigg(actionSave_simulation,fileHandler);
	trigg(actionAutosave_simulation,fileHandler);
	trigg(actionSet_sky_textures,fileHandler);

	timeKeeper = new actionhandling::TimeKeeper(this, currentModel);
	connect(timeKeeper, SIGNAL(updateSimulationAndGL()), currentModel,
SLOT(simulateNextStepAndUpdateGL()));

	connect(actionRun_simulation, SIGNAL(triggered()), timeKeeper, SLOT(toggleRunSimulation()));
	connect(actionAnimation_rendering_mode_Sequence_export, SIGNAL(triggered()), timeKeeper, SLOT(toggleAnimationExportingMode()));

	connect(timeKeeper, SIGNAL(grabFrameBuffer(int)), fileHandler, SLOT(grabAnimationFrame(int)));
	connect(timeKeeper, SIGNAL(grabGeometry(int)), fileHandler, SLOT(grabGeometryFrame(int)));
	connect(timeKeeper, SIGNAL(sequenceExportStopped()), fileHandler, SLOT(resetLastNames()));

	///PROPERTIES MENU
	connect(actionSelect_next_tree_parameters_tree_edit_mode, SIGNAL(triggered()), currentModel,SLOT(selectNextTreeSpecies()));
	trigg(actionEnlarge_leave_size, currentModel);
	trigg(actionReduce_leave_size, currentModel);

	trigg(actionGrow_tree, currentModel);
	trigg(actionSpin_tree, currentModel);
	trigg(actionSway_tree_in_wind, currentModel);
	connect(currentModel, SIGNAL(startTreeAnimation()), timeKeeper, SLOT(startTreeAnimation()));
	connect(currentModel, SIGNAL(stopTreeAnimation()), timeKeeper, SLOT(stopTreeAnimation()));

	trigg(actionRender_more_often_skip_less_frames, currentModel);
	trigg(actionRender_less_often_skip_more_frames, currentModel);

	designHandler = new actionhandling::DesignHandler(this, this, currentModel);
	trigg(actionDesign_strand_distribution, designHandler);
	trigg(actionDesign_branching_angles, designHandler);
	trigg(actionDesign_branch_absolute_minimal_length, designHandler);
	trigg(actionDesign_branch_absolute_maximal_length, designHandler);
	trigg(actionDesign_branch_lenght_scaling, designHandler);
	trigg(actionDesign_gravimorphism_impact_distribution, designHandler);
	trigg(actionAdvanced_tree_properties, designHandler);

	viewHandler = new actionhandling::ViewHandler(this, currentModel);

	trigg(actionShow_ecosystem, viewHandler);
	trigg(actionDisplay_leaves, viewHandler);
	trigg(actionColour_uppon, viewHandler);
	trigg(actionRender_wireframe, viewHandler);
	trigg(actionHigh_quality_rendering_more_polygons, viewHandler);
	trigg(actionLighting_enabled, viewHandler);
	trigg(actionDisplay_background_sky, viewHandler);
	trigg(actionShow_frame_rate, viewHandler);
	trigg(actionDo_not_draw_trees_ecosystem_mode, viewHandler);
	trigg(actionDo_not_refresh_display_only_run_simulation_ecosystem_mode, viewHandler);
	trigg(actionShow_ecologic_neighborhood_ecosystem_mode, viewHandler);
	trigg(actionDisplay_terrain_normals, viewHandler);
	trigg(actionDisplay_terrain_inclination, viewHandler);
	trigg(actionDisplay_terrain_moisture, viewHandler);
	trigg(actionDisplay_terrain_sunniness, viewHandler);
	trigg(actionDisplay_terrain_windiness, viewHandler);
	trigg(actionDisplay_terrain_land_markings_S_N_E_W, viewHandler);
	trigg(actionDisplay_sun, viewHandler);

	navigationHandler = new actionhandling::NavigationHandler(this, this,
currentModel);
	trigg(action_Navigation_panel, navigationHandler);
	trigg(actionSwitch_to_next_predefined_camera, navigationHandler);
	trigg(actionReset_camera_view, navigationHandler);
	trigg(actionMove_up, navigationHandler);
	trigg(actionMove_down, navigationHandler);
	trigg(actionMove_up_more, navigationHandler);
	trigg(actionMove_down_more, navigationHandler);
	trigg(actionMove_up_less, navigationHandler);
	trigg(actionMove_down_less, navigationHandler);
	trigg(actionMove_left, navigationHandler);
	trigg(actionMove_left_more, navigationHandler);
	trigg(actionMove_left_less, navigationHandler);
	trigg(actionMove_right, navigationHandler);
	trigg(actionMove_right_more, navigationHandler);
	trigg(actionMove_right_less, navigationHandler);
	trigg(actionMove_forward, navigationHandler);
	trigg(actionMove_backward, navigationHandler);
	trigg(actionRotate_left, navigationHandler);
	trigg(actionRotate_right, navigationHandler);
	trigg(actionRotate_up, navigationHandler);
	trigg(actionRotate_down, navigationHandler);
	trigg(actionRotate_in, navigationHandler);
	trigg(actionRotate_out, navigationHandler);
	trigg(actionMove_forward_more, navigationHandler);
	trigg(actionMove_backward_more, navigationHandler);
	trigg(actionMove_forward_less, navigationHandler);
	trigg(actionMove_backward_less, navigationHandler);
	trigg(actionForce_camera_position_at_terrain_surface_ecosystem_mode, navigationHandler);
	trigg(actionKeep_camera_position_above_terrain_surface_ecosystem_mode, navigationHandler);
	trigg(actionSave_camera_as_predefined, navigationHandler);

	connect(navigationHandler, SIGNAL(startCameraAnimation()),
			timeKeeper, SLOT(startCameraAnimation()));
	connect(navigationHandler, SIGNAL(stopCameraAnimation()),
			timeKeeper, SLOT(stopCameraAnimation()));
	connect(timeKeeper, SIGNAL(animateCamera()),
			navigationHandler, SLOT(animateCamera()));
#undef trigg
}

void MainWindow::on_actionAdd_view_triggered() {
	QDockWidget* dock1=new QDockWidget(centralwidget);
	gui::OglWidget* oglWidget1 = new gui::OglWidget( dock1, currentModel );
	/// ta se tako in tako ne izvede:
	/*connect(oglWidget1,
	SIGNAL(newModelCreated(ecosystem::render::EcosystemRender *)), this,
	SLOT(bindNewModel(ecosystem::render::EcosystemRender *)));*/
	connect(oglWidget1, SIGNAL(grabCamera()), this, SLOT(cycleView()));
	connect(oglWidget1, SIGNAL(newGlContextInitialized(gui::OglWidget*)),
			this, SLOT(newGlContextInitialized(gui::OglWidget *)));
	dock1->setWidget(oglWidget1);
	vboxLayout->addWidget(dock1);
}

void MainWindow::cycleView() {
}

void MainWindow::on_actionShow_statistics_triggered() {
	if (!currentModel->isSimulationRedyAndRunning()) {
		( void ) QMessageBox::information( 0, tr( "Error" ),
		tr( "First run simulation please." ), QMessageBox::Ok );
		return;
	}

	EcosystemStatistics *ecosystemStatistics = new EcosystemStatistics ( this, currentModel->getEntitiesStatistics() );
	connect(currentModel, SIGNAL(nrPlantsLivingChanged()),
			ecosystemStatistics, SLOT(nrPlantsLivingChanged()));
	connect(currentModel, SIGNAL(nrPlantsRenderedChanged()),
			ecosystemStatistics, SLOT(nrPlantsLivingChanged()));
	ecosystemStatistics->show();
}

void MainWindow::on_actionUsage_triggered() {
	Help help( this );
	help.exec();
}

void MainWindow::on_actionAbout_triggered() {
	About about( this );
	about.exec();
}

void MainWindow::on_actionSet_plant_area_triggered() {
	PlantArea plantArea( this );
	plantArea.minX->setValue(currentModel->getLupaMinX());
	plantArea.maxX->setValue(currentModel->getLupaMaxX());
	plantArea.minY->setValue(currentModel->getLupaMinY());
	plantArea.maxY->setValue(currentModel->getLupaMaxY());

	connect (plantArea.minX, SIGNAL(valueChanged(int)), currentModel, SLOT(setLupaMinX(int)));
	connect (plantArea.maxX, SIGNAL(valueChanged(int)), currentModel, SLOT(setLupaMaxX(int)));
	connect (plantArea.minY, SIGNAL(valueChanged(int)), currentModel, SLOT(setLupaMinY(int)));
	connect (plantArea.maxY, SIGNAL(valueChanged(int)), currentModel, SLOT(setLupaMaxY(int)));

	plantArea.exec();
}

void MainWindow::on_actionEdit_random_generator_seeds_triggered() {
	RandomGeneratorSeedsEditor editor(this);
	connect(&editor, SIGNAL(updateSeeds(const std::vector<unsigned long long>&)),
			 currentModel, SLOT(setRandomGeneratorSeeds(const std::vector<unsigned long long>&)));
	editor.setSeeds( currentModel->getRandomGeneratorSeeds());
	editor.exec();
}
