/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   159 Temple Place - Suite 3hugeMoveStep, Boston, MA  02111-1hugeMoveStep7, USA.             *
 ***************************************************************************/
#include "navigationhandler.h"

#include "navigationpanel.h"
#include "drevo.h"
#include "render/ecosystemrender.h"
#include "render/camera.h"
#include "simulation/data/terraindata.h"
#include "tocka.h"
#ifdef _WIN32
#define _USE_MATH_DEFINES
#endif
#include <math.h>

namespace gui {

	namespace actionhandling {

		NavigationHandler::NavigationHandler(QObject *parent, QWidget *parentWindow, ecosystem::render::EcosystemRender * model)
				: QObject(parent), parentWindow(parentWindow), model(model)  {
			navigationPanel=0;

			fineMoveStep=0.1f;
			ordinalMoveStep=1;
			hugeMoveStep=30.0f;
			normalRotateStep=1;

			treeMoveHugeStep=10;

			hitrostiPremikaRelative=drevo::Tocka(0,0,0);
			hitrostiZasuka=drevo::Tocka(0,0,0);
		}

		NavigationHandler::~NavigationHandler() {}

		void NavigationHandler::action_Navigation_panel() {
			navigationPanel = new NavigationPanel(parentWindow);
			updateNavigationPanelValues();

			connect(navigationPanel->X, SIGNAL(valueChanged(int)), this, SLOT(absoluteMoveX(int)));
			connect(navigationPanel->Y, SIGNAL(valueChanged(int)), this, SLOT(absoluteMoveY(int)));
			connect(navigationPanel->Z, SIGNAL(valueChanged(int)), this, SLOT(absoluteMoveZ(int)));

			connect(navigationPanel->Turn, SIGNAL(valueChanged(int)), this, SLOT(absoluteMoveTurn(int)));
			connect(navigationPanel->Roll, SIGNAL(valueChanged(int)), this, SLOT(absoluteMoveRoll(int)));
			connect(navigationPanel->Yow, SIGNAL(valueChanged(int)), this, SLOT(absoluteMoveYow(int)));

			connect(navigationPanel->TravelForward, SIGNAL(pressed()), this, SLOT(actionMove_forward()));
			connect(navigationPanel->TravelBackward, SIGNAL(pressed()), this, SLOT(actionMove_backward()));

			connect(this, SIGNAL(forceRefresh()), this, SLOT(updateNavigationPanelValues()));

			navigationPanel->show();
		}

		void NavigationHandler::updateNavigationPanelValues() {
			navigationPanel->X->setValue(int(model->getCurrentCamera()->x));
			navigationPanel->Y->setValue(int(model->getCurrentCamera()->y));
			navigationPanel->Z->setValue(int(model->getCurrentCamera()->z));
			navigationPanel->Turn->setValue(int(model->getCurrentCamera()->alfa));
			navigationPanel->Roll->setValue(int(model->getCurrentCamera()->beta));
			navigationPanel->Yow->setValue(int(model->getCurrentCamera()->gama));
			navigationPanel->dmr->setText(QString("(%1, %2, %3)")
			                              .arg(model->getCurrentCamera()->x+model->getTerrainData()->dmrMinX)
			                              .arg(model->getCurrentCamera()->z+model->getTerrainData()->dmrMinY)
			                              .arg(model->getCurrentCamera()->y+model->getTerrainData()->dmrMinZ));
		}

		void NavigationHandler::absoluteMoveX(int value) {
			model->getCurrentCamera()->x=value;
			emit forceRefresh();
		}

		void NavigationHandler::absoluteMoveY(int value) {
			model->getCurrentCamera()->y=value;
			emit forceRefresh();
		}

		void NavigationHandler::absoluteMoveZ(int value) {
			model->getCurrentCamera()->z=value;
			emit forceRefresh();
		}

		void NavigationHandler::absoluteMoveTurn(int value) {
			model->getCurrentCamera()->alfa=value;
			emit forceRefresh();
		}

		void NavigationHandler::absoluteMoveRoll(int value) {
			model->getCurrentCamera()->beta=value;
			emit forceRefresh();
		}

		void NavigationHandler::absoluteMoveYow(int value) {
			model->getCurrentCamera()->gama=value;
			emit forceRefresh();
		}

		void NavigationHandler::actionSwitch_to_next_predefined_camera() {
			if (model->isSingleRender()) {}
			else {
				model->getCurrentCamera()->switchEcosystemCamera();
				hitrostiPremikaRelative=drevo::Tocka(0,0,0);
				hitrostiZasuka=drevo::Tocka(0,0,0);

				emit forceRefresh();
			}
		}

		void NavigationHandler::actionReset_camera_view() {
			model->getCurrentCamera()->reset();
			emit forceRefresh();
		}

		void NavigationHandler::actionMove_up() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				hitrostiPremikaRelative.y+=ordinalMoveStep;
				emit startCameraAnimation();
			}
		}

		void NavigationHandler::actionMove_down() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				hitrostiPremikaRelative.y+=-ordinalMoveStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionMove_up_more() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				hitrostiPremikaRelative.y += hugeMoveStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionMove_down_more() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				hitrostiPremikaRelative.y += -hugeMoveStep;
				emit startCameraAnimation();
			}
		}

		void NavigationHandler::actionMove_up_less() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				hitrostiPremikaRelative.y += fineMoveStep;
				emit startCameraAnimation();
			}
		}

		void NavigationHandler::actionMove_down_less() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				hitrostiPremikaRelative.y += -fineMoveStep;
				emit startCameraAnimation();
			}
		}

		void NavigationHandler::actionMove_forward() {
			if (model->isSingleRender()) {
				model->getCurrentCamera()->dol_pomik+=ordinalMoveStep;
				emit forceRefresh();
			} else {
				hitrostiPremikaRelative.x += model->getCurrentCamera()->lx*ordinalMoveStep;
				hitrostiPremikaRelative.z += model->getCurrentCamera()->lz*ordinalMoveStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionMove_backward() {
			if (model->isSingleRender()) {
				model->getCurrentCamera()->dol_pomik-=ordinalMoveStep;
				emit forceRefresh();
			} else {
				hitrostiPremikaRelative.x += -model->getCurrentCamera()->lx*ordinalMoveStep;
				hitrostiPremikaRelative.z += -model->getCurrentCamera()->lz*ordinalMoveStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionMove_forward_more() {
			if (model->isSingleRender()) {
				model->getCurrentCamera()->dol_pomik+=treeMoveHugeStep;
				emit forceRefresh();
			} else {
				hitrostiPremikaRelative.x += model->getCurrentCamera()->lx*hugeMoveStep;
				hitrostiPremikaRelative.z += model->getCurrentCamera()->lz*hugeMoveStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionMove_backward_more() {
			if (model->isSingleRender()) {
				model->getCurrentCamera()->dol_pomik-=treeMoveHugeStep;
				emit forceRefresh();
			} else {
				hitrostiPremikaRelative.x += -model->getCurrentCamera()->lx*hugeMoveStep;
				hitrostiPremikaRelative.z += -model->getCurrentCamera()->lz*hugeMoveStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionMove_forward_less() {
			if (model->isSingleRender()) {
				model->getCurrentCamera()->dol_pomik+=fineMoveStep;
				emit forceRefresh();
			} else {
				hitrostiPremikaRelative.x += model->getCurrentCamera()->lx*fineMoveStep;
				hitrostiPremikaRelative.z += model->getCurrentCamera()->lz*fineMoveStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionMove_backward_less() {
			if (model->isSingleRender()) {
				model->getCurrentCamera()->dol_pomik-=fineMoveStep;
				emit forceRefresh();
			} else {
				hitrostiPremikaRelative.x += -model->getCurrentCamera()->lx*fineMoveStep;
				hitrostiPremikaRelative.z += -model->getCurrentCamera()->lz*fineMoveStep;
				emit startCameraAnimation();
			}
		}

		void NavigationHandler::actionMove_left() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				drevo::Tocka tL(model->getCurrentCamera()->lx,model->getCurrentCamera()->ly,model->getCurrentCamera()->lz);
				drevo::Tocka tUp(0,1,0);
				drevo::Tocka strafe=drevo::Tocka::vektorskiProd(tL,tUp);

				hitrostiPremikaRelative.x+=-strafe.x*ordinalMoveStep;
				hitrostiPremikaRelative.z+=-strafe.z*ordinalMoveStep;
				emit startCameraAnimation();
			}
		}

		void NavigationHandler::actionMove_left_more() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				drevo::Tocka tL(model->getCurrentCamera()->lx,model->getCurrentCamera()->ly,model->getCurrentCamera()->lz);
				drevo::Tocka tUp(0,1,0);
				drevo::Tocka strafe=drevo::Tocka::vektorskiProd(tL,tUp);

				hitrostiPremikaRelative.x+=-strafe.x*hugeMoveStep;
				hitrostiPremikaRelative.z+=-strafe.z*hugeMoveStep;
				emit startCameraAnimation();
			}
		}

		void NavigationHandler::actionMove_left_less() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				drevo::Tocka tL(model->getCurrentCamera()->lx,model->getCurrentCamera()->ly,model->getCurrentCamera()->lz);
				drevo::Tocka tUp(0,1,0);
				drevo::Tocka strafe=drevo::Tocka::vektorskiProd(tL,tUp);

				hitrostiPremikaRelative.x+=-strafe.x*fineMoveStep;
				hitrostiPremikaRelative.z+=-strafe.z*fineMoveStep;
				emit startCameraAnimation();
			}
		}

		void NavigationHandler::actionMove_right() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				drevo::Tocka tL(model->getCurrentCamera()->lx,model->getCurrentCamera()->ly,model->getCurrentCamera()->lz);
				drevo::Tocka tUp(0,1,0);
				drevo::Tocka strafe=drevo::Tocka::vektorskiProd(tL,tUp);

				hitrostiPremikaRelative.x+=strafe.x*ordinalMoveStep;
				hitrostiPremikaRelative.z+=strafe.z*ordinalMoveStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionMove_right_more() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				drevo::Tocka tL(model->getCurrentCamera()->lx,model->getCurrentCamera()->ly,model->getCurrentCamera()->lz);
				drevo::Tocka tUp(0,1,0);
				drevo::Tocka strafe=drevo::Tocka::vektorskiProd(tL,tUp);

				hitrostiPremikaRelative.x+=strafe.x*hugeMoveStep;
				hitrostiPremikaRelative.z+=strafe.z*hugeMoveStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionMove_right_less() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				drevo::Tocka tL(model->getCurrentCamera()->lx,model->getCurrentCamera()->ly,model->getCurrentCamera()->lz);
				drevo::Tocka tUp(0,1,0);
				drevo::Tocka strafe=drevo::Tocka::vektorskiProd(tL,tUp);

				hitrostiPremikaRelative.x+=strafe.x*fineMoveStep;
				hitrostiPremikaRelative.z+=strafe.z*fineMoveStep;
				emit startCameraAnimation();
			}
		}

		void NavigationHandler::actionRotate_left() {
			if (model->isSingleRender()) {
				model->getCurrentCamera()->levo_pomik+=ordinalMoveStep;
				emit forceRefresh();
			} else {
				hitrostiZasuka.x+=-normalRotateStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionRotate_right() {
			if (model->isSingleRender()) {
				model->getCurrentCamera()->levo_pomik-=ordinalMoveStep;
				emit forceRefresh();
			} else {
				hitrostiZasuka.x+=normalRotateStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionRotate_up() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				hitrostiZasuka.y+=-normalRotateStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionRotate_down() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				hitrostiZasuka.y+=normalRotateStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionRotate_in() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				hitrostiZasuka.z+=-normalRotateStep;
				emit startCameraAnimation();
			}
		}
		void NavigationHandler::actionRotate_out() {
			if (model->isSingleRender()) {
				emit forceRefresh();
			} else {
				hitrostiZasuka.z+=normalRotateStep;
				emit startCameraAnimation();
			}
		}

		void NavigationHandler::actionForce_camera_position_at_terrain_surface_ecosystem_mode() {
			model->getCurrentCamera()->atSurface=!model->getCurrentCamera()->atSurface;
			emit forceRefresh();
		}

		void NavigationHandler::actionKeep_camera_position_above_terrain_surface_ecosystem_mode() {
			model->getCurrentCamera()->aboveSurface=!model->getCurrentCamera()->aboveSurface;
			emit forceRefresh();
		}

		void NavigationHandler::actionSave_camera_as_predefined() {
			model->getCurrentCamera()->saveCamera();
		}

		void NavigationHandler::animateCamera() {
			float stopnjaDusitve=0.9f;
			float epsilonAlpha=0.5;
			hitrostiPremikaRelative.x*=stopnjaDusitve;
			model->getCurrentCamera()->x += hitrostiPremikaRelative.x;
			hitrostiPremikaRelative.z*=stopnjaDusitve;
			model->getCurrentCamera()->z += hitrostiPremikaRelative.z;
			hitrostiPremikaRelative.y*=stopnjaDusitve;
			model->getCurrentCamera()->y += hitrostiPremikaRelative.y;

			hitrostiZasuka.x*=stopnjaDusitve;
			model->getCurrentCamera()->alfa += hitrostiZasuka.x;
			hitrostiZasuka.y*=stopnjaDusitve;
			model->getCurrentCamera()->beta += hitrostiZasuka.y;
			hitrostiZasuka.z*=stopnjaDusitve;
			model->getCurrentCamera()->gama += hitrostiZasuka.z;

			if (fabs(hitrostiPremikaRelative.x)<epsilonAlpha &&
			        fabs(hitrostiPremikaRelative.y)<epsilonAlpha &&
			        fabs(hitrostiPremikaRelative.z)<epsilonAlpha &&
			        fabs(hitrostiZasuka.x)<epsilonAlpha &&
			        fabs(hitrostiZasuka.y)<epsilonAlpha &&
			        fabs(hitrostiZasuka.z)<epsilonAlpha) {
				hitrostiPremikaRelative=drevo::Tocka(0,0,0);
				hitrostiZasuka=drevo::Tocka(0,0,0);
				emit stopCameraAnimation();
			}
			emit forceRefresh();
		}
	}

}
