/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GUI_ACTIONHANDLINGNAVIGATIONHANDLER_H
#define GUI_ACTIONHANDLINGNAVIGATIONHANDLER_H

#include <QObject>
#include "tocka.h"

namespace ecosystem {
	namespace render{
		class EcosystemRender;
	}
}
namespace gui {
	class NavigationPanel;
	namespace actionhandling {

		/**
		 * Obravnava akcije za navigacijo kamere po ekosistemu
		 * ali drevesa v pogledu samega drevesa.
		 *
		 * Nastavlja pogled na drevo, pri čemer premika drevo,
		 * ali nastavlja kamero ekosistema.
		 *
		 * Kamero ekosistema premika ali takojšnje in absolutno
		 * preko navigacijske plošče, ali pa s smernimi tipkami,
		 * relativno in gladko. Korake premikov določajo čisto
		 * spodaj določeni atributi objekta.
		 *
		 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
		*/
		class NavigationHandler : public QObject {
			Q_OBJECT
		public:
			/**
			 * Ustvari objekt, shrani podane kazalce
			 * @param parent očetovski objekt, skrbi za brisanje
			 * @param parentWindow vrhnje okno
			 * @param model model, ki ga spreminjamo
			 */
			NavigationHandler(QObject *parent, QWidget *parentWindow, ecosystem::render::EcosystemRender * model);
			/**
			 * Odstrani objekt
			 */
			~NavigationHandler();

		signals:
			/**
			 * Zahteva ponoven izris OpenGL.
			 */
			void forceRefresh();
			/**
			 * Zahteva začetek animiranja premika kamere.
			 */
			void startCameraAnimation();
			/**
			 * Zahteva konec animiranja premika kamere.
			 */
			void stopCameraAnimation();

		public slots:
			/**
			 * Prikaže navigacijsko ploščo.
			 */
			void action_Navigation_panel();
			/**
			 * Nastavi prikazane vrednosti v navigacijski plošči
			 */
			void updateNavigationPanelValues();
			/**
			 * Premakne kamero na absolutno lokacijo X.
			 * @param value absolutna lokacija X
			 */
			void absoluteMoveX(int value);
			/**
			 * Premakne kamero na absolutno lokacijo y.
			 * @param value absolutna lokacija Y
			 */
			void absoluteMoveY(int value);
			/**
			 * Premakne kamero na absolutno lokacijo Z.
			 * @param value absolutna lokacija Z
			 */
			void absoluteMoveZ(int value);
			/**
			 * Zavrti kamero na absolutni kot okoli osi Y
			 * @param value kot okoli osi Y
			 */
			void absoluteMoveTurn(int value);
			/**
			 * Zavrti kamero na absolutni kot okoli osi X
			 * @param value kot okoli osi X
			 */
			void absoluteMoveRoll(int value);
			/**
			 * Zavrti kamero na absolutni kot okoli osi Z
			 * @param value kot okoli osi Z
			 */
			void absoluteMoveYow(int value);

			/**
			 * Preklopi na naslednjo predefinirano kamero
			 */
			void actionSwitch_to_next_predefined_camera();
			/**
			 * Ponastavi parametre kamere.
			 */
			void actionReset_camera_view();
			/**
			 * Premakne kamero gor navadno.
			 */
			void actionMove_up();
			/**
			 * Premakne kamero dol navadno.
			 */
			void actionMove_down();
			/**
			 * Premakne kamero gor, bolj.
			 */
			void actionMove_up_more();
			/**
			 * Premakne kamero dol, bolj.
			 */
			void actionMove_down_more();
			/**
			 * Premakne kamero gor, manj.
			 */
			void actionMove_up_less();
			/**
			 * Premakne kamero dol, manj.
			 */
			void actionMove_down_less();
			/**
			 * Premakne kamero levo navadno.
			 */
			void actionMove_left();
			/**
			 * Premakne kamero levo, bolj.
			 */
			void actionMove_left_more();
			/**
			 * Premakne kamero levo, manj.
			 */
			void actionMove_left_less();
			/**
			 * Premakne kamero desno navadno.
			 */
			void actionMove_right();
			/**
			 * Premakne kamero desno bolj.
			 */
			void actionMove_right_more();
			/**
			 * Premakne kamero desno manj.
			 */
			void actionMove_right_less();
			/**
			 * Premakne kamero najprej navadno.
			 */
			void actionMove_forward();
			/**
			 * Premakne kamero nazaj navadno.
			 */
			void actionMove_backward();
			/**
			 * Premakne kamero naprej manj.
			 */
			void actionMove_forward_less();
			/**
			 * Premakne kamero nazaj manj.
			 */
			void actionMove_backward_less();

			/**
			 * Suče kamero levo.
			 */
			void actionRotate_left();
			/**
			 * Suče kamero desno.
			 */
			void actionRotate_right();
			/**
			 * Suče kamero gor.
			 */
			void actionRotate_up();
			/**
			 * Suče kamero dol.
			 */
			void actionRotate_down();
			/**
			 * Suče kamero notri.
			 */
			void actionRotate_in();
			/**
			 * Suče kamero ven.
			 */
			void actionRotate_out();
			/**
			 * Premakne kamero naprej, bolj.
			 */
			void actionMove_forward_more();
			/**
			 * Premakne kamero nazaj, bolj.
			 */
			void actionMove_backward_more();
			/**
			 * Shrani ohranjanje položaja kamere na površju terena
			 */
			void actionForce_camera_position_at_terrain_surface_ecosystem_mode();
			/**
			 * Shrani ohranjanje položaja kamera nad površjem terena po y
			 * in znotraj boundingboxa po xz.
			 */
			void actionKeep_camera_position_above_terrain_surface_ecosystem_mode();
			/**
			 * Shrani trenutno kamero.
			 */
			void actionSave_camera_as_predefined();
			/**
			 * Animira kamero, tako da prišteje že prej določene vektorje
			 */
			void animateCamera();

		private:
			/**
			 * Vrhnje okno za prikaz plošče
			 */
			QWidget *parentWindow;
			/**
			 * Model, ki ga nastavljamo
			 */
			ecosystem::render::EcosystemRender * model;
			/**
			 * Navigacijska plošča za nastavljanje kamere
			 */
			NavigationPanel *navigationPanel;

			/**
			 * Korak manj za fini premik kamere
			 */
			float fineMoveStep;
			/**
			 * Korak navadno za privzet premik kamere
			 */
			float ordinalMoveStep;
			/**
			 * Korak bolj za velik premik kamere
			 */
			float hugeMoveStep;
			/**
			 * Korak navaden za navaden zasuk kamere
			 */
			float normalRotateStep;
			/**
			 * Korak navaden za navaden premik drevesa (način single tree)
			 */
			float treeMoveHugeStep;

			/**
			 * Vektor smeri spreminjanja položaja.
			 */
			drevo::Tocka hitrostiPremikaRelative;
			/**
			 * Vektor smeri spreminjanja zasuka
			 */
			drevo::Tocka hitrostiZasuka;
		};
	}
}

#endif
