/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "oglwidget.h"

#include "render/ecosystemrender.h"
#include "render/camera.h"
#include "simulation/statistics/entitiesstatistics.h"

#include <QMessageBox>
#include <QImage>
#include <QString>
#include <QGLContext>
#include <QTime>
#include <QMouseEvent>
#include <QPoint>

namespace gui {

	OglWidget::OglWidget( QWidget *parent, ecosystem::render::EcosystemRender *currentModel )
	: QGLWidget( parent ), currentModel(currentModel) {
		setAttribute( Qt::WA_NoBackground );
		mouseSensivity=10.0f;
	}

	OglWidget::~OglWidget() {
		///Model bo izbrisal MainWindow:
		/// delete currentModel;
	}

	void OglWidget::initializeGL() {
		GLfloat global_ambient[] = {1.0,1.0,1.0,1.0};

		glShadeModel(GL_SMOOTH);
		///neželeno: GL_POLYGON_SMOOTH trikotnike obrobi z aliasom!
		///glEnable(GL_POLYGON_SMOOTH);

		glColorMaterial(GL_FRONT_AND_BACK,GL_DIFFUSE);
		glEnable(GL_COLOR_MATERIAL);

		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

		//glEnable(GL_NORMALIZE);
		glEnable(GL_DEPTH_TEST);

		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);
		glEnable(GL_LIGHT0+1);
		glEnable(GL_LIGHT0+2);

		///ovetlitev: 2 luči
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

		glLightModelfv(GL_LIGHT_MODEL_AMBIENT,global_ambient);
		glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,GL_TRUE);
		///osvetlitev iz obeh strani: onemogočeno
		/// glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
		glColorMaterial(GL_FRONT_AND_BACK,GL_DIFFUSE);
		glEnable(GL_COLOR_MATERIAL);

		glEnable(GL_TEXTURE_2D);
		glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

		checkExtensions();

		if (!currentModel) {
			///context pošljem zaradi getprocaddress, this pa zaradi bindTexture in deleteTexture
			///-- ostalega sodelovanja ni.
			currentModel = new ecosystem::render::EcosystemRender(context(), this);
			emit newModelCreated(currentModel);
		} else {
			currentModel->initAllTextures(this);
		}
		connect(currentModel, SIGNAL(modelDataChanged()),
				this, SLOT(updateGL()));
		emit newGlContextInitialized(this);
	}

	void OglWidget::checkExtensions() {
		const GLubyte *extensions = glGetString(GL_EXTENSIONS);
		if (!extensions) {
			(void) QMessageBox::information(this, tr("OpenGL Extensions"),
 			tr("Could not query for supported OpenGL extensions, extensions will not be supported."), QMessageBox::Ok);
			return;
		}
		QString ext((char*)extensions);
		QStringList extensionsList = ext.split(" ");
		supportMultitexture = extensionsList.contains("GL_ARB_multitexture");
		supportCompiledVertexArrays = extensionsList.contains("GL_EXT_compiled_vertex_array");
		static bool firstWidget=true;
		if (firstWidget) {
			firstWidget=false;
			if (!supportMultitexture) {
				(void) QMessageBox::information(this, tr("OpenGL Extensions"),
				tr("Compiled vertex array is not supported with this driver, using failover."), QMessageBox::Ok);
				return;
			}
			if (!supportMultitexture) {
				(void) QMessageBox::information(this, tr("OpenGL Extensions"),
				tr("Multitexturing is not supported with this driver, using failover."), QMessageBox::Ok);
				return;
			}
		}
	}

	void OglWidget::paintGL() {
		currentModel->render();
	}

	void OglWidget::resizeGL( int width, int height ) {
		glViewport(0,0,(GLsizei)width,(GLsizei)height);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(90.0,1.0,1.0,30000.0);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	}

	/**
	 * Zamenja model, trenutnega ne pobriše.
	 * @param model
	 */
	void OglWidget::switchToModel(ecosystem::render::EcosystemRender *model) {
		currentModel = model;
	}

	void OglWidget::mousePressEvent ( QMouseEvent * event ) {
		emit grabCamera();
		grabMouse();
		if (event->buttons()&Qt::LeftButton) {
			t_z = event->pos().y();
			actionMove = true;
		} else if (event->buttons()&Qt::RightButton) {
			s_x = event->pos().x();
			s_y = event->pos().y();
			actionRotate = true;
		}
	}

	void OglWidget::mouseReleaseEvent ( QMouseEvent * ) {
		actionRotate = false;
		actionMove = false;

		releaseMouse();
	}

	void OglWidget::mouseMoveEvent ( QMouseEvent * event) {
		int x =  event->pos().x();
		int y =  event->pos().y();

		if (actionMove) {
			currentModel->getCurrentCamera()->z_pomik += (y - t_z)/mouseSensivity;

			t_z = y;
			updateGL();
		}
		if (actionRotate) {
			currentModel->getCurrentCamera()->y_zasuk += (x - s_x)/mouseSensivity;
			currentModel->getCurrentCamera()->x_zasuk += (y - s_y)/mouseSensivity;

			s_x = x;
			s_y = y;
			updateGL();
		}
	}
}
