/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/


#include "skytextures.h"
#include "render/ecosystemrender.h"
#include <QFileDialog>

using namespace gui;

SkyTextures::SkyTextures( QWidget* parent, ecosystem::render::EcosystemRender *model )
		: QDialog( parent ), model( model ) {
	setupUi( this );
	QString filenameSky1;
	QString filenameSky2;
	QString filenameSky3;
	QString filenameSky4;
	QString filenameSky5;
	QString filenameSky6;

	model->getSkyTextureFileNames( filenameSky1,
	                               filenameSky2,
	                               filenameSky3,
	                               filenameSky4,
	                               filenameSky5,
	                               filenameSky6 );
	north->setText( filenameSky1 );
	south->setText( filenameSky2 );
	east->setText( filenameSky3 );
	west->setText( filenameSky4 );
	up->setText( filenameSky5 );
	down->setText( filenameSky6 );

	connect(pbNorth, SIGNAL(clicked()), this, SLOT(pbNorthClicked()));
	connect(pbSouth, SIGNAL(clicked()), this, SLOT(pbSouthClicked()));
	connect(pbEast, SIGNAL(clicked()), this, SLOT(pbEastClicked()));
	connect(pbWest, SIGNAL(clicked()), this, SLOT(pbWestClicked()));
	connect(pbUp, SIGNAL(clicked()), this, SLOT(pbUpClicked()));
	connect(pbDown, SIGNAL(clicked()), this, SLOT(pbDownClicked()));
}

SkyTextures::~SkyTextures() {}

/*$SPECIALIZATION$*/
void SkyTextures::reject() {
	QDialog::reject();
}

void SkyTextures::accept() {
	QString filenameSky1 = north->text();
	QString filenameSky2 = south->text();
	QString filenameSky3 = east->text();
	QString filenameSky4 = west->text();
	QString filenameSky5 = up->text();
	QString filenameSky6 = down->text();

	model->reloadSkyTextures( filenameSky1,
	                          filenameSky2,
	                          filenameSky3,
	                          filenameSky4,
	                          filenameSky5,
	                          filenameSky6 );

	QDialog::accept();
}

QString SkyTextures::getTexFileName(const QString& oldName) {
	QString s = QFileDialog::getOpenFileName(
			this,
	tr( "Choose a file to open" ), oldName,
	tr( "Images (*.png *.xpm *.jpg *.bmp)" ) );

	return s;
}

void SkyTextures::pbNorthClicked() {
	QString s = getTexFileName(north->text());
	if (s==0) return;
	north->setText( s );
}

void SkyTextures::pbSouthClicked() {
	QString s = getTexFileName(south->text());
	if (s==0) return;
	south->setText( s );
}

void SkyTextures::pbEastClicked() {
	QString s = getTexFileName(east->text());
	if (s==0) return;
	east->setText( s );
}

void SkyTextures::pbWestClicked() {
	QString s = getTexFileName(west->text());
	if (s==0) return;
	west->setText( s );
}

void SkyTextures::pbUpClicked() {
	QString s = getTexFileName(up->text());
	if (s==0) return;
	up->setText( s );
}

void SkyTextures::pbDownClicked() {
	QString s = getTexFileName(down->text());
	if (s==0) return;
	down->setText( s );
}
