/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef GUI_SKYTEXTURES_H
#define GUI_SKYTEXTURES_H

#include "ui_skytextures.h"

namespace ecosystem {
namespace render {
	class EcosystemRender;
}
}
namespace gui {
	/**
	 * Služi nalaganju novih tekstur za obzorje.
	 */
	class SkyTextures : public QDialog, protected Ui::SkyTextures {
		Q_OBJECT

	public:
		/**
		 * Kreira dialog, 
		 * @param parent vrhnje okno za modalnost
		 * @param model model, v katerem postavljamo teksture obzorja
		 */
		SkyTextures( QWidget* parent, ecosystem::render::EcosystemRender *model);
		/**
		 * Pobriše objekt
		 */
		~SkyTextures();

	protected slots:
		/**
		 * Uporabnik zavrne dialog.
		 */
		virtual void reject();
		/**
		 * Uporabnik sprejme dialog, shranimo imena datotek za obzorje,
		 * znova naložimo vse teksture v model (v Sky).
		 */
		virtual void accept();
		/**
		 * Izberemo datoteko za teksturo na severu.
		 */
		virtual void pbNorthClicked();
		/**
		 * Izberemo datoteko za teksturo na jugu.
		 */
		virtual void pbSouthClicked();
		/**
		 * Izberemo datoteko za teksturo na vzhodu.
		 */
		virtual void pbEastClicked();
		/**
		 * Izberemo datoteko za teksturo na zahodu.
		 */
		virtual void pbWestClicked();
		/**
		 * Izberemo datoteko za teksturo zgoraj.
		 */
		virtual void pbUpClicked();
		/**
		 * Izberemo datoteko za teksturo na spodaj.
		 */
		virtual void pbDownClicked();

	private:
		/**
		 * Iz dialoga vrne ime datoteke za odpiranje teksture.
		 * @return ime datoteke za teksturo
		 * @param oldName staro ime datoteke
		 */
		QString getTexFileName(const QString& oldName);
		/**
		 * Uporabljan model za določanje novih tekstur
		 */
		ecosystem::render::EcosystemRender *model;
	};
}
#endif

