/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "timekeeper.h"
#include <QTimer>
#include "drevo.h"
#include "render/ecosystemrender.h"

namespace gui {

	namespace actionhandling {

		TimeKeeper::TimeKeeper(QObject *parent, ecosystem::render::EcosystemRender * model)
			: QObject(parent), model(model) {
			treeTimer=0;
			cameraTimer=0;
			simmulationTimer=0;

			animationExportingMode=false;
			animationFrameNumber=0;
		}

		TimeKeeper::~TimeKeeper() {}

		///utils:

		void TimeKeeper::setCurrentModel(ecosystem::render::EcosystemRender * model) {
			this->model=model;
		}

		void TimeKeeper::toggleAnimationExportingMode() {
			animationExportingMode=!animationExportingMode;
			///resetiramo counter
			if (!animationExportingMode) {
				animationFrameNumber=0;
				emit sequenceExportStopped();
				return;
			}
		}

		void TimeKeeper::forceUpdateGl() {
			if (animationExportingMode)
				emit grabGeometry(animationFrameNumber++);

			emit updateOnlyGL();

			if (animationExportingMode)
				emit grabFrameBuffer(animationFrameNumber++);
		}

		/// simulacija:

		void TimeKeeper::toggleRunSimulation() {
			if (simmulationTimer) {
				delete simmulationTimer;
				simmulationTimer=0;
				return;
			}

			simmulationTimer = new QTimer( this );
			connect( simmulationTimer, SIGNAL( timeout() ), this, SLOT( simmulationTimerUpdate() ) );
			simmulationTimer->setSingleShot( false );
			simmulationTimer->start( 0 ); // sproži takoj, ko pride skozi event message loop
		}

		void TimeKeeper::simmulationTimerUpdate() {
			emit updateSimulationAndGL();
			if (animationExportingMode)
				emit grabFrameBuffer(animationFrameNumber++);
		}

		/// tree animacija:

		void TimeKeeper::startTreeAnimation() {
			if (treeTimer) return;

			treeTimer = new QTimer( this );
			connect( treeTimer, SIGNAL( timeout() ), this, SLOT( treeTimerUpdate() ) );
			treeTimer->setSingleShot( false );
			treeTimer->start( 20 ); // sproži takoj, ko pride skozi event message loop
		}

		void TimeKeeper::stopTreeAnimation() {
			if (!treeTimer) return;

			delete treeTimer;
			treeTimer=0;
			return;
		}

		void TimeKeeper::treeTimerUpdate() {
			emit animateTree();
			forceUpdateGl();
		}

		/// kamera animacija:

		void TimeKeeper::startCameraAnimation() {
			if (cameraTimer) return;

			cameraTimer = new QTimer( this );
			connect( cameraTimer, SIGNAL( timeout() ), this, SLOT( cameraTimerUpdate() ) );
			cameraTimer->setSingleShot( false );
			cameraTimer->start( 10 ); // sproži takoj, ko pride skozi event message loop
		}

		void TimeKeeper::stopCameraAnimation() {
			if (!cameraTimer) return;

			delete cameraTimer;
			cameraTimer=0;
			return;
		}

		void TimeKeeper::cameraTimerUpdate() {
			emit animateCamera();
			forceUpdateGl();
		}
	}

}
