/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GUI_ACTIONHANDLINGTIMEKEEPER_H
#define GUI_ACTIONHANDLINGTIMEKEEPER_H

#include <QObject>
class QTimer;
namespace ecosystem {
	namespace render{
		class EcosystemRender;
	}
}

namespace gui {

	namespace actionhandling {

		/**
		 * Vzdržuje timerje in ob njih proži ustrezne akcije.
		 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
		*/
		class TimeKeeper : public QObject {
			Q_OBJECT
		public:
			/**
			 * Kreira objekt in shrani podane kazalce
			 * @param parent vrhnje okno
			 * @param model urejan model
			 */
			TimeKeeper(QObject *parent, ecosystem::render::EcosystemRender * model);

			/**
			 * Izbriše objekt.
			 */
			~TimeKeeper();

		signals:
			/**
			 * Zahteva samo osvežitev GL, brez koraka simulacije
			 */
			void updateOnlyGL();
			/**
			 * Zahteva izvedbo koraka simulacije in osvežitev GL po njem
			 */
			void updateSimulationAndGL();
			/**
			 * Zahteva zajem rastra okna
			 * @param value št. okvirja
			 */
			void grabFrameBuffer(int);
			/**
			 * Zahteva zajem geometrije drevesa
			 * @param  value št. okvirja
			 */
			void grabGeometry(int);
			/**
			 * Zahteva zaustavitev sequence export dreves.
			 */
			void sequenceExportStopped();
			/**
			 * Zahteva animacijo drevesa
			 */
			void animateTree();
			/**
			 * Zahteva animacijo kamere
			 */
			void animateCamera();

		public slots:
			/**
			 * Shrani kazalec za trenutni urejan model
			 * @param model kazalec na novi model
			 */
			void setCurrentModel(ecosystem::render::EcosystemRender * model);
			/**
			 * Preklopi način shranjevanja animacije
			 */
			void toggleAnimationExportingMode();
			/**
			 * Osveži GL
			 */
			void forceUpdateGl();
			/**
			 * Preklopi poganjanje simulacije
			 */
			void toggleRunSimulation();

			/**
			 * Požene timer za animacijo drevesa
			 */
			void startTreeAnimation();
			/**
			 * Ustavi timer za animacijo drevesa
			 */
			void stopTreeAnimation();

			/**
			 * Požene timer za animacijo kamere
			 */
			void startCameraAnimation();
			/**
			 * Ustavi timer za animacijo kamere
			 */
			void stopCameraAnimation();


		protected slots:
			/**
			 * Timer ob signalu za animacijo drevesa
			 */
			void treeTimerUpdate();
			/**
			 * Timer ob signalu za animacijo kamere
			 */
			void cameraTimerUpdate();
			/**
			 * Timer ob signalu za poganjanje simulacije
			 */
			void simmulationTimerUpdate();

		///stanja
		private:
			/** Model aplikacije -- komponenta za renderiranje ekosistema */
			ecosystem::render::EcosystemRender * model;

			/**
			 * Način shranjevanja animacije v datoteke
			 */
			bool animationExportingMode;
			/**
			 * Zaporedna številka okvirja animacije
			 */
			int animationFrameNumber;

			/** Animacija drevesa: orbit, grow, sway */
			QTimer *treeTimer;

			/** Animacija premikanja kamere */
			QTimer *cameraTimer;

			/** Timer za simulacijo */
			QTimer *simmulationTimer;
		};
	}
}

#endif
