/***************************************************************************
 *   Copyright (C) 2006 by Ales Zamuda                                     *
 *   ales.zamuda@uni-mb.si                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GUI_ACTIONHANDLINGVIEWHANDLER_H
#define GUI_ACTIONHANDLINGVIEWHANDLER_H

#include <QObject>
namespace ecosystem {
	namespace render{
		class EcosystemRender;
	}
}

namespace gui {

	namespace actionhandling {

		/**
		 * Obravnava spreminjanje izgleda drevesa.
		 * @author Ales Zamuda <ales.zamuda@uni-mb.si>
		*/
		class ViewHandler : public QObject {
			Q_OBJECT
		public:
			/**
			 * Ustvari objekt, shrani model.
			 * @param parent vrhnje okno
			 * @param model urejanj model
			 * @return
			 */
			ViewHandler(QObject *parent, ecosystem::render::EcosystemRender * model);
			/**
			 * Odstrani objekt
			 * @return
			 */
			~ViewHandler();

		signals:
			/**
			 * Zahteva osvežitev izrisa
			 */
			void forceRepaint();

		public slots:
			/**
			 * Preklopi prikaz ekosistema ali enega samega drevesa
			 */
			void actionShow_ecosystem();

			/**
			 * Preklopi prikaz listov na drevesu
			 */
			void actionDisplay_leaves();
			/**
			 * Preklopi barvanje drevesa vzdolž vej
			 */
			void actionColour_uppon();
			/**
			 * Preklopi žično upodabljanje
			 */
			void actionRender_wireframe();
			/**
			 * Preklopi upodabljanje preseka vej drevesa s 4 ali 15 kotniki
			 */
			void actionHigh_quality_rendering_more_polygons();
			/**
			 * Preklopi uporabo osvetljevanja
			 */
			void actionLighting_enabled();
			/**
			 * Preklopi prikaz barve neba ozadja.
			 */
			void actionDisplay_background_sky();
			/**
			 * Preklopi prikaz FPS.
			 */
			void actionShow_frame_rate();

			/**
			 * Preklopi prikaz dreves v ekosistemu
			 */
			void actionDo_not_draw_trees_ecosystem_mode();
			/**
			 * Preklopi osveževanje prikaza v celoti, v načinu ekosistema
			 */
			void actionDo_not_refresh_display_only_run_simulation_ecosystem_mode();
			/**
			 * Preklopi prikaz ekoloških radijev
			 */
			void actionShow_ecologic_neighborhood_ecosystem_mode();
			/**
			 * Preklopi prikaz normal terena
			 */
			void actionDisplay_terrain_normals();
			/**
			 * Preklopi prikaz strmin
			 */
			void actionDisplay_terrain_inclination();
			/**
			 * Preklopi prikaz vlažnosti
			 */
			void actionDisplay_terrain_moisture();
			/**
			 * Preklopi prikaz osončenosti
			 */
			void actionDisplay_terrain_sunniness();
			/**
			 * Preklopi prikaz vetrovnosti
			 */
			void actionDisplay_terrain_windiness();
			/**
			 * Preklopi prikaz oznak terena
			 */
			void actionDisplay_terrain_land_markings_S_N_E_W();
			/**
			 * Preklopi prikaz vzorcev sonca
			 */
			void actionDisplay_sun();

		private:
			/** Kazalec na upravljan model */
			ecosystem::render::EcosystemRender * model;
		};
	}
}

#endif
