/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "widgetgraph.h"
#include "hoverpoints.h"
#include <QtGui>

using namespace gui;

WidgetGraph::WidgetGraph( QWidget *parent )
		: QWidget( parent ), m_gradient( QLinearGradient( 0, 0, 0, 0 ) ) {
	setupUi( this );

	setAttribute( Qt::WA_NoBackground );

	m_hoverPoints = new HoverPoints( this, HoverPoints::CircleShape );

	m_hoverPoints = new HoverPoints( this, HoverPoints::CircleShape );
	m_hoverPoints->setConnectionType(HoverPoints::LineConnection);
// 	m_hoverPoints->setPoints( points );
/*	m_hoverPoints->setPointLock( 0, HoverPoints::LockToLeft );
	m_hoverPoints->setPointLock( 1, HoverPoints::LockToRight );*/
	m_hoverPoints->setSortType( HoverPoints::XSort );
	m_hoverPoints->setConnectionPen( QPen(QColor( 255, 0, 0 ), 2));

	setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Fixed );

	connect( m_hoverPoints, SIGNAL( pointsChanged( const QPolygonF & ) ), this, SIGNAL( valuesChanged() ) );
}

QPolygonF WidgetGraph::points() const {
	return m_hoverPoints->points();
}

uint WidgetGraph::valueAt( int x ) {
	generateImage();

	QPolygonF pts = m_hoverPoints->points();
	for ( int i = 1; i < pts.size(); ++i ) {
		if ( pts.at( i - 1 ).x() <= x && pts.at( i ).x() >= x ) {
			QLineF l( pts.at( i - 1 ), pts.at( i ) );
			l.setLength( l.length() * ( ( x - l.x1() ) / l.dx() ) );
			return m_image.pixel( qRound( qMin( l.x2(), ( qreal( m_image.width() - 1 ) ) ) ),
			                      qRound( qMin( l.y2(), qreal( m_image.height() - 1 ) ) ) );
		}
	}
	return 0;
}

void WidgetGraph::setGradientStops( const QGradientStops &/*stops*/ ) {
	/*	if ( m_image_type == ARGBShade ) {
			m_alpha_gradient = QLinearGradient( 0, 0, width(), 0 );

			for ( int i = 0; i < stops.size(); ++i ) {
				QColor c = stops.at( i ).second;
				m_alpha_gradient.setColorAt( stops.at( i ).first, QColor( c.red(), c.green(), c.blue() ) );
			}

			m_image = QImage();
			generateShade();
			update();
	}*/
}

void WidgetGraph::paintEvent( QPaintEvent * ) {
	generateImage();

	QPainter p( this );
	p.drawImage( 0, 0, m_image );

	p.setPen( QColor( 146, 146, 146 ) );
	p.drawRect( 0, 0, width() - 1, height() - 1 );

	p.setPen( QColor( 170, 170, 170 ) );
	int nrLines=10;
	for (int i = 0; i < nrLines; i++) {
		/// po x
		p.drawLine( i*width()/nrLines, 0, i*width()/nrLines, height() - 1 );
		/// po y
		p.drawLine( 0, i*height()/nrLines, width() - 1, i*height()/nrLines );
	}
}

void WidgetGraph::generateImage() {
	if ( m_image.isNull() || m_image.size() != size() ) {
		m_image = QImage( size(), QImage::Format_RGB32 );
		QLinearGradient shade( 0, 0, 0, height() );
		//shade.setColorAt( 1, QColor( 127, 127, 127 ) );
		shade.setColorAt( 0, QColor( 189, 189, 189 ) );

		QPainter p( &m_image );
		p.fillRect( rect(), shade );
	}
}
