/***************************************************************************
*   Copyright (C) 2006 by Ales Zamuda                                     *
*   ales.zamuda@uni-mb.si                                                 *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef WIDGETGRAPH_H
#define WIDGETGRAPH_H

#include "ui_widget_graph.h"

class QPolygonF;

namespace gui {
	class HoverPoints;

	/**
	 * Kontrola za prikaz grafov, ki jih riše HoverPoints.
	 *
	 * @author: Aleš Zamuda, (Derived from Qt4 demo class ShadeWidget).
	 */
	class WidgetGraph: public QWidget, private Ui::WidgetGraph {
		Q_OBJECT
	public:
		/**
		 * Ustvari novo kontrolo.
		 */
		WidgetGraph( QWidget *parent = 0 );
		/**
		* Nastavi pribitke točke
		* @return pribitki točke
		*/
		void setGradientStops( const QGradientStops &stops );
		/**
		 * Sprožen pred risanjem grafa, nariše obkrožajoči kvader in črte.
		 */
		void paintEvent( QPaintEvent *e );

		/**
		 * Privzeta velikost kontrole.
		 */
		QSize sizeHint() const {
			return QSize( 150, 40 );
		}
		/**
		 * Vrne točke na hoverpoints.
		 */
		QPolygonF points() const;

		/**
		 * Vrne HoverPoints: graf.
		 */
		HoverPoints *hoverPoints() const {
			return m_hoverPoints;
		}

		/**
		 * Vrne vrednost funkcije (grafa) na x.
		 * @param x x, kjer naj vrnemo vrednost
		 */
		uint valueAt( int x );

	signals:
		/**
		 * Sprožen, ko se spremenijo vrednosti na grafu.
		 */
		void valuesChanged();

	private:
		/**
		 * Generira sliko, po kateri se riše (ozadje).
		 */
		void generateImage();

		/**
		 * Slika za ozadje, ta se nariše prvo.
		 */
		QImage m_image;
		/**
		 * Točke grafa, ki jih je mogoče interaktivno
		 * spreminjati, podajo pa tudi, katera je označena.
		 */
		HoverPoints *m_hoverPoints;
		/**
		 * Gradienti za risanje ozadja.
		 */
		QLinearGradient m_gradient;
	};
}
#endif
